@@ -36,8 +36,8 @@ class H5_DLLCPP DataType : public H5Object {
 
 	// Commits a transient datatype to a file; this datatype becomes 
 	// a named datatype which can be accessed from the location.
-	void commit( H5Object& loc, const string& name ) const;
-	void commit( H5Object& loc, const char* name ) const;
+	void commit( CommonFG& loc, const string& name ) const;
+	void commit( CommonFG& loc, const char* name ) const;
 
 	// Determines whether this datatype is a named datatype or 
 	// a transient datatype. 
@@ -50,12 +50,6 @@ class H5_DLLCPP DataType : public H5Object {
 	// Converts data from between specified datatypes. 
 	void convert( const DataType& dest, hsize_t nelmts, void *buf, void *background, PropList& plist ) const;
 
-	// Sets the overflow handler to a specified function. 
-	void setOverflow(H5T_overflow_t func) const;
-
-	// Returns a pointer to the current global overflow function. 
-	H5T_overflow_t getOverflow(void) const;
-
 	// Assignment operator
 	DataType& operator=( const DataType& rhs );
 
@@ -65,6 +59,12 @@ class H5_DLLCPP DataType : public H5Object {
 	// Locks a datatype. 
 	void lock() const;
 
+	// Returns a pointer to the current global overflow function. 
+	H5T_overflow_t getOverflow(void) const;
+
+	// Sets the overflow handler to a specified function. 
+	void setOverflow(H5T_overflow_t func) const;
+
 	// Returns the size of a datatype. 
 	size_t getSize() const;
 
@@ -87,12 +87,24 @@ class H5_DLLCPP DataType : public H5Object {
 	// Gets the tag associated with an opaque datatype. 
 	string getTag() const;
 
-	// Creates a new variable-length datatype - not implemented yet
-	// Will be moved into a subclass when completed
-	//DataType vlenCreate( const DataType& base_type );
+	// Checks whether this datatype contains (or is) a certain type class.
+	bool detectClass(H5T_class_t cls) const;
 
-	// Used by the API to appropriately close a datatype
-        void p_close() const;
+	// Checks whether this datatype is a variable-length string.
+	bool isVariableStr() const;
+
+	// Creates a reference to a named Hdf5 object in this object.
+	void* Reference(const char* name) const;
+
+	// Creates a reference to a named Hdf5 object or to a dataset region
+	// in this object.
+	void* Reference(const char* name, DataSpace& dataspace, H5R_type_t ref_type = H5R_DATASET_REGION) const;
+
+	// Retrieves the type of object that an object reference points to.
+	H5G_obj_t getObjType(void *ref, H5R_type_t ref_type) const;
+
+	// Retrieves a dataspace with the region pointed to selected.
+	DataSpace getRegion(void *ref, H5R_type_t ref_type = H5R_DATASET_REGION) const;
 
 	// Creates a copy of an existing DataType using its id 
 	DataType( const hid_t type_id, bool predtype = false );
@@ -100,6 +112,12 @@ class H5_DLLCPP DataType : public H5Object {
 	// Default constructor
 	DataType();
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+	// Used by the API to appropriately close a datatype
+        void p_close() const;
+#endif // DOXYGEN_SHOULD_SKIP_THIS
+
+	// Destructor: properly terminates access to this datatype.
 	virtual ~DataType();
 
    protected:
