@@ -15,11 +15,12 @@
 #include <string>
 
 #include "H5Include.h"
-#include "H5RefCounter.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
 #include "H5Object.h"
+#include "H5DcreatProp.h"
+#include "H5CommonFG.h"
 #include "H5DataType.h"
 #include "H5AtomType.h"
 #include "H5Library.h"
@@ -29,25 +30,36 @@
 namespace H5 {
 #endif
 
-// Default constructor 
-PredType::PredType() : AtomType() {}
-
-// creates predefined datatype, so set DataType::is_predtype to true by default
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+//--------------------------------------------------------------------------
+// Function:	PredType overloaded constructor
+///\brief	Creates a PredType object using the id of an existing 
+///		predefined datatype.
+///\param	predtype_id - IN: Id of a predefined datatype
+// Description
+// 		This constructor creates a predefined datatype, so it sets 
+// 		DataType::is_predtype to true.
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
 PredType::PredType( const hid_t predtype_id ) : AtomType( predtype_id )
 { 
    is_predtype = true; 
 }
 
-// Copy constructor: makes a copy of this PredType object.
-PredType::PredType( const PredType& original ) : AtomType( original ) {}
+//--------------------------------------------------------------------------
+// Function:	PredType default constructor
+///\brief	Default constructor: Creates a stub predefined datatype
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+PredType::PredType() : AtomType() {}
 
-// Makes a copy of the predefined type and stores the new
-// id in the left hand side object.  
-PredType& PredType::operator=( const PredType& rhs )
-{
-   copy(rhs);
-   return(*this);
-}
+//--------------------------------------------------------------------------
+// Function:	PredType copy constructor
+///\brief	Copy constructor: makes a copy of the original PredType object.
+///\param	original - IN: PredType instance to copy
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+PredType::PredType( const PredType& original ) : AtomType( original ) {}
 
 const PredType PredType::NotAtexit;	// only for atexit/global dest. problem
 
@@ -196,7 +208,31 @@ const PredType PredType::NATIVE_INT_LEAST64( E_NATIVE_INT_LEAST64 );
 const PredType PredType::NATIVE_UINT_LEAST64( E_NATIVE_UINT_LEAST64 );
 const PredType PredType::NATIVE_INT_FAST64( E_NATIVE_INT_FAST64 );
 const PredType PredType::NATIVE_UINT_FAST64( E_NATIVE_UINT_FAST64 );
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
+//--------------------------------------------------------------------------
+// Function:	PredType::operator=
+///\brief	Assignment operator.
+///\param	rhs - IN: Reference to the predefined datatype
+///\return	Reference to PredType instance
+///\exception	H5::DataTypeIException
+// Description
+//		Makes a copy of the type on the right hand side and stores
+//		the new id in the left hand side object.
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+PredType& PredType::operator=( const PredType& rhs )
+{
+   copy(rhs);
+   return(*this);
+}
+
+//--------------------------------------------------------------------------
+// Function:	PredType::getId
+///\brief	Returns the HDF5 predefined type id.
+///\return	HDF5 predefined type id or INVALID
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
 hid_t PredType::getId() const
 {
     switch( id ) {
@@ -468,6 +504,7 @@ hid_t PredType::getId() const
     }   // end switch
 }   // end of getId()
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 // These dummy functions do not inherit from DataType - they'll
 // throw an DataTypeIException if invoked.
 void PredType::commit( H5Object& loc, const char* name )
@@ -485,8 +522,14 @@ bool PredType::committed()
    throw DataTypeIException("PredType::committed", "Error: Attempting to check for commit status on a predefined datatype." );
    return (0);
 }  
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 // Default destructor
+//--------------------------------------------------------------------------
+// Function:	PredType destructor
+///\brief	Noop destructor.
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
 PredType::~PredType() {}
 
 #ifndef H5_NO_NAMESPACE
