@@ -21,8 +21,6 @@
      These routines are in the test directory of the C library:
         h5_reset() -- in h5test.c, resets the library by closing it
         h5_fileaccess() -- in h5test.c, returns a file access template
-        h5_fixname() -- in h5test.c, create a file name from a file base name
-        h5_cleanup() -- in h5test.c, cleanup temporary test files
 
  ***************************************************************************/
 
@@ -32,9 +30,8 @@
 #include <iostream>
 #endif
 
-#include "H5Cpp.h"
-#include "h5test.h"
 #include "testhdf5.h"
+#include "H5Cpp.h"
 
 #ifndef H5_NO_NAMESPACE
 using namespace H5;
@@ -42,23 +39,20 @@ using namespace H5;
 
 #include "h5cpputil.h"
 
-const char *FILENAME[] = {
-    "dataset",
-    NULL
-};
+const string	FILE1("dataset.h5");
+const string	DSET_DEFAULT_NAME("default");
+const string	DSET_CHUNKED_NAME("chunked");
+const string	DSET_SIMPLE_IO_NAME("simple_io");
+const string	DSET_TCONV_NAME	("tconv");
+const string	DSET_COMPRESS_NAME("compressed");
+const string	DSET_BOGUS_NAME	("bogus");
 
-#define DSET_DEFAULT_NAME	"default"
-#define DSET_CHUNKED_NAME	"chunked"
-#define DSET_SIMPLE_IO_NAME	"simple_io"
-#define DSET_TCONV_NAME		"tconv"
-#define DSET_COMPRESS_NAME	"compressed"
-#define DSET_BOGUS_NAME		"bogus"
+const int H5Z_FILTER_BOGUS = 305;
 
-#define H5Z_FILTER_BOGUS		305
-
-/* Local prototypes for filter functions */
+// Local prototypes
 static size_t bogus(unsigned int flags, size_t cd_nelmts, 
     const unsigned int *cd_values, size_t nbytes, size_t *buf_size, void **buf);
+void cleanup_dsets(void);
 
 
 /*-------------------------------------------------------------------------
@@ -82,72 +76,64 @@ test_create( H5File& file)
 {
    TESTING("create, open, close");
 
+    // Setting this to NULL for cleaning up in failure situations
+    DataSet *dataset = NULL;
     try {
-	/* Create the data space */
+	// Create the data space
 	hsize_t     dims[2];
 	dims[0] = 256;
 	dims[1] = 512;
 	DataSpace space (2, dims, NULL);
 
-	/*
-	* Create a dataset using the default dataset creation properties.	
-	* We're not sure what they are, so we won't check.
-	*/
+	// Create a dataset using the default dataset creation properties.	
+	// We're not sure what they are, so we won't check.
 	DataSet *dataset = new DataSet (file.createDataSet 
 		(DSET_DEFAULT_NAME, PredType::NATIVE_DOUBLE, space));
 
-	/* Close the dataset */
+	// Close the dataset
 	delete dataset;
+	dataset = NULL;
 
-	/* Add a comment to the dataset */
+	// Add a comment to the dataset
 	file.setComment (DSET_DEFAULT_NAME, "This is a dataset");
 
-	/*
-	* Try creating a dataset that already exists.  This should fail since a
-	* dataset can only be created once.  If an exception is not thrown
-	* for this action by createDataSet, then display failure information
-	* and jump to label error: to return.
-	*/
+        // Try creating a dataset that already exists.  This should fail since a
+        // dataset can only be created once.  If an exception is not thrown
+        // for this action by createDataSet, then display failure information
+        // and throw an exception.
 	try {
 	    dataset = new DataSet (file.createDataSet 
 			(DSET_DEFAULT_NAME, PredType::NATIVE_DOUBLE, space));
 	    // continuation here, that means no exception has been thrown
-	    H5_FAILED();
-	    cout << "    Library allowed overwrite of existing dataset." << endl;
-	    goto error;
+            throw InvalidActionException("H5File::createDataSet", "Library allowed overwrite of existing dataset");
         }
         catch (FileIException E ) // catching invalid creating dataset
-        {
-	    // Exception is expected.  Do nothing here.
-        }
-	/*
-	* Open the dataset we created above and then close it.  This is how
-	* existing datasets are accessed.
-	*/
+        {} // do nothing, exception expected
+
+	// Open the dataset we created above and then close it.  This is how
+	// existing datasets are accessed.
 	dataset = new DataSet (file.openDataSet (DSET_DEFAULT_NAME));
+
+	// Close the dataset when accessing is completed
 	delete dataset;
-    
-	/*
-	* Try opening a non-existent dataset.  This should fail so if an
-	* exception is not thrown for this action by openDataSet, then 
-	* display failure information and jump to label error: to return.
-	*/
+
+        // This is another way to open an existing dataset for accessing.
+        DataSet another_dataset(file.openDataSet (DSET_DEFAULT_NAME));
+
+	// Try opening a non-existent dataset.  This should fail so if an
+	// exception is not thrown for this action by openDataSet, then
+	// display failure information and throw an exception.
 	try {
 	    dataset = new DataSet (file.openDataSet( "does_not_exist" ));
+
 	    // continuation here, that means no exception has been thrown
-	    H5_FAILED();
-	    cout << "    Opened a non-existent dataset." << endl;
-	    goto error;
+	    throw InvalidActionException("H5File::openDataSet", "Attempted to open a non-existent dataset");
 	}
 	catch (FileIException E ) // catching creating non-existent dataset
-	{
-	    // Exception is expected.  Do nothing here.
-	}
+	{} // do nothing, exception expected
 
-	/*
-	* Create a new dataset that uses chunked storage instead of the default
-	* layout.
-	*/
+	// Create a new dataset that uses chunked storage instead of the default
+	// layout.
 	DSetCreatPropList create_parms;
 	hsize_t     csize[2];
 	csize[0] = 5;
@@ -158,20 +144,33 @@ test_create( H5File& file)
 		(DSET_CHUNKED_NAME, PredType::NATIVE_DOUBLE, space, create_parms));
 	// Note: this one has no error message in C when failure occurs?
 
-	/*
-	* Close the chunked dataset.
-	*/
+	// clean up and return with success
 	delete dataset;
 
 	PASSED();
 	return 0;
    }	// outer most try block
 
-   // catch all dataset, file, space, plist exceptions
-   catch (Exception E) { goto error; }
-   
- error:
-    return -1;
+    catch (InvalidActionException E)
+    {
+	cerr << " FAILED" << endl;
+	cerr << "    <<<  " << E.getDetailMsg() << "  >>>" << endl << endl;
+
+	// clean up and return with failure
+	if (dataset != NULL)
+	    delete dataset;
+	return -1;
+    }
+    // catch all other exceptions
+    catch (Exception E)
+    {
+	issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__);
+
+	// clean up and return with failure
+	if (dataset != NULL)
+	    delete dataset;
+	return -1;
+    }
 }
 
 /*-------------------------------------------------------------------------
@@ -203,8 +202,8 @@ check_values (hsize_t i, hsize_t j, int apoint, int acheck)
     if (apoint != acheck)
     {
 	H5_FAILED();
-	cout << "    Read different values than written.\n" << endl;
-	cout << "    At index " << (unsigned long)i << "," << 
+	cerr << "    Read different values than written.\n" << endl;
+	cerr << "    At index " << (unsigned long)i << "," << 
    	(unsigned long)j << endl;
 	return -1;
     }
@@ -239,7 +238,7 @@ test_simple_io( H5File& file)
     int	check[100][200];
     int		i, j, n;
 
-    /* Initialize the dataset */
+    // Initialize the dataset
     for (i = n = 0; i < 100; i++)
     {
 	for (j = 0; j < 200; j++) {
@@ -250,32 +249,33 @@ test_simple_io( H5File& file)
     char* tconv_buf = new char [1000];
     try 
     {
-	/* Create the data space */
+	// Create the data space
 	hsize_t	dims[2];
 	dims[0] = 100;
 	dims[1] = 200;
 	DataSpace space (2, dims, NULL);
 
-	/* Create a small conversion buffer to test strip mining */
+	// Create a small conversion buffer to test strip mining
 	DSetMemXferPropList xfer;
 
 	xfer.setBuffer (1000, tconv_buf, NULL);
 
-	/* Create the dataset */
+	// Create the dataset
 	DataSet dataset (file.createDataSet (DSET_SIMPLE_IO_NAME, PredType::NATIVE_INT, space));
 
-	/* Write the data to the dataset */
+	// Write the data to the dataset
 	dataset.write ((void*) points, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Read the dataset back */
+	// Read the dataset back
 	dataset.read ((void*) check, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i = 0; i < 100; i++)
 	    for (j = 0; j < 200; j++)
 	    {
 		int status = check_values (i, j, points[i][j], check[i][j]);
-		if (status == -1) goto error;
+		if (status == -1)
+		    throw Exception("DataSet::read");
 	    }
 
 	delete [] tconv_buf;
@@ -284,13 +284,16 @@ test_simple_io( H5File& file)
     }  // end try
 
     // catch all dataset, space, plist exceptions
-    catch (Exception E) { goto error; }
-   
-  error:
-    // cleaning up
-    if (tconv_buf)
-	delete [] tconv_buf;
-    return -1;
+    catch (Exception E)
+    {
+        cerr << " FAILED" << endl;
+        cerr << "    <<<  " << E.getDetailMsg() << "  >>>" << endl << endl;
+
+        // clean up and return with failure
+        if (tconv_buf)
+            delete [] tconv_buf;
+        return -1;
+    }
 }
 
 /*-------------------------------------------------------------------------
@@ -321,7 +324,7 @@ test_tconv( H5File& file)
 
     TESTING("data type conversion");
     
-    /* Initialize the dataset */
+    // Initialize the dataset
     for (int i = 0; i < 1000000; i++) {
 	out[i*4+0] = 0x11;
 	out[i*4+1] = 0x22;
@@ -331,46 +334,49 @@ test_tconv( H5File& file)
 
     try
     {
-	/* Create the data space */
+	// Create the data space
 	hsize_t	dims[1];
 	dims[0] = 1000000;
 	DataSpace space (1, dims, NULL);
 
-	/* Create the data set */
+	// Create the data set
 	DataSet dataset (file.createDataSet (DSET_TCONV_NAME, PredType::STD_I32LE, space));
 
-	/* Write the data to the dataset */
+	// Write the data to the dataset
 	dataset.write ((void*) out, PredType::STD_I32LE);
 
-	/* Read data with byte order conversion */
+	// Read data with byte order conversion
 	dataset.read ((void*) in, PredType::STD_I32BE);
 
-	/* Check */
+	// Check
 	for (int i = 0; i < 1000000; i++) {
 	    if (in[4*i+0]!=out[4*i+3] ||
 		in[4*i+1]!=out[4*i+2] ||
 		in[4*i+2]!=out[4*i+1] ||
 		in[4*i+3]!=out[4*i+0]) 
 	    {
-		H5_FAILED();
-		cout << "    Read with byte order conversion failed." << endl;
-		goto error;
+		throw Exception("DataSet::read", "Read with byte order conversion failed");
 	    }
 	}
 
+	// clean up and return with success
 	delete [] out;
 	delete [] in;
-	cout << " PASSED" << endl;
+	cerr << " PASSED" << endl;
 	return 0;
     }  // end try
 
     // catch all dataset and space exceptions
-    catch (Exception E) { goto error; }
-   
-  error:
-	delete [] out;
-	delete [] in;
-	return -1;
+    catch (Exception E)
+    {
+        cerr << " FAILED" << endl;
+        cerr << "    <<<  " << E.getDetailMsg() << "  >>>" << endl << endl;
+
+        // clean up and return with failure
+        delete [] out;
+        delete [] in;
+        return -1;
+    }
 }
 
 /* This message derives from H5Z */
@@ -440,7 +446,7 @@ test_compression(H5File& file)
     int		check[100][200];
     hsize_t	i, j, n;
 
-    /* Initialize the dataset */
+    // Initialize the dataset
     for (i = n = 0; i < 100; i++)
     {
 	for (j = 0; j < 200; j++) {
@@ -448,34 +454,30 @@ test_compression(H5File& file)
 	}
     }
     char* tconv_buf = new char [1000];
-
+    DataSet* dataset = NULL;
     try 
     {
 	const hsize_t	size[2] = {100, 200};
-	/* Create the data space */
+	// Create the data space
 	DataSpace space1(2, size, NULL);
 
-	/*
-	* Create a small conversion buffer to test strip mining. We
-	* might as well test all we can!
-	*/
+	// Create a small conversion buffer to test strip mining. We
+	// might as well test all we can!
 	DSetMemXferPropList xfer;
 
 	xfer.setBuffer (1000, tconv_buf, NULL);
   
-	/* Use chunked storage with compression */
+	// Use chunked storage with compression
 	DSetCreatPropList dscreatplist;
 
 	const hsize_t	chunk_size[2] = {2, 25};
 	dscreatplist.setChunk (2, chunk_size);
 	dscreatplist.setDeflate (6);
   
-	DataSet* dataset;
-
 #ifdef H5_HAVE_FILTER_DEFLATE
     TESTING("compression (setup)");
     
-	/* Create the dataset */
+	// Create the dataset
 	dataset = new DataSet (file.createDataSet 
 	    (DSET_COMPRESS_NAME, PredType::NATIVE_INT, space1, dscreatplist));
   
@@ -493,10 +495,10 @@ test_compression(H5File& file)
 	    for (j=0; j<size[1]; j++) {
 		if (0!=check[i][j]) {
 		    H5_FAILED();
-		    cout << "    Read a non-zero value." << endl;
-		    cout << "    At index " << (unsigned long)i << "," << 
+		    cerr << "    Read a non-zero value." << endl;
+		    cerr << "    At index " << (unsigned long)i << "," << 
 		   (unsigned long)j << endl;
-		    goto error;
+		    throw Exception("test_compression", "Failed in uninitialized read");
 		}
 	    }
 	}
@@ -527,15 +529,16 @@ test_compression(H5File& file)
 	*/
 	TESTING("compression (read)");
 
-	/* Read the dataset back */
+	// Read the dataset back
 	dataset->read ((void*)check, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i = 0; i < size[0]; i++)
 	    for (j = 0; j < size[1]; j++)
 	    {
 		int status = check_values (i, j, points[i][j], check[i][j]);
-		if (status == -1) goto error;
+		if (status == -1)
+		    throw Exception("test_compression", "Failed in read");
 	    }
 
     PASSED();
@@ -558,15 +561,16 @@ test_compression(H5File& file)
 	}
 	dataset->write ((void*)points, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Read the dataset back and check it */
+	// Read the dataset back and check it
 	dataset->read ((void*)check, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i = 0; i < size[0]; i++)
 	    for (j = 0; j < size[1]; j++)
 	    {
 		int status = check_values (i, j, points[i][j], check[i][j]);
-		if (status == -1) goto error;
+		if (status == -1)
+		    throw Exception("test_compression", "Failed in modify");
 	    }
 
     PASSED();
@@ -579,17 +583,19 @@ test_compression(H5File& file)
 	*/
 	TESTING("compression (re-open)");
     
+	// close this dataset
 	delete dataset;
     
 	dataset = new DataSet (file.openDataSet (DSET_COMPRESS_NAME));
 	dataset->read ((void*)check, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i = 0; i < size[0]; i++)
 	    for (j = 0; j < size[1]; j++)
 	    {
 		int status = check_values (i, j, points[i][j], check[i][j]);
-		if (status == -1) goto error;
+		if (status == -1)
+		    throw Exception("test_compression", "Failed in re-open");
 	    }
 
     PASSED();
@@ -604,7 +610,7 @@ test_compression(H5File& file)
 	TESTING("compression (partial I/O)");
 
 	const hsize_t	hs_size[2] = {4, 50};
-	const hssize_t	hs_offset[2] = {7, 30};
+	const hsize_t	hs_offset[2] = {7, 30};
 	for (i = 0; i < hs_size[0]; i++) {
 	    for (j = 0; j < hs_size[1]; j++) {
 		points[hs_offset[0]+i][hs_offset[1]+j] = rand ();
@@ -614,31 +620,32 @@ test_compression(H5File& file)
 	dataset->write ((void*)points, PredType::NATIVE_INT, space1, space1, xfer);
 	dataset->read ((void*)check, PredType::NATIVE_INT, space1, space1, xfer);
     
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i=0; i<hs_size[0]; i++) {
 	for (j=0; j<hs_size[1]; j++) {
 	    if (points[hs_offset[0]+i][hs_offset[1]+j] !=
 		check[hs_offset[0]+i][hs_offset[1]+j]) {
 		H5_FAILED();
-		cout << "    Read different values than written.\n" << endl;
-		cout << "    At index " << (unsigned long)(hs_offset[0]+i) << 
+		cerr << "    Read different values than written.\n" << endl;
+		cerr << "    At index " << (unsigned long)(hs_offset[0]+i) << 
 		   "," << (unsigned long)(hs_offset[1]+j) << endl;
 		
-		cout << "    At original: " << (int)points[hs_offset[0]+i][hs_offset[1]+j] << endl;
-		cout << "    At returned: " << (int)check[hs_offset[0]+i][hs_offset[1]+j] << endl;
-		goto error;
+		cerr << "    At original: " << (int)points[hs_offset[0]+i][hs_offset[1]+j] << endl;
+		cerr << "    At returned: " << (int)check[hs_offset[0]+i][hs_offset[1]+j] << endl;
+		throw Exception("test_compression", "Failed in partial I/O");
 	    }
-	}
-	}
+	} // for j
+	} // for i
 
 	delete dataset;
+	dataset = NULL;
 
     PASSED();
 
 #else
     TESTING("deflate filter");
     SKIPPED();
-    cout << not_supported << endl;
+    cerr << not_supported << endl;
 #endif
 
 	/*----------------------------------------------------------------------
@@ -649,11 +656,14 @@ test_compression(H5File& file)
 	TESTING("compression (app-defined method)");
 
 #ifdef H5_WANT_H5_V1_4_COMPAT
-        if (H5Zregister (H5Z_FILTER_BOGUS, "bogus", bogus)<0) goto error;
+        if (H5Zregister (H5Z_FILTER_BOGUS, "bogus", bogus)<0)
+	    throw Exception("test_compression", "Failed in app-defined method");
 #else /* H5_WANT_H5_V1_4_COMPAT */
-        if (H5Zregister (H5Z_BOGUS)<0) goto error;
+        if (H5Zregister (H5Z_BOGUS)<0)
+	    throw Exception("test_compression", "Failed in app-defined method");
 #endif /* H5_WANT_H5_V1_4_COMPAT */
-	if (H5Pset_filter (dscreatplist.getId(), H5Z_FILTER_BOGUS, 0, 0, NULL)<0) goto error;
+	if (H5Pset_filter (dscreatplist.getId(), H5Z_FILTER_BOGUS, 0, 0, NULL)<0)
+	    throw Exception("test_compression", "Failed in app-defined method");
 	dscreatplist.setFilter (H5Z_FILTER_BOGUS, 0, 0, NULL);
 
 	DataSpace space2 (2, size, NULL);
@@ -662,13 +672,13 @@ test_compression(H5File& file)
 	dataset->write ((void*)points, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
 	dataset->read ((void*)check, PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
     
-	/* Check that the values read are the same as the values written */
+	// Check that the values read are the same as the values written
 	for (i = 0; i < size[0]; i++)
 	    for (j = 0; j < size[1]; j++)
 	    {
 		int status = check_values (i, j, points[i][j], check[i][j]);
 		if (status == -1) 
-		    goto error;
+		    throw Exception("test_compression", "Failed in app-defined method");
 	    }
 
 	PASSED();
@@ -683,13 +693,18 @@ test_compression(H5File& file)
     } // end try
 
     // catch all dataset, file, space, and plist exceptions
-    catch (Exception E) { goto error; }
-   
-  error:
-    // cleaning up
-    if (tconv_buf)
-	delete [] tconv_buf;
-    return -1;
+    catch (Exception E)
+    {
+        cerr << " FAILED" << endl;
+        cerr << "    <<<  " << E.getDetailMsg() << "  >>>" << endl << endl;
+
+        // clean up and return with failure
+        if (dataset != NULL)
+            delete dataset;
+        if (tconv_buf)
+            delete [] tconv_buf;
+        return -1;
+    }
 }
 
 /*-------------------------------------------------------------------------
@@ -716,6 +731,8 @@ test_multiopen (H5File& file)
 {
 
     TESTING("multi-open with extending");
+
+    DataSpace* space = NULL;
     try {
 
 	// Create a dataset creation property list
@@ -727,7 +744,7 @@ test_multiopen (H5File& file)
 
 	// Create a simple data space with unlimited size
 	static hsize_t	max_size[1] = {H5S_UNLIMITED};
-	DataSpace* space = new DataSpace (1, cur_size, max_size);
+	space = new DataSpace (1, cur_size, max_size);
 
 	// Create first dataset
 	DataSet dset1 = file.createDataSet ("multiopen", PredType::NATIVE_INT, *space, dcpl);
@@ -737,25 +754,25 @@ test_multiopen (H5File& file)
 
 	// Relieve the dataspace
 	delete space;
+	space = NULL;
 
 	// Extend the dimensionality of the first dataset
 	cur_size[0] = 20;
 	dset1.extend (cur_size);
 
-	/* Get the size from the second handle */
+	// Get the size from the second handle
 	space = new DataSpace (dset2.getSpace());
 
 	hsize_t		tmp_size[1];
 	space->getSimpleExtentDims (tmp_size);
 	if (cur_size[0]!=tmp_size[0]) 
 	{
-	    H5_FAILED();
-	    cout << "    Got " << (int)tmp_size[0] << " instead of " 
+	    cerr << "    Got " << (int)tmp_size[0] << " instead of " 
 		    << (int)cur_size[0] << "!" << endl;
-	    delete space;
-	    goto error;
+	    throw Exception("test_multiopen", "Failed in multi-open with extending");
 	}
     
+	// clean up and return with success
 	delete space;
 	PASSED();
 	return 0;
@@ -763,10 +780,14 @@ test_multiopen (H5File& file)
 
     // catch all dataset, file, space, and plist exceptions
     catch (Exception E)
-    { goto error; }
+    {
+        cerr << " FAILED" << endl;
+        cerr << "    <<<  " << E.getDetailMsg() << "  >>>" << endl << endl;
 
- error:
-    return -1;
+        if (space != NULL)
+            delete space;
+        return -1;
+    }
 }
 
 
@@ -789,9 +810,10 @@ test_multiopen (H5File& file)
 static herr_t
 test_types(H5File& file)
 {
-    size_t		i;
-
     TESTING("various datatypes");
+
+    size_t		i;
+    DataSet* dset = NULL;
     try {
 
 	// Create a group in the file that was passed in from the caller
@@ -808,6 +830,7 @@ test_types(H5File& file)
 	    // Test copying a user-defined type using DataType::copy
 	    DataType copied_type;
 	    copied_type.copy(type);
+
 	    // Test copying a user-defined type using DataType::operator=
 	    DataType another_copied_type;
 	    another_copied_type = type;
@@ -825,97 +848,94 @@ test_types(H5File& file)
 	    another_int_type = new_int_type;
 
 	    DataSpace space (1, &nelmts);
-	    DataSet* dset = new DataSet(grp.createDataSet("bitfield_1", type, space));
+	    dset = new DataSet(grp.createDataSet("bitfield_1", type, space));
 
 	    // Fill buffer
 	    for (i=0; i<sizeof buf; i++) 
 	    	buf[i] = (unsigned char)0xff ^ (unsigned char)i;
 
-	    // Write data from buf using all default dataspaces and property 
-	    // list; if writing fails, deallocate dset and return.
-	    try { dset->write (buf, type); }
-	    catch(DataSetIException E) 
-	    {
-		delete dset;
-		goto error;
-	    }
-	    delete dset;
+            // Write data from buf using all default dataspaces and property
+            // list
+            dset->write (buf, type);
 
+            // no failure in bitfield_1, close this dataset
+            delete dset;
 	} // end try block of bitfield_1
 
 	// catch exceptions thrown in try block of bitfield_1
-	catch (Exception E) { 
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
-	}
+	catch (Exception E) 
+        {
+            cerr << " FAILED" << endl;
+            cerr << "    <<<  " << "bitfield_1: " << E.getFuncName()
+                 << " - " << E.getDetailMsg() << "  >>>" << endl << endl;
+            if (dset != NULL)
+                delete dset;
+            return -1;
+        }
 
 	/* bitfield_2 */
 	nelmts = sizeof(buf)/2;
 	try { // bitfield_2 block
 	    type.copy (PredType::STD_B16LE);
 	    DataSpace space (1, &nelmts);
-	    DataSet* dset = new DataSet(grp.createDataSet("bitfield_2", type, space));
+	    dset = new DataSet(grp.createDataSet("bitfield_2", type, space));
 
 	    // Fill buffer
 	    for (i=0; i<sizeof(buf); i++) 
 		buf[i] = (unsigned char)0xff ^ (unsigned char)i;
 
-	    // Write data from buf using all default dataspaces and property 
-	    // list; if writing fails, deallocate dset and return.
-	    try { dset->write (buf, type); }
-	    catch(DataSetIException E) 
-	    {
-	    	cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-		delete dset;
-		goto error;
-	    }
-	    delete dset;
+            // Write data from buf using all default dataspaces and property
+            // list; if writing fails, deallocate dset and return.
+            dset->write (buf, type);
+
+            // no failure in bitfield_2, close this dataset and reset for
+            // variable reuse
+            delete dset;
+            dset = NULL;
 	} // end try block of bitfield_2
 
 	// catch exceptions thrown in try block of bitfield_2
 	catch (Exception E) {
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
-	}
+            cerr << " FAILED" << endl;
+            cerr << "    <<<  " << "bitfield_2: " << E.getFuncName()
+                 << " - " << E.getDetailMsg() << "  >>>" << endl << endl;
+            if (dset != NULL)
+                delete dset;
+            throw E; // propagate the exception
+        }
 
         /* opaque_1 */
-	DataType* optype = new DataType(H5T_OPAQUE, 1);
+	DataType* optype = NULL;
 	try { // opaque_1 block
+	    optype = new DataType(H5T_OPAQUE, 1);
 	    nelmts = sizeof(buf);
 	    DataSpace space (1, &nelmts);
 	    optype->setTag ("testing 1-byte opaque type");
-	    DataSet* dset = new DataSet(grp.createDataSet("opaque_1", *optype, space));
+	    dset = new DataSet(grp.createDataSet("opaque_1", *optype, space));
 
 	    // Fill buffer
 	    for (i=0; i<sizeof buf; i++) 
 		buf[i] = (unsigned char)0xff ^ (unsigned char)i;
 
-	    // Write data from buf using all default dataspaces and property 
-	    // list; if writing fails, deallocate dset and return.
-	    try { dset->write (buf, *optype); }
-	    catch(DataSetIException E) 
-	    {
-		delete dset;
-		goto error;
-	    }
-	    delete dset;
-	    delete optype;
+            // Write data from buf using all default dataspaces and property
+            // list; if writing fails, deallocate dset and return.
+            dset->write (buf, *optype);
+
+            // no failure in opaque_1
+            delete dset; dset = NULL;
+            delete optype; optype = NULL;
 	} // end try block of opaque_1
 
 	// catch exceptions thrown in try block of opaque_1
 	catch (DataSetIException E) { 
-	    delete optype;
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
-	}
-	catch (Exception E) {
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
+            cerr << " FAILED" << endl;
+            cerr << "    <<<  " << "opaque_1: " << E.getFuncName()
+                 << " - " << E.getDetailMsg() << "  >>>" << endl << endl;
+            if (dset != NULL)
+                delete dset;
+            if (optype != NULL)
+                delete optype;
+            throw E; // propagate the exception
 	}
     
 	/* opaque_2 */
@@ -924,47 +944,41 @@ test_types(H5File& file)
 	    DataSpace space (1, &nelmts);
 	    optype = new DataType(H5T_OPAQUE, 4);
 	    optype->setTag ("testing 4-byte opaque type");
-	    DataSet* dset = new DataSet(grp.createDataSet("opaque_2", *optype, space));
+	    dset = new DataSet(grp.createDataSet("opaque_2", *optype, space));
 
 	    // Fill buffer
 	    for (i=0; i<sizeof(buf); i++) 
 	        buf[i] = (unsigned char)0xff ^ (unsigned char)i;
 
-	    // Write data from buf using all default dataspaces and property 
-	    // list; if writing fails, deallocate dset and return.
-	    try { dset->write (buf, *optype); }
-	    catch(DataSetIException E) 
-	    {
-		delete dset;
-		goto error;
-	    }
-	    delete dset;
-	    delete optype;
+            // Write data from buf using all default dataspaces and property
+            // list; if writing fails, deallocate dset and return.
+            dset->write (buf, *optype);
+
+            // no failure in opaque_1
+            delete dset; dset = NULL;
+            delete optype; optype = NULL;
 	} //end try block of opaque_2
-	catch (DataSetIException E) { 
-	    delete optype;
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
-	}
+
+	// catch exceptions thrown in try block of opaque_2
 	catch (Exception E) {
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
+            cerr << " FAILED" << endl;
+            cerr << "    <<<  " << "opaque_2: " << E.getFuncName()
+                 << " - " << E.getDetailMsg() << "  >>>" << endl << endl;
+            if (dset != NULL)
+                delete dset;
+            if (optype != NULL)
+                delete optype;
+            throw E; // propagate the exception
 	}
     
     PASSED();
     return 0;
     } // end top try block
 
-    catch (Exception E) { // Group and DataType exceptions
-	    cout << "Failure in " << E.getFuncName() << " - " 
-		 << E.getDetailMsg() << endl;
-	    goto error;
+    catch (Exception E)
+    {
+        return -1;
     }
-
- error:
-    return -1;
 }
 
 /*-------------------------------------------------------------------------
@@ -972,9 +986,9 @@ test_types(H5File& file)
  *
  * Purpose:	Tests the dataset interface (H5D)
  *
- * Return:	Success:	exit(0)
+ * Return:	Success: 0
  *
- *		Failure:	exit(1)
+ *		Failure: -1
  *
  * Programmer:	Binh-Minh Ribler (using C version)
  *		Friday, January 5, 2001
@@ -984,6 +998,8 @@ test_types(H5File& file)
  *		- moved h5_cleanup to outside of try block because
  *		  dataset.h5 cannot be removed until "file" is out of
  *		  scope and dataset.h5 is closed. 
+ *	Feb 20, 05:
+ *		- cleanup_dsets took care of the cleanup now.
  *
  *-------------------------------------------------------------------------
  */
@@ -995,9 +1011,6 @@ main(void)
     hid_t	fapl_id;
     fapl_id = h5_fileaccess(); // in h5test.c, returns a file access template
     
-    char        filename[1024];
-    h5_fixname(FILENAME[0], fapl_id, filename, sizeof filename);
-
     int		nerrors=0;	// keep track of number of failures occurr
     try 
     {
@@ -1010,9 +1023,9 @@ main(void)
 	// list object to pass in H5File::H5File
 	FileAccPropList fapl(fapl_id);
 
-	H5File file( filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl);
+	H5File file(FILE1, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl);
        
-	/* Cause the library to emit initial messages */
+	// Cause the library to emit initial messages
 	Group grp = file.createGroup( "emit diagnostics", 0);
 	grp.setComment( ".", "Causes diagnostic messages to be emitted");
 
@@ -1022,20 +1035,36 @@ main(void)
 	nerrors += test_compression(file)<0	?1:0;
 	nerrors += test_multiopen (file)<0	?1:0;
 	nerrors += test_types(file)<0       ?1:0;
-
-	// increment the ref count of this property list so that the 
-	// property list id won't be closed when fapl goes out of scope.
-	// This is a bad hack, but I want to use existing routine h5_cleanup!
-	fapl.incRefCount();
     }
     catch (Exception E) 
     {
 	return(test_report(nerrors, string(" Dataset")));
     }
-    /* use C test utility routine to clean up data files */
-    h5_cleanup(FILENAME, fapl_id);
 
-    /* print out dsets test results */
+    // Clean up data file
+    cleanup_dsets();
+
+    // Print out dsets test results 
     cerr << endl << endl;
     return(test_report(nerrors, string(" Dataset")));
 }
+
+/*-------------------------------------------------------------------------
+ * Function:    cleanup_dsets
+ *
+ * Purpose:     Cleanup temporary test files
+ *
+ * Return:      none
+ *
+ * Programmer:  (use C version)
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+void
+cleanup_dsets(void)
+{
+    remove(FILE1.c_str());
+} // cleanup_dsets
+
