@@ -125,11 +125,12 @@ DONE:
  *              buf      - character data buffer
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Wednesday, August 6, 1999
- * Modifications:
+ *              Tuesday, May 14, 2002
+ * Modifications: This function is added to accomodate oveloaded h5dwrite_f
+ *                with the dims argument being of INTEGER(HSIZE_T) type
  *---------------------------------------------------------------------------*/
 int_f
-nh5dwritec_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, int_f *dims)
+nh5dwritec_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      
@@ -152,83 +153,12 @@ nh5dwritec_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid
  *              buf      - data buffer
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Wednesday, August 6, 1999
- * Modifications:
- *---------------------------------------------------------------------------*/
-int_f
-nh5dwrite_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, int_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Call H5Dwrite function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dwrite(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf);
-
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}      
-/*----------------------------------------------------------------------------
- * Name:        h5dwritec_c_b
- * Purpose:     Call h5dwrite_c_b to write a dataset of characters 
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - character data buffer
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
  *              Tuesday, May 14, 2002
  * Modifications: This function is added to accomodate oveloaded h5dwrite_f
  *                with the dims argument being of INTEGER(HSIZE_T) type
  *---------------------------------------------------------------------------*/
 int_f
-nh5dwritec_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, hsize_t_f *dims)
-{
-     int ret_value = -1;
-     
-     /*
-      * Call h5dwrite_c  function.
-      */
-     ret_value = nh5dwrite_c_b(dset_id, mem_type_id, mem_space_id, file_space_id, xfer_prp, _fcdtocp(buf), dims);
-
-     return ret_value;
-}
-
-/*----------------------------------------------------------------------------
- * Name:        h5dwrite_c_b
- * Purpose:     Call H5Dwrite to write a dataset 
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - data buffer
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
- *              Tuesday, May 14, 2002
- * Modifications: This function is added to accomodate oveloaded h5dwrite_f
- *                with the dims argument being of INTEGER(HSIZE_T) type
- *---------------------------------------------------------------------------*/
-int_f
-nh5dwrite_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, hsize_t_f *dims)
+nh5dwrite_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      herr_t ret;
@@ -270,114 +200,53 @@ nh5dwrite_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hi
  *              n - number of references to be stored.   
  * Returns:     0 on success,e-1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, July 24, 2000 
- * Modifications:
- *---------------------------------------------------------------------------*/
-int_f
-nh5dwrite_ref_obj_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f *buf, int_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hobj_ref_t *buf_c = NULL;
-     int i, n;
-     n = (int)*dims;
-
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-
-     /*
-      * Allocate temporary buffer and copy references from Fortran.
-      */
-      buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(n));
-      if ( buf_c != NULL ) {
-      for (i = 0; i < n; i++) {
-           HDmemcpy(buf_c[i].oid, buf, H5R_OBJ_REF_BUF_SIZE);
-           buf = buf + REF_OBJ_BUF_LEN_F;
-      }
-      }  
-      else return ret_value;
-     
-     /*
-      * Call H5Dwrite function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dwrite(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     HDfree(buf_c);
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}      
-/*----------------------------------------------------------------------------
- * Name:        h5dwrite_ref_obj_c_b
- * Purpose:     Call H5Dwrite to write a dataset  of object references
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - data buffer with references to the objects.
- *              n - number of references to be stored.   
- * Returns:     0 on success,e-1 on failure
- * Programmer:  Elena Pourmal
  *              Tuesday, May 14, 2002
  * Modifications: This function was added to accomodate h5dwrite_f with the
  *                dims argumnet being of INTEGER(HSIZE_T) type.
  *---------------------------------------------------------------------------*/
 int_f
-nh5dwrite_ref_obj_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f *buf, hsize_t_f *dims)
+nh5dwrite_ref_obj_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, haddr_t_f *buf, hsize_t_f *dims)
 {
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hobj_ref_t *buf_c = NULL;
-     int i, n;
-     n = (int)*dims;
+    int ret_value = -1;
+    herr_t ret;
+    hid_t c_dset_id;
+    hid_t c_mem_type_id;
+    hid_t c_mem_space_id;
+    hid_t c_file_space_id;
+    hid_t c_xfer_prp;
+    hobj_ref_t *buf_c;
+    int i, n;
 
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-
-     /*
-      * Allocate temporary buffer and copy references from Fortran.
-      */
-      buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(n));
-      if ( buf_c != NULL ) {
-      for (i = 0; i < n; i++) {
-           HDmemcpy(buf_c[i].oid, buf, H5R_OBJ_REF_BUF_SIZE);
-           buf = buf + REF_OBJ_BUF_LEN_F;
-      }
-      }  
-      else return ret_value;
-     
-     /*
-      * Call H5Dwrite function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dwrite(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     HDfree(buf_c);
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
+    /*
+     * Define transfer property
+     */
+    c_xfer_prp = (hid_t)*xfer_prp;
 
+    /*
+     * Allocate temporary buffer and copy references from Fortran.
+     */
+    n = (int)*dims;
+    buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(n));
+    if ( buf_c != NULL ) {
+        for (i = 0; i < n; i++)
+             HDmemcpy(&buf_c[i], &buf[i], sizeof(haddr_t));
+    }  
+    else return ret_value;
+    
+    /*
+     * Call H5Dwrite function.
+     */
+    c_dset_id = (hid_t)*dset_id;
+    c_mem_type_id = (hid_t)*mem_type_id;
+    c_mem_space_id = (hid_t)*mem_space_id;
+    c_file_space_id = (hid_t)*file_space_id;
+    ret = H5Dwrite(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
+    HDfree(buf_c);
+    if (ret < 0) return ret_value;
+    ret_value = 0;
+    return ret_value;
 }
+
 /*----------------------------------------------------------------------------
  * Name:        h5dwrite_ref_reg_c
  * Purpose:     Call H5Dwrite to write a dataset of dataset region references
@@ -390,72 +259,12 @@ nh5dwrite_ref_obj_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_spac
  *              n - number of references to be stored.   
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, July 24, 2000 
- * Modifications:
- *---------------------------------------------------------------------------*/
-int_f
-nh5dwrite_ref_reg_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f *buf, int_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hdset_reg_ref_t *buf_c = NULL;
-     int i, n;
-
-      n = (int)*dims;
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Allocate temporary buffer and copy references from Fortran.
-      */
-      buf_c = (hdset_reg_ref_t *)HDmalloc(sizeof(hdset_reg_ref_t)*(n));
-      if ( buf_c != NULL ) {
-      for (i = 0; i < n; i++) {
-           HDmemcpy(buf_c[i].heapid, buf, H5R_DSET_REG_REF_BUF_SIZE);
-           buf = buf + REF_REG_BUF_LEN_F;
-      }
-      }  
-      else return ret_value;
-        
-     
-     /*
-      * Call H5Dwrite function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dwrite(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     HDfree(buf_c);
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}      
-/*----------------------------------------------------------------------------
- * Name:        h5dwrite_ref_reg_c_b
- * Purpose:     Call H5Dwrite to write a dataset of dataset region references
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - data buffer with references to the objects.
- *              n - number of references to be stored.   
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
  *              Tuesday, May 14, 2002
  * Modifications: This function was added to accomodate h5dwrite_f with the
  *                dims argument being of INTEGER(HSIZE_T) type
  *---------------------------------------------------------------------------*/
 int_f
-nh5dwrite_ref_reg_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f *buf, hsize_t_f *dims)
+nh5dwrite_ref_reg_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f *buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      herr_t ret;
@@ -479,7 +288,7 @@ nh5dwrite_ref_reg_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_spac
       buf_c = (hdset_reg_ref_t *)HDmalloc(sizeof(hdset_reg_ref_t)*(n));
       if ( buf_c != NULL ) {
       for (i = 0; i < n; i++) {
-           HDmemcpy(buf_c[i].heapid, buf, H5R_DSET_REG_REF_BUF_SIZE);
+           HDmemcpy(&buf_c[i], buf, H5R_DSET_REG_REF_BUF_SIZE);
            buf = buf + REF_REG_BUF_LEN_F;
       }
       }  
@@ -513,11 +322,12 @@ nh5dwrite_ref_reg_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_spac
  * Outputs:     buf      - character data buffer
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, August 9, 1999
- * Modifications:
+ *              Wednesday, May 15, 2002
+ * Modifications: This function was added to accomodate h5dread_f subroutine
+ *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
  *---------------------------------------------------------------------------*/
 int_f
-nh5dreadc_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, int_f *dims)
+nh5dreadc_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      
@@ -540,11 +350,12 @@ nh5dreadc_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_
  * Outputs:     buf      - data buffer
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, August 9, 1999
- * Modifications:
+ *              Wednesday, May 15, 2002
+ * Modifications: This function was added to accomodate h5dread_f subroutine
+ *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
  *---------------------------------------------------------------------------*/
 int_f
-nh5dread_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, int_f *dims)
+nh5dread_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      herr_t ret;
@@ -572,79 +383,6 @@ nh5dread_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t
      ret_value = 0;
      return ret_value;
 }
-/*----------------------------------------------------------------------------
- * Name:        h5dreadc_c_b
- * Purpose:     Call h5dread_c_b to read a dataset of characters 
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- * Outputs:     buf      - character data buffer
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
- *              Wednesday, May 15, 2002
- * Modifications: This function was added to accomodate h5dread_f subroutine
- *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
- *---------------------------------------------------------------------------*/
-int_f
-nh5dreadc_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, _fcd buf, hsize_t_f *dims)
-{
-     int ret_value = -1;
-     
-     /*
-      * Call h5dread_c  function.
-      */
-     ret_value = nh5dread_c_b(dset_id, mem_type_id, mem_space_id, file_space_id, xfer_prp, _fcdtocp(buf), dims);
-
-     return ret_value;
-}
-
-/*----------------------------------------------------------------------------
- * Name:        h5dread_c
- * Purpose:     Call H5Draed to read a dataset 
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- * Outputs:     buf      - data buffer
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
- *              Wednesday, May 15, 2002
- * Modifications: This function was added to accomodate h5dread_f subroutine
- *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
- *---------------------------------------------------------------------------*/
-int_f
-nh5dread_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, void *buf, hsize_t_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Call H5Dread function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf);
-
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}
-
 
 /*----------------------------------------------------------------------------
  * Name:        h5dread_ref_obj_c
@@ -658,110 +396,51 @@ nh5dread_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid
  *              n - number of references to be stored.   
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, July 24, 2000 
- * Modifications:
- *---------------------------------------------------------------------------*/
-int_f
-nh5dread_ref_obj_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f * buf, int_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hobj_ref_t *buf_c = NULL;
-     int i, n;
-     n = (int)*dims;
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Allocate temporary buffer.
-      */
-     buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(n));
-     if ( buf_c != NULL ) {
-     /*
-      * Call H5Dread function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     if (ret >=0) {
-        for (i = 0; i < n; i++) {
-           HDmemcpy(buf, buf_c[i].oid, H5R_OBJ_REF_BUF_SIZE);
-           buf = buf + REF_OBJ_BUF_LEN_F;
-        }  
-     }
-     if ( buf_c != NULL ) HDfree(buf_c);
-     } 
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}      
-/*----------------------------------------------------------------------------
- * Name:        h5dread_ref_obj_c_b
- * Purpose:     Call H5Dread to read a dataset  of object references
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - data buffer to store references to the objects.
- *              n - number of references to be stored.   
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
  *              Wednesday, May 15, 2002
  * Modifications: This function was added to accomodate h5dread_f subroutine
  *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
  *---------------------------------------------------------------------------*/
 int_f
-nh5dread_ref_obj_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f * buf, hsize_t_f *dims)
+nh5dread_ref_obj_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, haddr_t_f * buf, hsize_t_f *dims)
 {
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hobj_ref_t *buf_c = NULL;
-     hsize_t i,n;
-     n = (hsize_t)*dims;
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Allocate temporary buffer.
-      */
-     buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(n));
-     if ( buf_c != NULL ) {
-     /*
-      * Call H5Dread function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     if (ret >=0) {
-        for (i = 0; i < n; i++) {
-           HDmemcpy(buf, buf_c[i].oid, H5R_OBJ_REF_BUF_SIZE);
-           buf = buf + REF_OBJ_BUF_LEN_F;
-        }  
-     }
-     if ( buf_c != NULL ) HDfree(buf_c);
-     } 
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
+    int ret_value = -1;
+    herr_t ret;
+    hid_t c_dset_id;
+    hid_t c_mem_type_id;
+    hid_t c_mem_space_id;
+    hid_t c_file_space_id;
+    hid_t c_xfer_prp;
+    hobj_ref_t *buf_c = NULL;
+    hsize_t i,n;
+
+    /*
+     * Define transfer property
+     */
+    c_xfer_prp = (hid_t)*xfer_prp;
+    
+    /*
+     * Allocate temporary buffer.
+     */
+    n = (hsize_t)*dims;
+    buf_c = (hobj_ref_t*)HDmalloc(sizeof(hobj_ref_t)*(size_t)n);
+    if ( buf_c != NULL ) {
+        /*
+         * Call H5Dread function.
+         */
+        c_dset_id = (hid_t)*dset_id;
+        c_mem_type_id = (hid_t)*mem_type_id;
+        c_mem_space_id = (hid_t)*mem_space_id;
+        c_file_space_id = (hid_t)*file_space_id;
+        ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
+        if (ret >=0) {
+           for (i = 0; i < n; i++)
+              HDmemcpy(&buf[i], &buf_c[i], sizeof(haddr_t));
+        }
+        if ( buf_c != NULL ) HDfree(buf_c);
+    } 
+    if (ret < 0) return ret_value;
+    ret_value = 0;
+    return ret_value;
 }
 
 /*----------------------------------------------------------------------------
@@ -776,70 +455,12 @@ nh5dread_ref_obj_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space
  *              n - number of references to be stored.   
  * Returns:     0 on success, -1 on failure
  * Programmer:  Elena Pourmal
- *              Monday, July 24, 2000 
- * Modifications:
- *---------------------------------------------------------------------------*/
-int_f
-nh5dread_ref_reg_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f * buf, int_f *dims)
-{
-     int ret_value = -1;
-     herr_t ret;
-     hid_t c_dset_id;
-     hid_t c_mem_type_id;
-     hid_t c_mem_space_id;
-     hid_t c_file_space_id;
-     hid_t c_xfer_prp;
-     hdset_reg_ref_t *buf_c = NULL;
-     int i, n;
-     n = (int)*dims;
-     /*
-      * Define transfer property
-      */
-     c_xfer_prp = (hid_t)*xfer_prp;
-     
-     /*
-      * Allocate temporary buffer.
-      */
-     buf_c = (hdset_reg_ref_t *)HDmalloc(sizeof(hdset_reg_ref_t)*(n));
-     if ( buf_c != NULL ) {
-     /*
-      * Call H5Dread function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     if (ret >=0) {
-        for (i = 0; i < n; i++) {
-           HDmemcpy(buf, buf_c[i].heapid, H5R_DSET_REG_REF_BUF_SIZE);
-           buf = buf + REF_REG_BUF_LEN_F;
-        }  
-     }
-     if ( buf_c != NULL ) HDfree(buf_c);
-     } 
-     if (ret < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
-}      
-/*----------------------------------------------------------------------------
- * Name:        h5dread_ref_reg_c_b
- * Purpose:     Call H5Dread to read a dataset of dataset region references
- * Inputs:      dset_id - dataset identifier 
- *              mem_type_id - memory datatype identifier
- *              mem_space_id - memory dataspace identifier
- *              file_space_id - memory dataspace identifier
- *              xfer_pr  - identifier of transfer property list
- *              buf      - data buffer to store references to the objects.
- *              n - number of references to be stored.   
- * Returns:     0 on success, -1 on failure
- * Programmer:  Elena Pourmal
  *              Wednesday, May 15, 2002
  * Modifications: This function was added to accomodate h5dread_f subroutine
  *                with the dims parameter being of INTEGER(HSIZE_T_F) size.
  *---------------------------------------------------------------------------*/
 int_f
-nh5dread_ref_reg_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f * buf, hsize_t_f *dims)
+nh5dread_ref_reg_c (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space_id, hid_t_f *file_space_id, hid_t_f *xfer_prp, int_f * buf, hsize_t_f *dims)
 {
      int ret_value = -1;
      herr_t ret;
@@ -859,23 +480,23 @@ nh5dread_ref_reg_c_b (hid_t_f *dset_id, hid_t_f *mem_type_id, hid_t_f *mem_space
      /*
       * Allocate temporary buffer.
       */
-     buf_c = (hdset_reg_ref_t *)HDmalloc(sizeof(hdset_reg_ref_t)*(n));
+     buf_c = (hdset_reg_ref_t *)HDmalloc(sizeof(hdset_reg_ref_t)*(size_t)n);
      if ( buf_c != NULL ) {
-     /*
-      * Call H5Dread function.
-      */
-     c_dset_id = (hid_t)*dset_id;
-     c_mem_type_id = (hid_t)*mem_type_id;
-     c_mem_space_id = (hid_t)*mem_space_id;
-     c_file_space_id = (hid_t)*file_space_id;
-     ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
-     if (ret >=0) {
-        for (i = 0; i < n; i++) {
-           HDmemcpy(buf, buf_c[i].heapid, H5R_DSET_REG_REF_BUF_SIZE);
-           buf = buf + REF_REG_BUF_LEN_F;
-        }  
-     }
-     if ( buf_c != NULL ) HDfree(buf_c);
+         /*
+          * Call H5Dread function.
+          */
+         c_dset_id = (hid_t)*dset_id;
+         c_mem_type_id = (hid_t)*mem_type_id;
+         c_mem_space_id = (hid_t)*mem_space_id;
+         c_file_space_id = (hid_t)*file_space_id;
+         ret = H5Dread(c_dset_id, c_mem_type_id, c_mem_space_id, c_file_space_id, c_xfer_prp, buf_c);
+         if (ret >=0) {
+            for (i = 0; i < n; i++) {
+               HDmemcpy(buf, &buf_c[i], H5R_DSET_REG_REF_BUF_SIZE);
+               buf = buf + REF_REG_BUF_LEN_F;
+            }  
+         }
+         if ( buf_c != NULL ) HDfree(buf_c);
      } 
      if (ret < 0) return ret_value;
      ret_value = 0;
@@ -1093,7 +714,7 @@ nh5dvlen_get_max_len_c ( hid_t_f *dset_id ,  hid_t_f *type_id, hid_t_f *space_id
   num_elem = H5Sget_select_npoints(c_space_id);  
   if( num_elem < 0) return ret_value;
 
-  c_buf = (hvl_t *)malloc(sizeof(hvl_t)*num_elem); 
+  c_buf = (hvl_t *)malloc(sizeof(hvl_t)*(size_t)num_elem); 
   if (c_buf == NULL) return ret_value;
   status = H5Dread(c_dset_id, c_type_id, H5S_ALL, c_space_id, H5P_DEFAULT, c_buf);
   if(status < 0) goto DONE;
@@ -1142,7 +763,7 @@ nh5dwrite_vl_integer_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_
   size_t max_len;
 
   hvl_t *c_buf;
-  int i;
+  hsize_t i;
   hsize_t num_elem;
   
   max_len = (size_t)dims[0];
@@ -1154,7 +775,7 @@ nh5dwrite_vl_integer_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_
   c_file_space_id = (hid_t)*file_space_id;
   c_xfer_prp      = (hid_t)*xfer_prp;
 
-  c_buf = (hvl_t *)malloc(num_elem * sizeof(hvl_t)); 
+  c_buf = (hvl_t *)malloc((size_t)num_elem * sizeof(hvl_t)); 
   if (c_buf == NULL) return ret_value;
   tmp = (int *)buf; 
   for (i=0; i < num_elem; i++) {
@@ -1219,7 +840,7 @@ nh5dread_vl_integer_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_s
   num_elem = H5Sget_select_npoints(c_mem_space_id);  
   if(num_elem != dims[1]) return ret_value;
 
-  c_buf = (hvl_t *)malloc(num_elem * sizeof(hvl_t)); 
+  c_buf = (hvl_t *)malloc((size_t)num_elem * sizeof(hvl_t)); 
   if (c_buf == NULL) return ret_value;
   /*
    * Call H5Dread function.
@@ -1269,7 +890,7 @@ nh5dwrite_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_sp
   size_t max_len;
 
   char **c_buf;
-  int i;
+  hsize_t i;
   hsize_t num_elem;
   
   max_len = (size_t)dims[0];
@@ -1284,11 +905,11 @@ nh5dwrite_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_sp
   /*
    * Allocate arra of character pointers
    */
-  c_buf = (char **)malloc(num_elem * sizeof(char *)); 
+  c_buf = (char **)malloc((size_t)num_elem * sizeof(char *)); 
   if (c_buf == NULL) return ret_value;
 
   /* Copy data to long C string */
-  tmp = (char *)HD5f2cstring(buf, max_len*num_elem);
+  tmp = (char *)HD5f2cstring(buf, (int)(max_len*num_elem));
   if (tmp == NULL) { free(c_buf);
                      return ret_value;
                    }
@@ -1297,8 +918,8 @@ nh5dwrite_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_sp
    */
    tmp_p = tmp;
    for (i=0; i < num_elem; i++) {
-        c_buf[i] = (char *) malloc(len[i]+1);
-        memcpy(c_buf[i], tmp_p, len[i]);
+        c_buf[i] = (char *) malloc((size_t)len[i]+1);
+        memcpy(c_buf[i], tmp_p, (size_t)len[i]);
         c_buf[i][len[i]] = '\0'; 
         tmp_p = tmp_p + max_len;
    }
@@ -1348,7 +969,7 @@ nh5dread_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
   size_t max_len;
 
   char **c_buf;
-  int i;
+  hsize_t i;
   hsize_t num_elem;
   
   max_len = (size_t)dims[0];
@@ -1363,7 +984,7 @@ nh5dread_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
   /*
    * Allocate array of character pointers
    */
-  c_buf = (char **)malloc(num_elem * sizeof(char *)); 
+  c_buf = (char **)malloc((size_t)num_elem * sizeof(char *)); 
   if (c_buf == NULL) return ret_value;
 
   /*
@@ -1374,7 +995,7 @@ nh5dread_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
                      return ret_value;
                    }
   /* Copy data to long C string */
-  tmp = (char *)malloc(max_len*num_elem +1);
+  tmp = (char *)malloc((size_t)(max_len*num_elem) +1);
   tmp_p = tmp;
   for (i=0; i<max_len*num_elem; i++) tmp[i] = ' ';
   tmp[max_len*num_elem] = '\0';
@@ -1383,7 +1004,7 @@ nh5dread_vl_string_c( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
         len[i] = (size_t_f)strlen(c_buf[i]);
         tmp_p = tmp_p + max_len;
   }
-  HD5packFstring(tmp, _fcdtocp(buf), max_len*num_elem);  
+  HD5packFstring(tmp, _fcdtocp(buf), (int)(max_len*num_elem));
   ret_value = 0;
   H5Dvlen_reclaim(c_mem_type_id, c_mem_space_id, H5P_DEFAULT, c_buf);
   free(c_buf);
@@ -1420,15 +1041,11 @@ nh5dwrite_vl_real_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
   hid_t c_file_space_id;
   hid_t c_xfer_prp;
   herr_t status;
-#if defined (_UNICOS)
-  double *tmp;
-#else
-  float *tmp;
-#endif
+  real_f *tmp;
   size_t max_len;
 
   hvl_t *c_buf;
-  int i;
+  hsize_t i;
   hsize_t num_elem;
   
   max_len = (size_t)dims[0];
@@ -1440,13 +1057,9 @@ nh5dwrite_vl_real_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spa
   c_file_space_id = (hid_t)*file_space_id;
   c_xfer_prp      = (hid_t)*xfer_prp;
 
-  c_buf = (hvl_t *)malloc(num_elem * sizeof(hvl_t)); 
+  c_buf = (hvl_t *)malloc((size_t)num_elem * sizeof(hvl_t)); 
   if (c_buf == NULL) return ret_value;
-#if defined (_UNICOS)
-  tmp = (double *)buf; 
-#else
-  tmp = (float *)buf;
-#endif
+  tmp = (real_f *)buf; 
   for (i=0; i < num_elem; i++) {
        c_buf[i].len = (size_t)len[i];  
        c_buf[i].p   = tmp;
@@ -1509,7 +1122,7 @@ nh5dread_vl_real_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spac
   num_elem = H5Sget_select_npoints(c_mem_space_id);  
   if(num_elem != dims[1]) return ret_value;
 
-  c_buf = (hvl_t *)malloc(num_elem * sizeof(hvl_t)); 
+  c_buf = (hvl_t *)malloc((size_t)num_elem * sizeof(hvl_t)); 
   if (c_buf == NULL) return ret_value;
   /*
    * Call H5Dread function.
@@ -1518,11 +1131,7 @@ nh5dread_vl_real_c ( hid_t_f *dset_id ,  hid_t_f *mem_type_id, hid_t_f *mem_spac
  if ( status <0  )  goto DONE;
   for (i=0; i < num_elem; i++) {
        len[i] = (size_t_f)c_buf[i].len;  
-#if defined (_UNICOS)
-       memcpy(&buf[i*max_len], c_buf[i].p, c_buf[i].len*sizeof(double));
-#else
-       memcpy(&buf[i*max_len], c_buf[i].p, c_buf[i].len*sizeof(float));
-#endif
+       memcpy(&buf[i*max_len], c_buf[i].p, c_buf[i].len*sizeof(real_f));
   }
   
   H5Dvlen_reclaim(c_mem_type_id, c_mem_space_id, H5P_DEFAULT, c_buf);
