@@ -29,7 +29,7 @@
  * Modifications:
  *---------------------------------------------------------------------------*/
 int_f
-nh5rcreate_object_c (int_f *ref, hid_t_f *loc_id, _fcd name, int_f *namelen)
+nh5rcreate_object_c (haddr_t_f *ref, hid_t_f *loc_id, _fcd name, int_f *namelen)
 {
      int ret_value = -1;
      hid_t c_loc_id;
@@ -53,11 +53,10 @@ nh5rcreate_object_c (int_f *ref, hid_t_f *loc_id, _fcd name, int_f *namelen)
 
      HDfree(c_name);
      if (ret_value_c >= 0)  {
-         HDmemcpy(ref, ref_c.oid, H5R_OBJ_REF_BUF_SIZE);
+         *ref=(haddr_t_f)ref_c;
          ret_value = 0;
      }
        
-     ret_value = 0;
      return ret_value;
 }      
 
@@ -102,7 +101,7 @@ nh5rcreate_region_c (int_f *ref, hid_t_f *loc_id, _fcd name, int_f *namelen, hid
 
      HDfree(c_name);
      if (ret_value_c >= 0) {
-         HDmemcpy (ref, ref_c.heapid, H5R_DSET_REG_REF_BUF_SIZE);
+         HDmemcpy (ref, &ref_c, H5R_DSET_REG_REF_BUF_SIZE);
          ret_value = 0;
      }
      return ret_value;
@@ -127,7 +126,7 @@ nh5rdereference_region_c (hid_t_f *dset_id, int_f *ref, hid_t_f *obj_id)
      hdset_reg_ref_t ref_c;
      hid_t c_obj_id;
 
-     HDmemcpy (ref_c.heapid, ref, H5R_DSET_REG_REF_BUF_SIZE);
+     HDmemcpy (&ref_c, ref, H5R_DSET_REG_REF_BUF_SIZE);
 
      /*
       * Call H5Rdereference function.
@@ -153,14 +152,14 @@ nh5rdereference_region_c (hid_t_f *dset_id, int_f *ref, hid_t_f *obj_id)
  * Modifications:
  *---------------------------------------------------------------------------*/
 int_f
-nh5rdereference_object_c (hid_t_f *dset_id, int_f *ref, hid_t_f *obj_id)
+nh5rdereference_object_c (hid_t_f *dset_id, haddr_t_f *ref, hid_t_f *obj_id)
 {
      int ret_value = -1;
      hid_t c_dset_id;
      hid_t c_obj_id;
      hobj_ref_t ref_c;
 
-     HDmemcpy (ref_c.oid, ref, H5R_OBJ_REF_BUF_SIZE);
+     ref_c=*ref;
 
      /*
       * Call H5Rdereference function.
@@ -192,7 +191,7 @@ nh5rget_region_region_c (hid_t_f *dset_id, int_f *ref, hid_t_f *space_id)
      hid_t c_space_id;
      hdset_reg_ref_t ref_c;
 
-     HDmemcpy (ref_c.heapid, ref, H5R_DSET_REG_REF_BUF_SIZE);
+     HDmemcpy (&ref_c, ref, H5R_DSET_REG_REF_BUF_SIZE);
 
      /*
       * Call H5Rget_region function.
@@ -218,14 +217,14 @@ nh5rget_region_region_c (hid_t_f *dset_id, int_f *ref, hid_t_f *space_id)
  * Modifications:
  *---------------------------------------------------------------------------*/
 int_f
-nh5rget_object_type_obj_c (hid_t_f *dset_id, int_f *ref, int_f *obj_type)
+nh5rget_object_type_obj_c (hid_t_f *dset_id, haddr_t_f *ref, int_f *obj_type)
 {
      int ret_value = -1;
      hid_t c_dset_id;
      int c_obj_type;
      hobj_ref_t ref_c;
 
-     HDmemcpy (ref_c.oid, ref, H5R_OBJ_REF_BUF_SIZE);
+     ref_c=*ref;
 
      /*
       * Call H5Rget_object_type function.
