@@ -435,9 +435,9 @@
 
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: space_id ! Dataspace identifier 
-            INTEGER(HSSIZE_T), DIMENSION(*), INTENT(OUT) :: start
+            INTEGER(HSIZE_T), DIMENSION(*), INTENT(OUT) :: start
                                              !Starting coordinates of the bounding box. 
-            INTEGER(HSSIZE_T), DIMENSION(*), INTENT(OUT) :: end
+            INTEGER(HSIZE_T), DIMENSION(*), INTENT(OUT) :: end
                                              !Ending coordinates of the bounding box,
                                              !i.e., the coordinates of the diagonally 
                                              !opposite corner 
@@ -453,8 +453,8 @@
 !MS$ATTRIBUTES C,reference,alias:'_H5SGET_SELECT_BOUNDS_C'::h5sget_select_bounds_c
               !DEC$ ENDIF
               INTEGER(HID_T), INTENT(IN) :: space_id
-              INTEGER(HSSIZE_T), DIMENSION(*), INTENT(OUT) :: start
-              INTEGER(HSSIZE_T), DIMENSION(*), INTENT(OUT) :: end
+              INTEGER(HSIZE_T), DIMENSION(*), INTENT(OUT) :: start
+              INTEGER(HSIZE_T), DIMENSION(*), INTENT(OUT) :: end
               END FUNCTION h5sget_select_bounds_c
             END INTERFACE
 
@@ -634,13 +634,13 @@
             INTEGER, INTENT(IN) :: rank     ! Number of dataspace dimensions 
             INTEGER(SIZE_T), INTENT(IN) :: num_elements  ! Number of elements to be
                                                  ! selected
-            INTEGER(HSSIZE_T), & 
+            INTEGER(HSIZE_T), & 
             DIMENSION(rank,num_elements), INTENT(IN) :: coord 
                                           ! Array with the coordinates
                                           ! of the selected elements
                                           ! coord(rank, num_elements)
             INTEGER, INTENT(OUT) :: hdferr     ! Error code
-            INTEGER(HSSIZE_T), ALLOCATABLE, DIMENSION(:,:) :: c_coord
+            INTEGER(HSIZE_T), ALLOCATABLE, DIMENSION(:,:) :: c_coord
             INTEGER :: error, i,j
 
 !            INTEGER, EXTERNAL :: h5sselect_elements_c
@@ -656,7 +656,7 @@
               INTEGER(HID_T), INTENT(IN) :: space_id
               INTEGER, INTENT(IN) :: operator
               INTEGER(SIZE_T), INTENT(IN) :: num_elements
-              INTEGER(HSSIZE_T),DIMENSION(*) :: c_c_coord
+              INTEGER(HSIZE_T),DIMENSION(*) :: c_c_coord
               END FUNCTION h5sselect_elements_c
             END INTERFACE
 
@@ -1489,7 +1489,7 @@
                                                 ! H5S_SELECT_SET_F (0)
                                                 ! H5S_SELECT_OR_F (1)
                                                 !  
-            INTEGER(HSSIZE_T), DIMENSION(*), INTENT(IN) :: start
+            INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: start
                                           ! Starting coordinates of the hyperslab 
             INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: count 
                                           ! Number of blocks to select 
@@ -1517,7 +1517,7 @@
               !DEC$ ENDIF
               INTEGER(HID_T), INTENT(IN) :: space_id
               INTEGER, INTENT(IN) :: operator
-              INTEGER(HSSIZE_T), DIMENSION(*), INTENT(IN) :: start
+              INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: start
               INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: count
               INTEGER(HSIZE_T), DIMENSION(*), OPTIONAL, INTENT(IN) :: stride
               INTEGER(HSIZE_T), DIMENSION(*), OPTIONAL, INTENT(IN) :: block 
@@ -1576,308 +1576,6 @@
             deallocate(def_stride)
  
           END SUBROUTINE h5sselect_hyperslab_f
-!----------------------------------------------------------------------
-! Name:		h5scombine_hyperslab_f 
-!
-! Purpose:	Combine a hyperslab selection with the current 
-!               selection for a dataspace 
-!
-! Inputs:  
-!		space_id	- dataspace of selection to use 
-!		operator	- flag, valid values are:
-!				  H5S_SELECT_NOOP_F 
-!				  H5S_SELECT_SET_F 
-!				  H5S_SELECT_OR_F 
-!				  H5S_SELECT_AND_F 
-!				  H5S_SELECT_XOR_F 
-!				  H5S_SELECT_NOTB_F 
-!				  H5S_SELECT_NOTA_F 
-!				  H5S_SELECT_APPEND_F 
-!				  H5S_SELECT_PREPEND_F 
-!		start		- array with hyperslab offsets
-!		count		- number of blocks included in the 
-!				  hyperslab
-! Outputs:  
-!               hyper_id        - identifier for the new hyperslab
-!		hdferr:		- error code		
-!				 	Success:  0
-!				 	Failure: -1   
-! Optional parameters:
-!		stride		- array with hyperslab strides
-!		block		- array with hyperslab block sizes 
-!
-! Programmer:	Elena Pourmal
-!		October 7, 2002
-!
-! Modifications: 	
-!
-! Comment:	Commented out until 1.6 ? 10/08/2002	
-!----------------------------------------------------------------------
-
-!          SUBROUTINE h5scombine_hyperslab_f(space_id, operator, start, count, &
-!                                            hyper_id,  hdferr, stride, block) 
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5scombine_hyperslab_f
-!DEC$endif
-!
-!            IMPLICIT NONE
-!            INTEGER(HID_T), INTENT(IN) :: space_id ! Dataspace identifier 
-!            INTEGER, INTENT(IN) :: operator     ! Flag, valid values are:
-						!  H5S_SELECT_NOOP_F 
-						!  H5S_SELECT_SET_F 
-						!  H5S_SELECT_OR_F 
-						!  H5S_SELECT_AND_F 
-						!  H5S_SELECT_XOR_F 
-						!  H5S_SELECT_NOTB_F 
-						!  H5S_SELECT_NOTA_F 
-						!  H5S_SELECT_APPEND_F 
-						!  H5S_SELECT_PREPEND_F 
-                                                !  
-!            INTEGER(HSSIZE_T), DIMENSION(*), INTENT(IN) :: start
-                                          ! Starting coordinates of the hyperslab 
-!            INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: count 
-                                          ! Number of blocks to select 
-                                          ! from dataspace 
-!            INTEGER(HID_T), INTENT(OUT) :: hyper_id ! New hyperslab identifier 
-!            INTEGER, INTENT(OUT) :: hdferr     ! Error code
-!            INTEGER(HSIZE_T), DIMENSION(:), OPTIONAL, INTENT(IN) :: stride
-                                          ! Array of how many elements to move
-                                          ! in each direction
-!            INTEGER(HSIZE_T), DIMENSION(:), OPTIONAL, INTENT(IN) :: block 
-                                          ! Sizes of element block
-!            INTEGER(HSIZE_T), DIMENSION(:), ALLOCATABLE :: def_block 
-!            INTEGER(HSIZE_T), DIMENSION(:), ALLOCATABLE :: def_stride
-!            INTEGER :: rank
-!            INTEGER :: error1, error2 
-
-!            INTERFACE
-!              INTEGER FUNCTION h5scombine_hyperslab_c(space_id, operator, &
-!                               start, count, stride, block, hyper_id)
-!              USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5SCOMBINE_HYPERSLAB_C'::h5scombine_hyperslab_c
-              !DEC$ ENDIF
-!              INTEGER(HID_T), INTENT(IN) :: space_id
-!              INTEGER, INTENT(IN) :: operator
-!              INTEGER(HSSIZE_T), DIMENSION(*), INTENT(IN) :: start
-!              INTEGER(HSIZE_T), DIMENSION(*), INTENT(IN) :: count
-!              INTEGER(HSIZE_T), DIMENSION(*), OPTIONAL, INTENT(IN) :: stride
-!              INTEGER(HSIZE_T), DIMENSION(*), OPTIONAL, INTENT(IN) :: block 
-!              INTEGER(HID_T), INTENT(OUT) :: hyper_id
-!              END FUNCTION h5scombine_hyperslab_c
-!            END INTERFACE
-
-!            if (present(stride).and. present(block)) then
-!            hdferr = h5scombine_hyperslab_c(space_id, operator, start, count, &
-!                                           stride, block, hyper_id)
-!            return
-!            endif
-            ! Case of optional parameters.
-            !
-            ! Find the rank of the dataspace to allocate memery for
-            ! default stride and block arrays.
-            !
-!            CALL h5sget_simple_extent_ndims_f(space_id, rank, hdferr)
-!            if( hdferr .EQ. -1) return
-            !
-!            if (present(stride).and. .not.present(block)) then
-!            allocate(def_block(rank), stat=error1)
-!                if (error1.NE.0) then
-!                    hdferr = -1
-!                    return
-!                endif
-!            def_block = 1
-!            hdferr = h5scombine_hyperslab_c(space_id, operator, start, count, &
-!                                           stride, def_block, hyper_id)
-!            deallocate(def_block)
-!            return
-!            endif
-
-!            if (.not.present(stride).and. present(block)) then
-!            allocate(def_stride(rank), stat=error2)
-!                if (error2.NE.0) then
-!                    hdferr = -1
-!                    return
-!                endif
-!            def_stride = 1
-!            hdferr = h5scombine_hyperslab_c(space_id, operator, start, count, &
-!                                           def_stride, block, hyper_id)
-!            deallocate(def_stride)
-!            return
-!            endif
-!            allocate(def_block(rank), stat=error1)
-!            allocate(def_stride(rank), stat=error2)
-!                if ((error1.NE.0) .OR. (error2.NE.0)) then
-!                    hdferr = -1
-!                    return
-!                endif
-!            def_block = 1
-!            def_stride = 1
-!            hdferr = h5scombine_hyperslab_c(space_id, operator, start, count, &
-!                                           def_stride, def_block, hyper_id)
-!            deallocate(def_block)
-!            deallocate(def_stride)
- 
-!          END SUBROUTINE h5scombine_hyperslab_f
-
-!----------------------------------------------------------------------
-! Name:		h5scombine_select_f 
-!
-! Purpose:	Combine two hyperslab selections with an operation 
-!               and return a dataspace with resulting selection.
-!
-! Inputs:  
-!		space1_id	- dataspace of selection to use 
-!		operator	- flag, valid values are:
-!				  H5S_SELECT_NOOP_F 
-!				  H5S_SELECT_SET_F 
-!				  H5S_SELECT_OR_F 
-!				  H5S_SELECT_AND_F 
-!				  H5S_SELECT_XOR_F 
-!				  H5S_SELECT_NOTB_F 
-!				  H5S_SELECT_NOTA_F 
-!				  H5S_SELECT_APPEND_F 
-!				  H5S_SELECT_PREPEND_F 
-!		space2_id	- dataspace of selection to use 
-! Outputs:  
-!               ds_id           - idataspace identifier with the new selection
-!		hdferr:		- error code		
-!				 	Success:  0
-!				 	Failure: -1   
-! Optional parameters:		- NONE
-!
-! Programmer:	Elena Pourmal
-!		October 7, 2002
-!
-! Modifications: 	
-!
-! Comment: commented out until 1.6 release(?) 10/08/2002		
-!----------------------------------------------------------------------
-
-!          SUBROUTINE h5scombine_select_f(space1_id, operator, space2_id, &
-!                                            ds_id,  hdferr) 
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5scombine_select_f
-!DEC$endif
-!
-!            IMPLICIT NONE
-!            INTEGER(HID_T), INTENT(IN) :: space1_id ! First dataspace identifier 
-!            INTEGER(HID_T), INTENT(IN) :: space2_id ! Second dataspace identifier 
-!            INTEGER, INTENT(IN) :: operator     ! Flag, valid values are:
-						!  H5S_SELECT_NOOP_F 
-						!  H5S_SELECT_SET_F 
-						!  H5S_SELECT_OR_F 
-						!  H5S_SELECT_AND_F 
-						!  H5S_SELECT_XOR_F 
-						!  H5S_SELECT_NOTB_F 
-						!  H5S_SELECT_NOTA_F 
-						!  H5S_SELECT_APPEND_F 
-						!  H5S_SELECT_PREPEND_F 
-                                                !  
-!            INTEGER(HID_T), INTENT(OUT) :: ds_id ! New dataspace identifier 
-!            INTEGER, INTENT(OUT) :: hdferr     ! Error code
-!
-!            INTERFACE
-!              INTEGER FUNCTION h5scombine_select_c(space1_id, operator, &
-!                               space2_id, ds_id)
-!              USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5SCOMBINE_SELECT_C'::h5scombine_select_c
-              !DEC$ ENDIF
-!              INTEGER(HID_T), INTENT(IN) :: space1_id
-!              INTEGER(HID_T), INTENT(IN) :: space2_id
-!              INTEGER, INTENT(IN) :: operator
-!              INTEGER(HID_T), INTENT(OUT) :: ds_id
-!              END FUNCTION h5scombine_select_c
-!            END INTERFACE
-
-!            hdferr = h5scombine_select_c(space1_id, operator, space2_id, &
-!                                         ds_id)
-!            return
- 
-!          END SUBROUTINE h5scombine_select_f
-
-!----------------------------------------------------------------------
-! Name:		h5sselect_select_f 
-!
-! Purpose:	Refine a hyperslab selection with an operation 
-!               using second hyperslab
-!
-! Inputs:  
-!		space1_id	- dataspace of selection  to modify
-!		operator	- flag, valid values are:
-!				  H5S_SELECT_NOOP_F 
-!				  H5S_SELECT_SET_F 
-!				  H5S_SELECT_OR_F 
-!				  H5S_SELECT_AND_F 
-!				  H5S_SELECT_XOR_F 
-!				  H5S_SELECT_NOTB_F 
-!				  H5S_SELECT_NOTA_F 
-!				  H5S_SELECT_APPEND_F 
-!				  H5S_SELECT_PREPEND_F 
-!		space2_id	- dataspace of selection to use 
-!				  
-! Outputs:  
-!		hdferr:		- error code		
-!				 	Success:  0
-!				 	Failure: -1   
-! Optional parameters:		- NONE
-!
-! Programmer:	Elena Pourmal
-!		October 7, 2002
-!
-! Modifications: 	
-!
-! Comment:Commented out until 1.6 release(?) 10/08/2002 EIP		
-!----------------------------------------------------------------------
-
-!          SUBROUTINE h5sselect_select_f(space1_id, operator, space2_id, &
-!                                        hdferr) 
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5sselect_select_f
-!DEC$endif
-!
-!            IMPLICIT NONE
-!            INTEGER(HID_T), INTENT(INOUT) :: space1_id ! Dataspace identifier to
-                                                       ! modify 
-!            INTEGER(HID_T), INTENT(IN) :: space2_id ! Second dataspace identifier 
-!            INTEGER, INTENT(IN) :: operator     ! Flag, valid values are:
-						!  H5S_SELECT_NOOP_F 
-						!  H5S_SELECT_SET_F 
-						!  H5S_SELECT_OR_F 
-						!  H5S_SELECT_AND_F 
-						!  H5S_SELECT_XOR_F 
-						!  H5S_SELECT_NOTB_F 
-						!  H5S_SELECT_NOTA_F 
-						!  H5S_SELECT_APPEND_F 
-						!  H5S_SELECT_PREPEND_F 
-                                                !  
-!            INTEGER, INTENT(OUT) :: hdferr     ! Error code
-
-!            INTERFACE
-!              INTEGER FUNCTION h5sselect_select_c(space1_id, operator, &
-!                               space2_id)
-!              USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5SSELECT_SELECT_C'::h5sselect_select_c
-              !DEC$ ENDIF
-!              INTEGER(HID_T), INTENT(INOUT) :: space1_id
-!              INTEGER(HID_T), INTENT(IN) :: space2_id
-!              INTEGER, INTENT(IN) :: operator
-!              END FUNCTION h5sselect_select_c
-!            END INTERFACE
-
-!            hdferr = h5sselect_select_c(space1_id, operator, space2_id)
-!            return
- 
-!          END SUBROUTINE h5sselect_select_f
-
 !----------------------------------------------------------------------
 ! Name:		h5sget_select_type_f 
 !
