@@ -13,16 +13,15 @@
 ! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 !
     MODULE H5GLOBAL
-         USE H5FORTRAN_TYPES
+      USE H5FORTRAN_TYPES
 !
 ! Definitions for reference datatypes.
 ! If you change the value of these parameters, do not forget to change corresponding
 ! values in the H5f90.h file. 
-        INTEGER, PARAMETER :: REF_OBJ_BUF_LEN = 2 
         INTEGER, PARAMETER :: REF_REG_BUF_LEN = 3 
 
         TYPE hobj_ref_t_f
-             INTEGER ref(REF_OBJ_BUF_LEN)  
+             INTEGER(HADDR_T) ref
         END TYPE 
 
         TYPE hdset_reg_ref_t_f
@@ -372,7 +371,7 @@
       EQUIVALENCE(H5E_flags(24), H5E_TBBT_F)
 
 !
-! H5E flags declaration
+! H5I flags declaration
 !
       INTEGER, PARAMETER :: H5I_FLAGS_LEN = 7
       INTEGER H5I_flags(H5I_FLAGS_LEN)
@@ -572,7 +571,7 @@
 !
 ! H5Z flags declaration
 !
-      INTEGER, PARAMETER :: H5Z_FLAGS_LEN = 11 
+      INTEGER, PARAMETER :: H5Z_FLAGS_LEN = 14
       INTEGER H5Z_flags(H5Z_FLAGS_LEN)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5Z_FLAGS/
@@ -581,6 +580,7 @@
 
       INTEGER :: H5Z_FILTER_ERROR_F 
       INTEGER :: H5Z_FILTER_NONE_F 
+      INTEGER :: H5Z_FILTER_ALL_F
       INTEGER :: H5Z_FILTER_DEFLATE_F 
       INTEGER :: H5Z_FILTER_SHUFFLE_F 
       INTEGER :: H5Z_FILTER_FLETCHER32_F 
@@ -590,6 +590,8 @@
       INTEGER :: H5Z_ENABLE_EDC_F
       INTEGER :: H5Z_NO_EDC_F
       INTEGER :: H5Z_FLAG_OPTIONAL_F
+      INTEGER :: H5Z_FILTER_ENCODE_ENABLED_F
+      INTEGER :: H5Z_FILTER_DECODE_ENABLED_F
 
       EQUIVALENCE(H5Z_flags(1), H5Z_FILTER_ERROR_F)
       EQUIVALENCE(H5Z_flags(2), H5Z_FILTER_NONE_F)
@@ -602,26 +604,25 @@
       EQUIVALENCE(H5Z_flags(9), H5Z_NO_EDC_F)
       EQUIVALENCE(H5Z_flags(10), H5Z_FILTER_SZIP_F)
       EQUIVALENCE(H5Z_flags(11), H5Z_FLAG_OPTIONAL_F)
+      EQUIVALENCE(H5Z_flags(12), H5Z_FILTER_ENCODE_ENABLED_F)
+      EQUIVALENCE(H5Z_flags(13), H5Z_FILTER_DECODE_ENABLED_F)
+      EQUIVALENCE(H5Z_flags(14), H5Z_FILTER_ALL_F)
 
 
 !
 ! H5 Library flags declaration
 !
-     INTEGER, PARAMETER :: H5LIB_FLAGS_LEN = 4 
+     INTEGER, PARAMETER :: H5LIB_FLAGS_LEN =  2
      INTEGER :: H5LIB_flags(H5LIB_FLAGS_LEN) 
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5LIB_FLAGS/
 !DEC$endif
      COMMON /H5LIB_FLAGS/ H5LIB_flags
-      INTEGER :: H5_SZIP_AK13_OM_F
-      INTEGER :: H5_SZIP_CHIP_OM_F
       INTEGER :: H5_SZIP_EC_OM_F
       INTEGER :: H5_SZIP_NN_OM_F
 !
-      EQUIVALENCE(H5LIB_flags(1), H5_SZIP_AK13_OM_F)
-      EQUIVALENCE(H5LIB_flags(2), H5_SZIP_CHIP_OM_F)
-      EQUIVALENCE(H5LIB_flags(3), H5_SZIP_EC_OM_F)
-      EQUIVALENCE(H5LIB_flags(4), H5_SZIP_NN_OM_F)
+      EQUIVALENCE(H5LIB_flags(1), H5_SZIP_EC_OM_F)
+      EQUIVALENCE(H5LIB_flags(2), H5_SZIP_NN_OM_F)
 
     END MODULE H5GLOBAL
       
