@@ -79,7 +79,7 @@
      !data buffers 
      !         
      INTEGER, DIMENSION(NX,NY) :: data_out
-     INTEGER, DIMENSION(7) :: data_dims
+     INTEGER(HSIZE_T), DIMENSION(2) :: data_dims
      data_dims(1) = NX
      data_dims(2) = NY
 
@@ -176,8 +176,7 @@
      CALL h5close_f(error)
          CALL check("h5close_types_f",error,total_error)
      
-     ! if errors detected, exit with non-zero code. This is not truly fortran
-     ! standard but likely supported by most fortran compilers.
-     IF (total_error .ne. 0) CALL exit (total_error)
+     ! if errors detected, exit with non-zero code.
+     IF (total_error .ne. 0) CALL h5_exit_f (1)
 
      END PROGRAM FFLUSH2EXAMPLE
