@@ -22,6 +22,7 @@
 #include "H5Dpublic.h"
 
 /* Private headers needed by this file */
+#include "H5FDprivate.h"	/* File drivers				*/
 #include "H5Oprivate.h"		/* Object headers		  	*/
 
 /*
@@ -44,9 +45,9 @@
 #define H5D_CRT_CHUNK_DIM_DEF      1
 /* Definitions for chunk size */
 #define H5D_CRT_CHUNK_SIZE_NAME    "chunk_size"
-#define H5D_CRT_CHUNK_SIZE_SIZE    sizeof(hsize_t[H5O_LAYOUT_NDIMS])
+#define H5D_CRT_CHUNK_SIZE_SIZE    sizeof(size_t[H5O_LAYOUT_NDIMS])
 #define H5D_CRT_CHUNK_SIZE_DEF     {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,\
-                                   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}
+                                   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}
 /* Definitions for fill value.  size=0 means fill value will be 0 as 
  * library default; size=-1 means fill value is undefined. */
 #define H5D_CRT_FILL_VALUE_NAME    "fill_value"
@@ -56,7 +57,10 @@
 /* Definitions for space allocation time */
 #define H5D_CRT_ALLOC_TIME_NAME   "alloc_time"
 #define H5D_CRT_ALLOC_TIME_SIZE   sizeof(H5D_alloc_time_t)
-#define H5D_CRT_ALLOC_TIME_DEF    H5D_ALLOC_TIME_DEFAULT
+#define H5D_CRT_ALLOC_TIME_DEF    H5D_ALLOC_TIME_LATE
+#define H5D_CRT_ALLOC_TIME_STATE_NAME   "alloc_time_state"
+#define H5D_CRT_ALLOC_TIME_STATE_SIZE   sizeof(unsigned)
+#define H5D_CRT_ALLOC_TIME_STATE_DEF    1
 /* Definitions for time of fill value writing */
 #define H5D_CRT_FILL_TIME_NAME     "fill_time"
 #define H5D_CRT_FILL_TIME_SIZE     sizeof(H5D_fill_time_t)
@@ -90,6 +94,9 @@
 #define H5D_XFER_BKGR_BUF_TYPE_SIZE       sizeof(H5T_bkg_t)
 #define H5D_XFER_BKGR_BUF_TYPE_DEF      H5T_BKG_NO
 /* Definitions for B-tree node splitting ratio property */
+/* (These default B-tree node splitting ratios are also used for splitting
+ * group's B-trees as well as chunked dataset's B-trees - QAK)
+ */
 #define H5D_XFER_BTREE_SPLIT_RATIO_NAME       "btree_split_ratio"
 #define H5D_XFER_BTREE_SPLIT_RATIO_SIZE       sizeof(double[3])
 #define H5D_XFER_BTREE_SPLIT_RATIO_DEF      {0.1, 0.5, 0.9}
@@ -132,6 +139,9 @@
 #define H5D_XFER_VFL_INFO_SIZE       sizeof(void *)
 #define H5D_XFER_VFL_INFO_DEF  NULL
 /* Definitions for hyperslab vector size property */
+/* (Be cautious about increasing the default size, there are arrays allocated
+ *      on the stack which depend on it - QAK)
+ */
 #define H5D_XFER_HYPER_VECTOR_SIZE_NAME       "vec_size"
 #define H5D_XFER_HYPER_VECTOR_SIZE_SIZE       sizeof(size_t)
 #define H5D_XFER_HYPER_VECTOR_SIZE_DEF        1024
@@ -140,13 +150,19 @@
 #define H5D_XFER_IO_XFER_MODE_SIZE       sizeof(H5FD_mpio_xfer_t)
 #define H5D_XFER_IO_XFER_MODE_DEF        H5FD_MPIO_INDEPENDENT
 /* Definitions for EDC property */
-#define H5D_XFER_EDC_NAME       "error-detecting"
+#define H5D_XFER_EDC_NAME       "err_detect"
 #define H5D_XFER_EDC_SIZE       sizeof(H5Z_EDC_t)
 #define H5D_XFER_EDC_DEF        H5Z_ENABLE_EDC
 /* Definitions for filter callback function property */
 #define H5D_XFER_FILTER_CB_NAME       "filter_cb"
 #define H5D_XFER_FILTER_CB_SIZE       sizeof(H5Z_cb_t)
 #define H5D_XFER_FILTER_CB_DEF        {NULL,NULL}
+#ifdef H5_HAVE_INSTRUMENTED_LIBRARY
+/* Definitions for collective chunk I/O property */
+#define H5D_XFER_COLL_CHUNK_NAME       "coll_chunk"
+#define H5D_XFER_COLL_CHUNK_SIZE       sizeof(unsigned)
+#define H5D_XFER_COLL_CHUNK_DEF        1
+#endif /* H5_HAVE_INSTRUMENTED_LIBRARY */
 
 /****************************/
 /* Library Private Typedefs */
@@ -156,17 +172,51 @@
 typedef struct H5D_t H5D_t;
 
 /* Typedef for dataset storage information */
+typedef struct {
+    hsize_t index;          /* "Index" of chunk in dataset (must be first for TBBT routines) */
+    hsize_t *offset;        /* Chunk's coordinates in elements */
+} H5D_chunk_storage_t;
+
+typedef struct {
+    haddr_t dset_addr;      /* Address of dataset in file */
+    hsize_t dset_size;      /* Total size of dataset in file */
+} H5D_contig_storage_t;
+
 typedef union H5D_storage_t {
     H5O_efl_t   efl;            /* External file list information for dataset */
-    hssize_t    *chunk_coords;  /* chunk's coordinates in file chunks */
+    H5D_chunk_storage_t chunk;  /* Chunk information for dataset */
+    H5D_contig_storage_t contig; /* Contiguous information for dataset */
 } H5D_storage_t;
 
+/* Typedef for cached dataset transfer property list information */
+typedef struct H5D_dxpl_cache_t {
+    size_t max_temp_buf;        /* Maximum temporary buffer size (H5D_XFER_MAX_TEMP_BUF_NAME) */
+    void *tconv_buf;            /* Temporary conversion buffer (H5D_XFER_TCONV_BUF_NAME) */
+    void *bkgr_buf;             /* Background conversion buffer (H5D_XFER_BKGR_BUF_NAME) */
+    H5T_bkg_t bkgr_buf_type;    /* Background buffer type (H5D_XFER_BKGR_BUF_NAME) */
+    double btree_split_ratio[3];/* B-tree split ratios (H5D_XFER_BTREE_SPLIT_RATIO_NAME) */
+    size_t vec_size;            /* Size of hyperslab vector (H5D_XFER_HYPER_VECTOR_SIZE_NAME) */
+#ifdef H5_HAVE_PARALLEL
+    H5FD_mpio_xfer_t xfer_mode; /* Parallel transfer for this request (H5D_XFER_IO_XFER_MODE_NAME) */
+#endif /*H5_HAVE_PARALLEL*/
+    H5Z_EDC_t err_detect;       /* Error detection info (H5D_XFER_EDC_NAME) */
+    H5Z_cb_t filter_cb;         /* Filter callback function (H5D_XFER_FILTER_CB_NAME) */
+} H5D_dxpl_cache_t;
+
+/* Typedef for cached dataset creation property list information */
+typedef struct H5D_dcpl_cache_t {
+    H5O_pline_t pline;          /* I/O pipeline info (H5D_CRT_DATA_PIPELINE_NAME) */
+    H5O_fill_t fill;            /* Fill value info (H5D_CRT_FILL_VALUE_NAME) */
+    H5D_fill_time_t fill_time;  /* Fill time (H5D_CRT_FILL_TIME_NAME) */
+} H5D_dcpl_cache_t;
+
 /* Library-private functions defined in H5D package */
 H5_DLL herr_t H5D_init(void);
-H5_DLL hid_t H5D_open(H5G_entry_t *ent, hid_t dxpl_id);
+H5_DLL H5D_t *H5D_open(const H5G_entry_t *ent, hid_t dxpl_id);
+H5_DLL herr_t H5D_close(H5D_t *dataset);
 H5_DLL htri_t H5D_isa(H5G_entry_t *ent, hid_t dxpl_id);
 H5_DLL H5G_entry_t *H5D_entof(H5D_t *dataset);
-H5_DLL H5T_t *H5D_typeof(H5D_t *dset);
+H5_DLL H5T_t *H5D_typeof(const H5D_t *dset);
 H5_DLL herr_t H5D_crt_copy(hid_t new_plist_t, hid_t old_plist_t, 
                             void *copy_data);
 H5_DLL herr_t H5D_crt_close(hid_t dxpl_id, void *close_data);
@@ -174,6 +224,18 @@ H5_DLL herr_t H5D_xfer_create(hid_t dxpl_id, void *create_data);
 H5_DLL herr_t H5D_xfer_copy(hid_t new_plist_id, hid_t old_plist_id, 
                              void *copy_data);
 H5_DLL herr_t H5D_xfer_close(hid_t dxpl_id, void *close_data);
-H5_DLL herr_t H5D_flush(H5F_t *f, hid_t dxpl_id);
+H5_DLL herr_t H5D_flush(const H5F_t *f, hid_t dxpl_id, unsigned flags);
+H5_DLL herr_t H5D_get_dxpl_cache(hid_t dxpl_id, H5D_dxpl_cache_t **cache);
+H5_DLL herr_t H5D_get_dxpl_cache_real(hid_t dxpl_id, H5D_dxpl_cache_t *cache);
+
+/* Functions that operate on contiguous storage */
+H5_DLL herr_t H5D_contig_delete(H5F_t *f, hid_t dxpl_id,
+    const H5O_layout_t *layout);
+
+/* Functions that operate on indexed storage */
+H5_DLL herr_t H5D_istore_delete(H5F_t *f, hid_t dxpl_id,
+    const H5O_layout_t *layout);
+H5_DLL herr_t H5D_istore_debug(H5F_t *f, hid_t dxpl_id, haddr_t addr, FILE * stream,
+				int indent, int fwidth, unsigned ndims);
 
 #endif
