@@ -29,12 +29,11 @@
 #include "H5Opkg.h"             /* Object header functions                 */
 #include "H5Pprivate.h"		/* Property lists			*/
 
-#define PABLO_MASK	H5O_fill_mask
 
 static void  *H5O_fill_new_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p, H5O_shared_t *sh);
 static herr_t H5O_fill_new_encode(H5F_t *f, uint8_t *p, const void *_mesg);
-static void  *H5O_fill_new_copy(const void *_mesg, void *_dest);
-static size_t H5O_fill_new_size(H5F_t *f, const void *_mesg);
+static void  *H5O_fill_new_copy(const void *_mesg, void *_dest, unsigned update_flags);
+static size_t H5O_fill_new_size(const H5F_t *f, const void *_mesg);
 static herr_t H5O_fill_new_reset(void *_mesg);
 static herr_t H5O_fill_new_free(void *_mesg);
 static herr_t H5O_fill_new_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE *stream,
@@ -42,8 +41,8 @@ static herr_t H5O_fill_new_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FIL
 
 static void  *H5O_fill_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p, H5O_shared_t *sh);
 static herr_t H5O_fill_encode(H5F_t *f, uint8_t *p, const void *_mesg);
-static void  *H5O_fill_copy(const void *_mesg, void *_dest);
-static size_t H5O_fill_size(H5F_t *f, const void *_mesg);
+static void  *H5O_fill_copy(const void *_mesg, void *_dest, unsigned update_flags);
+static size_t H5O_fill_size(const H5F_t *f, const void *_mesg);
 static herr_t H5O_fill_reset(void *_mesg);
 static herr_t H5O_fill_free(void *_mesg);
 static herr_t H5O_fill_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE *stream,
@@ -85,15 +84,11 @@ const H5O_class_t H5O_FILL_NEW[1] = {{
     H5O_fill_new_debug,		/*debug the message			*/
 }};
 
-/* Initial version of the "new" fill value information */
+/* Initial version of the "old" fill value information */
 #define H5O_FILL_VERSION 	1
 /* Revised version of the "new" fill value information */
 #define H5O_FILL_VERSION_2 	2	
  
-/* Interface initialization */
-static int interface_initialize_g = 0;
-#define INTERFACE_INIT NULL
-
 /* Declare a free list to manage the H5O_fill_new_t struct */
 H5FL_DEFINE(H5O_fill_new_t);
 
@@ -127,7 +122,7 @@ H5O_fill_new_decode(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const uint8_t *p,
     int			version;
     void		*ret_value;
     
-    FUNC_ENTER_NOAPI(H5O_fill_new_decode, NULL);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_fill_new_decode);
 
     assert(f);
     assert(p);
@@ -150,9 +145,9 @@ H5O_fill_new_decode(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const uint8_t *p,
     /* Whether fill value is defined */
     mesg->fill_defined = *p++;
 
-    /* Check for version of fill value message */
-    if(version==H5O_FILL_VERSION) {
-        UINT32DECODE(p, mesg->size);
+    /* Only decode fill value information if one is defined */
+    if(mesg->fill_defined) {
+        INT32DECODE(p, mesg->size);
         if (mesg->size>0) {
             H5_CHECK_OVERFLOW(mesg->size,ssize_t,size_t);
             if (NULL==(mesg->buf=H5MM_malloc((size_t)mesg->size)))
@@ -160,22 +155,9 @@ H5O_fill_new_decode(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const uint8_t *p,
             HDmemcpy(mesg->buf, p, (size_t)mesg->size);
         }
     } /* end if */
-    else {
-        /* Only decode fill value information if one is defined */
-        if(mesg->fill_defined) {
-            UINT32DECODE(p, mesg->size);
-            if (mesg->size>0) {
-                H5_CHECK_OVERFLOW(mesg->size,ssize_t,size_t);
-                if (NULL==(mesg->buf=H5MM_malloc((size_t)mesg->size)))
-                    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for fill value");
-                HDmemcpy(mesg->buf, p, (size_t)mesg->size);
-            }
-        } /* end if */
-        else
-            mesg->size=(-1);
-    } /* end else */
-    
-    
+    else
+        mesg->size=(-1);
+
     /* Set return value */
     ret_value = (void*)mesg;
     
@@ -213,7 +195,7 @@ H5O_fill_decode(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const uint8_t *p,
     H5O_fill_t  *mesg=NULL;
     void        *ret_value;
     
-    FUNC_ENTER_NOAPI(H5O_fill_decode, NULL);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_fill_decode);
 
     assert(f);
     assert(p);
@@ -262,9 +244,8 @@ static herr_t
 H5O_fill_new_encode(H5F_t UNUSED *f, uint8_t *p, const void *_mesg)
 {
     const H5O_fill_new_t	*mesg = (const H5O_fill_new_t *)_mesg;
-    herr_t ret_value=SUCCEED;   /* Return value */
     
-    FUNC_ENTER_NOAPI(H5O_fill_new_encode, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_new_encode);
 
     assert(f);
     assert(p);
@@ -280,15 +261,14 @@ H5O_fill_new_encode(H5F_t UNUSED *f, uint8_t *p, const void *_mesg)
     *p++ = mesg->fill_defined;
 
     /* Does it handle undefined fill value? */
-    UINT32ENCODE(p, mesg->size);
+    INT32ENCODE(p, mesg->size);
     if(mesg->size>0)
         if(mesg->buf) {
             H5_CHECK_OVERFLOW(mesg->size,ssize_t,size_t);
             HDmemcpy(p, mesg->buf, (size_t)mesg->size);
         } /* end if */
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -310,9 +290,8 @@ static herr_t
 H5O_fill_encode(H5F_t UNUSED *f, uint8_t *p, const void *_mesg)
 {
     const H5O_fill_t    *mesg = (const H5O_fill_t *)_mesg;
-    herr_t ret_value=SUCCEED;   /* Return value */
-    
-    FUNC_ENTER_NOAPI(H5O_fill_encode, FAIL);
+
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_encode);
 
     assert(f);
     assert(p);
@@ -322,8 +301,7 @@ H5O_fill_encode(H5F_t UNUSED *f, uint8_t *p, const void *_mesg)
     if(mesg->buf)
         HDmemcpy(p, mesg->buf, mesg->size);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -347,13 +325,13 @@ done:
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_fill_new_copy(const void *_mesg, void *_dest)
+H5O_fill_new_copy(const void *_mesg, void *_dest, unsigned UNUSED update_flags)
 {
     const H5O_fill_new_t	*mesg = (const H5O_fill_new_t *)_mesg;
     H5O_fill_new_t		*dest = (H5O_fill_new_t *)_dest;
     void		*ret_value;
 
-    FUNC_ENTER_NOAPI(H5O_fill_new_copy, NULL);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_fill_new_copy);
 
     assert(mesg);
 
@@ -419,13 +397,13 @@ done:
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_fill_copy(const void *_mesg, void *_dest)
+H5O_fill_copy(const void *_mesg, void *_dest, unsigned UNUSED update_flags)
 {
     const H5O_fill_t    *mesg = (const H5O_fill_t *)_mesg;
     H5O_fill_t          *dest = (H5O_fill_t *)_dest;
     void                *ret_value;
 
-    FUNC_ENTER_NOAPI(H5O_fill_copy, NULL);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_fill_copy);
 
     assert(mesg);
 
@@ -486,12 +464,12 @@ done:
  *-------------------------------------------------------------------------
  */
 static size_t
-H5O_fill_new_size(H5F_t UNUSED *f, const void *_mesg)
+H5O_fill_new_size(const H5F_t UNUSED *f, const void *_mesg)
 {
     const H5O_fill_new_t	*mesg = (const H5O_fill_new_t *)_mesg;
     size_t			ret_value;
  
-    FUNC_ENTER_NOAPI(H5O_fill_new_size, 0);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_new_size);
 
     assert(f);
     assert(mesg);
@@ -503,7 +481,6 @@ H5O_fill_new_size(H5F_t UNUSED *f, const void *_mesg)
 		4 +		/* Fill value size	 */
 		(mesg->size>0 ? mesg->size : 0);	/* Size of fill value	 */
 
-done:
     FUNC_LEAVE_NOAPI(ret_value);
 }
 
@@ -527,21 +504,16 @@ done:
  *-------------------------------------------------------------------------
  */
 static size_t
-H5O_fill_size(H5F_t UNUSED *f, const void *_mesg)
+H5O_fill_size(const H5F_t UNUSED *f, const void *_mesg)
 {
     const H5O_fill_t    *mesg = (const H5O_fill_t *)_mesg;
-    size_t ret_value;           /* Return value */
     
-    FUNC_ENTER_NOAPI(H5O_fill_size, 0);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_size);
 
     assert(f);
     assert(mesg);
 
-    /* Set return value */
-    ret_value=4+mesg->size;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(4+mesg->size);
 }
 
 
@@ -565,9 +537,8 @@ static herr_t
 H5O_fill_new_reset(void *_mesg)
 {
     H5O_fill_new_t	*mesg = (H5O_fill_new_t *)_mesg;
-    herr_t ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_fill_new_reset, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_new_reset);
 
     assert(mesg);
     
@@ -582,8 +553,7 @@ H5O_fill_new_reset(void *_mesg)
     mesg->fill_time    = (H5D_fill_time_t)0;
     mesg->fill_defined = FALSE;
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -605,9 +575,8 @@ static herr_t
 H5O_fill_reset(void *_mesg)
 {
     H5O_fill_t  *mesg = (H5O_fill_t *)_mesg;
-    herr_t ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_fill_reset, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_reset);
 
     assert(mesg);
     
@@ -619,8 +588,7 @@ H5O_fill_reset(void *_mesg)
         mesg->type = NULL;
     }
     
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -641,16 +609,13 @@ done:
 static herr_t
 H5O_fill_new_free (void *mesg)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5O_fill_new_free, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_new_free);
 
     assert (mesg);
 
     H5FL_FREE(H5O_fill_new_t,mesg);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -671,16 +636,13 @@ done:
 static herr_t
 H5O_fill_free (void *mesg)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5O_fill_free, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_free);
 
     assert (mesg);
 
     H5FL_FREE(H5O_fill_t,mesg);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -704,9 +666,8 @@ H5O_fill_new_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FIL
 {
     const H5O_fill_new_t	*mesg = (const H5O_fill_new_t *)_mesg;
     H5D_fill_value_t fill_status;       /* Whether the fill value is defined */
-    herr_t ret_value=SUCCEED;   /* Return value */
     
-    FUNC_ENTER_NOAPI(H5O_fill_new_debug, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_new_debug);
 
     assert(f);
     assert(mesg);
@@ -784,8 +745,7 @@ H5O_fill_new_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FIL
 	fprintf(stream, "<dataset type>\n");
     }
     
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -808,9 +768,8 @@ H5O_fill_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE *s
 	       int indent, int fwidth)
 {
     const H5O_fill_t	*mesg = (const H5O_fill_t *)_mesg;
-    herr_t ret_value=SUCCEED;   /* Return value */
     
-    FUNC_ENTER_NOAPI(H5O_fill_debug, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_fill_debug);
 
     assert(f);
     assert(mesg);
@@ -828,8 +787,7 @@ H5O_fill_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE *s
 	fprintf(stream, "<dataset type>\n");
     }
     
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
 
@@ -856,10 +814,10 @@ H5O_fill_convert(void *_fill, H5T_t *dset_type, hid_t dxpl_id)
     H5O_fill_new_t	*fill = _fill;
     H5T_path_t		*tpath=NULL;		/*type conversion info	*/
     void		*buf=NULL, *bkg=NULL;	/*conversion buffers	*/
-    hid_t		src_id=-1, dst_id=-1;	/*data type identifiers	*/
+    hid_t		src_id=-1, dst_id=-1;	/*datatype identifiers	*/
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_fill_convert, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_fill_convert);
 
     assert(fill);
     assert(dset_type);
@@ -875,10 +833,9 @@ H5O_fill_convert(void *_fill, H5T_t *dset_type, hid_t dxpl_id)
     /*
      * Can we convert between source and destination data types?
      */
-    if (NULL==(tpath=H5T_path_find(fill->type, dset_type, NULL, NULL, dxpl_id))) {
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL,
-		    "unable to convert between src and dst data types");
-    }
+    if (NULL==(tpath=H5T_path_find(fill->type, dset_type, NULL, NULL, dxpl_id)))
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to convert between src and dst datatypes")
+
     /* Don't bother doing anything if there will be no actual conversion */
     if (!H5T_path_noop(tpath)) {
         if ((src_id = H5I_register(H5I_DATATYPE,
@@ -888,7 +845,7 @@ H5O_fill_convert(void *_fill, H5T_t *dset_type, hid_t dxpl_id)
             HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to copy/register data type");
 
         /*
-         * Data type conversions are always done in place, so we need a buffer
+         * Datatype conversions are always done in place, so we need a buffer
          * that is large enough for both source and destination.
          */
         if (H5T_get_size(fill->type)>=H5T_get_size(dset_type)) {
@@ -902,8 +859,8 @@ H5O_fill_convert(void *_fill, H5T_t *dset_type, hid_t dxpl_id)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed for type conversion");
 
         /* Do the conversion */
-        if (H5T_convert(tpath, src_id, dst_id, (hsize_t)1, 0, 0, buf, bkg, dxpl_id)<0)
-            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "data type conversion failed");
+        if (H5T_convert(tpath, src_id, dst_id, 1, 0, 0, buf, bkg, dxpl_id)<0)
+            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "datatype conversion failed");
 
         /* Update the fill message */
         if (buf!=fill->buf) {
