@@ -12,10 +12,11 @@
  * access to either file, you may request a copy from hdfhelp@ncsa.uiuc.edu. *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-/* $Id: H5R.c,v 1.66.2.1 2003/12/06 21:39:22 koziol Exp $ */
-
 #define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
-#define H5S_PACKAGE		/*suppress error about including H5Spkg	  */
+
+/* Interface initialization */
+#define H5_INTERFACE_INIT_FUNC	H5R_init_interface
+
 
 #include "H5private.h"		/* Generic Functions */
 #include "H5Iprivate.h"		/* ID Functions */
@@ -26,15 +27,9 @@
 #include "H5HGprivate.h"    /* Global Heaps */
 #include "H5MMprivate.h"    /* Memory Management */
 #include "H5Rprivate.h"		/* References */
-#include "H5Spkg.h"		/* Dataspaces */
+#include "H5Sprivate.h"		/* Dataspace functions			*/
 #include "H5Tprivate.h"		/* Datatypes */
 
-/* Interface initialization */
-#define PABLO_MASK	H5R_mask
-#define INTERFACE_INIT	H5R_init_interface
-static int		interface_initialize_g = 0;
-static herr_t		H5R_init_interface(void);
-
 /* Static functions */
 static herr_t H5R_create(void *ref, H5G_entry_t *loc, const char *name,
         H5R_type_t ref_type, H5S_t *space, hid_t dxpl_id);
@@ -96,12 +91,12 @@ H5R_term_interface(void)
 
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5R_term_interface);
     
-    if (interface_initialize_g) {
+    if (H5_interface_initialize_g) {
 	if ((n=H5I_nmembers(H5I_REFERENCE))) {
 	    H5I_clear_group(H5I_REFERENCE, FALSE);
 	} else {
 	    H5I_destroy_group(H5I_REFERENCE);
-	    interface_initialize_g = 0;
+	    H5_interface_initialize_g = 0;
 	    n = 1; /*H5I*/
 	}
     }
@@ -161,7 +156,7 @@ H5R_create(void *_ref, H5G_entry_t *loc, const char *name, H5R_type_t ref_type,
             uint8_t *p;       /* Pointer to OID to store */
 
             /* Set information for reference */
-            p=(uint8_t *)ref->oid;
+            p=(uint8_t *)ref;
             H5F_addr_pack(loc->file,&addr,&sb.objno[0]);
             H5F_addr_encode(loc->file,&p,addr);
             break;
@@ -183,8 +178,8 @@ H5R_create(void *_ref, H5G_entry_t *loc, const char *name, H5R_type_t ref_type,
             /* Return any previous heap block to the free list if we are garbage collecting */
             if(loc->file->shared->gc_ref) {
                 /* Check for an existing heap ID in the reference */
-                for(u=0, heapid_found=0; u<H5R_DSET_REG_REF_BUF_SIZE; u++)
-                    if(ref->heapid[u]!=0) {
+                for(u=0, heapid_found=0, p=(uint8_t *)ref; u<H5R_DSET_REG_REF_BUF_SIZE; u++)
+                    if(p[u]!=0) {
                         heapid_found=1;
                         break;
                     } /* end if */
@@ -195,10 +190,10 @@ H5R_create(void *_ref, H5G_entry_t *loc, const char *name, H5R_type_t ref_type,
             } /* end if */
 
             /* Zero the heap ID out, may leak heap space if user is re-using reference and doesn't have garbage collection on */
-            HDmemset(ref->heapid,H5R_DSET_REG_REF_BUF_SIZE,0);
+            HDmemset(ref,H5R_DSET_REG_REF_BUF_SIZE,0);
 
             /* Get the amount of space required to serialize the selection */
-            if ((buf_size = (*space->select.serial_size)(space)) < 0)
+            if ((buf_size = H5S_SELECT_SERIAL_SIZE(space)) < 0)
                 HGOTO_ERROR(H5E_REFERENCE, H5E_CANTINIT, FAIL, "Invalid amount of space for serializing selection");
 
             /* Increase buffer size to allow for the dataset OID */
@@ -215,7 +210,7 @@ H5R_create(void *_ref, H5G_entry_t *loc, const char *name, H5R_type_t ref_type,
             H5F_addr_encode(loc->file,&p,addr);
 
             /* Serialize the selection */
-            if ((*space->select.serialize)(space,p) < 0)
+            if (H5S_SELECT_SERIALIZE(space,p) < 0)
                 HGOTO_ERROR(H5E_REFERENCE, H5E_CANTCOPY, FAIL, "Unable to serialize selection");
 
             /* Save the serialized buffer for later */
@@ -224,7 +219,7 @@ H5R_create(void *_ref, H5G_entry_t *loc, const char *name, H5R_type_t ref_type,
                 HGOTO_ERROR(H5E_REFERENCE, H5E_WRITEERROR, FAIL, "Unable to serialize selection");
 
             /* Serialize the heap ID and index for storage in the file */
-            p=(uint8_t *)ref->heapid;
+            p=(uint8_t *)ref;
             H5F_addr_encode(loc->file,&p,hobjid.addr);
             INT32ENCODE(p,hobjid.idx);
 
@@ -333,8 +328,9 @@ done:
 static hid_t
 H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
 {
+    H5D_t *dset;                /* Pointer to dataset to open */
+    H5T_t *type;                /* Pointer to datatype to open */
     H5G_t *group;               /* Pointer to group to open */
-    H5T_t *datatype;            /* Pointer to datatype to open */
     H5G_entry_t ent;            /* Symbol table entry */
     uint8_t *p;                 /* Pointer to OID to store */
     int oid_type;               /* type of object being dereferenced */
@@ -360,7 +356,7 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
              *  open a dataset for now
              */
             /* Get the object oid */
-            p=(uint8_t *)ref->oid;
+            p=(uint8_t *)ref;
             H5F_addr_decode(ent.file,(const uint8_t **)&p,&(ent.header));
         } /* end case */
         break;
@@ -372,7 +368,7 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
             uint8_t *buf;   /* Buffer to store serialized selection in */
 
             /* Get the heap ID for the dataset region */
-            p=(uint8_t *)ref->heapid;
+            p=(uint8_t *)ref;
             H5F_addr_decode(ent.file,(const uint8_t **)&p,&(hobjid.addr));
             INT32DECODE(p,hobjid.idx);
 
@@ -403,14 +399,14 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
     if(H5O_link(&ent,0,dxpl_id)<=0)
         HGOTO_ERROR(H5E_REFERENCE, H5E_LINKCOUNT, FAIL, "dereferencing deleted object");
 
-    /* Open the dataset object */
+    /* Open the object */
     oid_type=H5G_get_type(&ent,dxpl_id);
     switch(oid_type) {
         case H5G_GROUP:
-            if ((group=H5G_open_oid(&ent,dxpl_id)) == NULL)
+            if ((group=H5G_open(&ent,dxpl_id)) == NULL)
                 HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "not found");
 
-            /* Create an atom for the dataset */
+            /* Create an atom for the group */
             if ((ret_value = H5I_register(H5I_GROUP, group)) < 0) {
                 H5G_close(group);
                 HGOTO_ERROR(H5E_SYM, H5E_CANTREGISTER, FAIL, "can't register group");
@@ -418,20 +414,26 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
             break;
 
         case H5G_TYPE:
-            if ((datatype=H5T_open_oid(&ent, dxpl_id)) == NULL)
+            if ((type=H5T_open(&ent, dxpl_id)) == NULL)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "not found");
 
-            /* Create an atom for the dataset */
-            if ((ret_value = H5I_register(H5I_DATATYPE, datatype)) < 0) {
-                H5T_close(datatype);
-                HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "can't register group");
+            /* Create an atom for the datatype */
+            if ((ret_value = H5I_register(H5I_DATATYPE, type)) < 0) {
+                H5T_close(type);
+                HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "can't register datatype");
             }
             break;
 
         case H5G_DATASET:
             /* Open the dataset */
-            if ((ret_value=H5D_open(&ent,dxpl_id)) < 0)
+            if ((dset=H5D_open(&ent,dxpl_id))==NULL)
                 HGOTO_ERROR(H5E_DATASET, H5E_NOTFOUND, FAIL, "not found");
+
+            /* Create an atom for the dataset */
+            if ((ret_value = H5I_register(H5I_DATASET, dset)) < 0) {
+                H5D_close(dset);
+                HGOTO_ERROR(H5E_DATASET, H5E_CANTREGISTER, FAIL, "can't register dataset");
+            }
             break;
 
         default:
@@ -539,7 +541,7 @@ H5R_get_region(H5F_t *file, hid_t dxpl_id, H5R_type_t UNUSED ref_type, void *_re
     ent.file=file;
 
     /* Get the heap ID for the dataset region */
-    p=(uint8_t *)ref->heapid;
+    p=(uint8_t *)ref;
     H5F_addr_decode(ent.file,(const uint8_t **)&p,&(hobjid.addr));
     INT32DECODE(p,hobjid.idx);
 
@@ -655,7 +657,7 @@ H5R_get_object_type(H5F_t *file, hid_t dxpl_id, void *_ref)
     uint8_t *p;                 /* Pointer to OID to store */
     int ret_value;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_get_object_type);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5R_get_object_type);
 
     assert(ref);
     assert(file);
@@ -666,7 +668,7 @@ H5R_get_object_type(H5F_t *file, hid_t dxpl_id, void *_ref)
     ent.file=file;
 
     /* Get the object oid */
-    p=(uint8_t *)ref->oid;
+    p=(uint8_t *)ref;
     H5F_addr_decode(ent.file,(const uint8_t **)&p,&(ent.header));
 
     /* Get the OID type */
@@ -775,7 +777,7 @@ H5R_get_obj_type(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
             hobj_ref_t *ref=(hobj_ref_t *)_ref; /* Only object references currently supported */
 
             /* Get the object oid */
-            p=(uint8_t *)ref->oid;
+            p=(uint8_t *)ref;
             H5F_addr_decode(ent.file,(const uint8_t **)&p,&(ent.header));
         } /* end case */
         break;
@@ -787,7 +789,7 @@ H5R_get_obj_type(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, void *_ref)
             uint8_t *buf;   /* Buffer to store serialized selection in */
 
             /* Get the heap ID for the dataset region */
-            p=(uint8_t *)ref->heapid;
+            p=(uint8_t *)ref;
             H5F_addr_decode(ent.file,(const uint8_t **)&p,&(hobjid.addr));
             INT32DECODE(p,hobjid.idx);
 
@@ -880,7 +882,11 @@ H5Rget_obj_type(hid_t id, H5R_type_t ref_type, void *_ref)
     file=loc->file;
 
     /* Get the object information */
-    if ((ret_value=H5R_get_obj_type(file,H5AC_ind_dxpl_id,ref_type,_ref))<0)
+    if ((ret_value=
+#ifndef H5_WANT_H5_V1_4_COMPAT
+		(H5G_obj_t)
+#endif /* H5_WANT_H5_V1_4_COMPAT */
+		H5R_get_obj_type(file,H5AC_ind_dxpl_id,ref_type,_ref))<0)
 	HGOTO_ERROR(H5E_REFERENCE, H5E_CANTINIT, H5G_UNKNOWN, "unable to determine object type");
     
 done:
