@@ -21,34 +21,20 @@
  *		I didn't make them portable.
  */
 
-#define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
 #define H5S_PACKAGE		/*suppress error about including H5Spkg	  */
 
-#include "H5private.h"          /* Internal types, etc. */
-#include "H5Eprivate.h"         /* Error reporting */
-#include "H5Fpkg.h"             /* Ugly, but necessary for the MPIO I/O accesses */
-#include "H5FDmpio.h"		/* MPIO file driver			*/
-#include "H5FDprivate.h"        /* Necessary for the H5FD_write & H5FD_read prototypes.. */
-#include "H5Iprivate.h"		/* Object IDs */
-#include "H5Pprivate.h"		/* Property Lists */
-#include "H5Spkg.h"             /* Dataspaces */
 
-#ifndef H5_HAVE_PARALLEL
-/* 
- * The H5S_mpio_xxxx functions are for parallel I/O only and are
- * valid only when H5_HAVE_PARALLEL is #defined.  This empty #ifndef
- * body is used to allow this source file be included in the serial
- * distribution.
- * Some compilers/linkers may complain about "empty" object file.
- * If that happens, uncomment the following statement to pacify
- * them.
- */
-/* const hbool_t H5S_mpio_avail = FALSE; */
-#else /* H5_HAVE_PARALLEL */
-/* Interface initialization */
-#define PABLO_MASK      H5Sall_mask
-#define INTERFACE_INIT  NULL
-static int             interface_initialize_g = 0;
+#include "H5private.h"		/* Generic Functions			*/
+#include "H5Dprivate.h"		/* Datasets				*/
+#include "H5Eprivate.h"		/* Error handling		  	*/
+#include "H5Fprivate.h"		/* File access				*/
+#include "H5FDprivate.h"	/* File drivers				*/
+#include "H5Iprivate.h"		/* IDs			  		*/
+#include "H5Oprivate.h"		/* Object headers		  	*/
+#include "H5Pprivate.h"         /* Property lists                       */
+#include "H5Spkg.h"		/* Dataspaces 				*/
+
+#ifdef H5_HAVE_PARALLEL
 
 static herr_t
 H5S_mpio_all_type( const H5S_t *space, size_t elmt_size,
@@ -71,17 +57,18 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
 		     size_t *count,
 		     hsize_t *extra_offset,
 		     hbool_t *is_derived_type );
+
 static herr_t
-H5S_mpio_space_type( const H5S_t *space, size_t elmt_size,
-		     /* out: */
-		     MPI_Datatype *new_type,
-		     size_t *count,
-		     hsize_t *extra_offset,
-		     hbool_t *is_derived_type );
-static herr_t
-H5S_mpio_spaces_xfer(H5F_t *f, const H5O_layout_t *layout, size_t elmt_size,
-                     const H5S_t *file_space, const H5S_t *mem_space,
-                     hid_t dxpl_id, void *buf/*out*/, hbool_t do_write);
+H5S_mpio_span_hyper_type( const H5S_t *space, size_t elmt_size,
+			  /* out: */
+			  MPI_Datatype *new_type,
+			  size_t *count,
+			  hsize_t *extra_offset,
+			  hbool_t *is_derived_type );
+
+static herr_t obtain_datatype(const hsize_t size[], 
+                              H5S_hyper_span_t* span,MPI_Datatype *span_type,
+                              size_t elmt_size,int dimindex);
 
 
 /*-------------------------------------------------------------------------
@@ -115,17 +102,21 @@ H5S_mpio_all_type( const H5S_t *space, size_t elmt_size,
 		     hbool_t *is_derived_type )
 {
     hsize_t	total_bytes;
-    unsigned		u;
+    hssize_t	snelmts;                /*total number of elmts	(signed) */
+    hsize_t	nelmts;                 /*total number of elmts	*/
+    herr_t		ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_mpio_all_type);
+    FUNC_ENTER_NOAPI_NOINIT(H5S_mpio_all_type);
 
     /* Check args */
     assert (space);
 
     /* Just treat the entire extent as a block of bytes */
-    total_bytes = (hsize_t)elmt_size;
-    for (u=0; u<space->extent.u.simple.rank; ++u)
-        total_bytes *= space->extent.u.simple.size[u];
+    if((snelmts = H5S_GET_EXTENT_NPOINTS(space))<0)
+	HGOTO_ERROR (H5E_ARGS, H5E_BADVALUE, FAIL, "src dataspace has invalid selection")
+    H5_ASSIGN_OVERFLOW(nelmts,snelmts,hssize_t,hsize_t);
+
+    total_bytes = (hsize_t)elmt_size*nelmts;
 
     /* fill in the return values */
     *new_type = MPI_BYTE;
@@ -133,7 +124,8 @@ H5S_mpio_all_type( const H5S_t *space, size_t elmt_size,
     *extra_offset = 0;
     *is_derived_type = 0;
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
+done:
+    FUNC_LEAVE_NOAPI(ret_value);
 } /* H5S_mpio_all_type() */
 
 
@@ -198,9 +190,13 @@ H5S_mpio_none_type( const H5S_t UNUSED *space, size_t UNUSED elmt_size,
  *		akc, rky 2000-11-16 Replaced hard coded dimension size with
  *		    H5S_MAX_RANK.
  *
- *      Quincey Koziol, June 18, 2002
- *      Added 'extra_offset' parameter.  Also accomodate
- *      selection offset in MPI type built.
+ *      	Quincey Koziol, June 18, 2002
+ *      	Added 'extra_offset' parameter.  Also accomodate selection
+ *      	offset in MPI type built.
+ *
+ *      	Albert Cheng, August 4, 2004
+ *      	Reimplemented the algorithm of forming the outer_type by
+ *      	defining it as (start, vector, extent) in one call.
  *
  *-------------------------------------------------------------------------
  */
@@ -228,9 +224,9 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
     int			max_xtent[H5S_MAX_RANK];
     H5S_hyper_dim_t	*diminfo;		/* [rank] */
     int		rank;
-    int			block_length[2];
-    MPI_Datatype	inner_type, outer_type, old_type[2];
-    MPI_Aint            extent_len, displacement[2];
+    int			block_length[3];
+    MPI_Datatype	inner_type, outer_type, old_types[3];
+    MPI_Aint            extent_len, displacement[3];
     int                 mpi_code;               /* MPI return code */
     herr_t		ret_value = SUCCEED;
 
@@ -254,7 +250,7 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
     /* make a local copy of the dimension info so we can operate with them */
 
     /* Check if this is a "flattened" regular hyperslab selection */
-    if(sel_iter.u.hyp.iter_rank!=0 && sel_iter.u.hyp.iter_rank<space->extent.u.simple.rank) {
+    if(sel_iter.u.hyp.iter_rank!=0 && sel_iter.u.hyp.iter_rank<space->extent.rank) {
         /* Flattened selection */
         rank=sel_iter.u.hyp.iter_rank;
         assert (rank >= 0 && rank<=H5S_MAX_RANK);	/* within array bounds */
@@ -288,7 +284,7 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
     } /* end if */
     else {
         /* Non-flattened selection */
-        rank = space->extent.u.simple.rank;
+        rank = space->extent.rank;
         assert (rank >= 0 && rank<=H5S_MAX_RANK);	/* within array bounds */
         if (0==rank)
             goto empty;
@@ -301,7 +297,7 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
             d[i].strid = diminfo[i].stride;
             d[i].block = diminfo[i].block;
             d[i].count = diminfo[i].count;
-            d[i].xtent = space->extent.u.simple.size[i];
+            d[i].xtent = space->extent.size[i];
 #ifdef H5Smpi_DEBUG
             HDfprintf(stderr, "%s: start=%Hd  stride=%Hu  count=%Hu  block=%Hu  xtent=%Hu",
                 FUNC, d[i].start, d[i].strid, d[i].count, d[i].block, d[i].xtent );
@@ -348,7 +344,7 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
 *  Construct contig type for inner contig dims:
 *******************************************************/
 #ifdef H5Smpi_DEBUG
-    HDfprintf(stderr, "%s: Making contig type %d MPI_BYTEs\n", FUNC, elmt_size );
+    HDfprintf(stderr, "%s: Making contig type %d MPI_BYTEs\n", FUNC,elmt_size );
     for (i=rank-1; i>=0; --i)
         HDfprintf(stderr, "d[%d].xtent=%Hu \n", i, d[i].xtent);
 #endif
@@ -362,27 +358,32 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
     for ( i=rank-1; i>=0; --i) {
 #ifdef H5Smpi_DEBUG
         HDfprintf(stderr, "%s: Dimension i=%d \n"
-            "count=%Hu block=%Hu stride=%Hu\n",
-            FUNC, i, d[i].count, d[i].block, d[i].strid );
+            "start=%Hd count=%Hu block=%Hu stride=%Hu, xtent=%Hu max_xtent=%d\n",
+            FUNC, i, d[i].start, d[i].count, d[i].block, d[i].strid, d[i].xtent, max_xtent[i]);
 #endif
 
 #ifdef H5Smpi_DEBUG
-        HDfprintf(stderr, "%s: i=%d  Making vector-type \n", FUNC, i);
+        HDfprintf(stderr, "%s: i=%d  Making vector-type \n", FUNC,i);
 #endif
        /****************************************
-       *  Build vector in current dimension:
+       *  Build vector type of the selection.
        ****************************************/
-        mpi_code =MPI_Type_vector((int)(d[i].count),        /* count */
+	mpi_code =MPI_Type_vector((int)(d[i].count),        /* count */
 				  (int)(d[i].block),        /* blocklength */
 				  (int)(d[i].strid),   	    /* stride */
 				  inner_type,	            /* old type */
-				  &outer_type );            /* new type */
+				  &outer_type);            /* new type */
 
         MPI_Type_free( &inner_type );
         if (mpi_code!=MPI_SUCCESS)
             HMPI_GOTO_ERROR(FAIL, "couldn't create MPI vector type", mpi_code);
 
-        displacement[1] = (MPI_Aint)elmt_size * max_xtent[i];
+       /****************************************
+       *  Then build the dimension type as (start, vector type, xtent).
+       ****************************************/
+       /* calculate start and extent values of this dimension */
+	displacement[1] = d[i].start * offset[i] * elmt_size;
+        displacement[2] = (MPI_Aint)elmt_size * max_xtent[i];
         if(MPI_SUCCESS != (mpi_code = MPI_Type_extent(outer_type, &extent_len)))
             HMPI_GOTO_ERROR(FAIL, "MPI_Type_extent failed", mpi_code);
 
@@ -391,33 +392,28 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
        *  so that it still starts at 0, but its extent
        *  is the full extent in this dimension.
        *************************************************/
-        if ((int)extent_len < displacement[1]) {
+        if (displacement[1] > 0 || (int)extent_len < displacement[2]) {
 
-#ifdef H5Smpi_DEBUG
-            HDfprintf(stderr, "%s: i=%d Extending struct type\n"
-                "***displacements: 0, %d\n", FUNC, i, displacement[1]);
-#endif
-
-#ifdef H5_HAVE_MPI2  /*  have MPI-2  (this function is not included in MPICH) */
-            mpi_code = MPI_Type_create_resized
-                                  ( outer_type,        /* old type  */
-                                    0,                 /* blocklengths */
-                                    displacement[1],   /* displacements */
-                                    &inner_type);      /* new type */
-#else        /*  do not have MPI-2  */
             block_length[0] = 1;
             block_length[1] = 1;
+            block_length[2] = 1;
   
             displacement[0] = 0;
   
-            old_type[0] = outer_type;
-            old_type[1] = MPI_UB;
-            mpi_code = MPI_Type_struct ( 2,               /* count */
+            old_types[0] = MPI_LB;
+            old_types[1] = outer_type;
+            old_types[2] = MPI_UB;
+#ifdef H5Smpi_DEBUG
+            HDfprintf(stderr, "%s: i=%d Extending struct type\n"
+                "***displacements: %d, %d, %d\n",
+		FUNC, i, displacement[0], displacement[1], displacement[2]);
+#endif
+
+            mpi_code = MPI_Type_struct ( 3,               /* count */
                                     block_length,    /* blocklengths */
                                     displacement,    /* displacements */
-                                    old_type,        /* old types */
+                                    old_types,       /* old types */
                                     &inner_type);    /* new type */
-#endif
   
             MPI_Type_free (&outer_type);
     	    if (mpi_code!=MPI_SUCCESS)
@@ -435,42 +431,7 @@ H5S_mpio_hyper_type( const H5S_t *space, size_t elmt_size,
 
     /* At this point inner_type is actually the outermost type, even for 0-trip loop */
 
-/***************************************************************
-*  Final task: create a struct which is a "clone" of the
-*  current struct, but displaced according to the d[i].start
-*  values given in the hyperslab description:
-***************************************************************/
-    displacement[0] = 0;
-    for (i=rank-1; i>=0; i--)
-        displacement[0] += d[i].start * offset[i];
-
-    if (displacement[0] > 0) {
-        displacement[0] *= elmt_size;
-        block_length[0] = 1;
-        old_type[0] = inner_type;
-
-#ifdef H5Smpi_DEBUG
-        HDfprintf(stderr, "%s:  Making final struct\n***count=1:\n", FUNC);
-        HDfprintf(stderr, "\tblocklength[0]=%d; displacement[0]=%d\n",
-                     block_length[0], displacement[0]);
-#endif
-
-  
-        if (MPI_SUCCESS != (mpi_code= MPI_Type_struct( 1,                  /* count */
-                     block_length,       /* blocklengths */
-                     displacement,       /* displacements */
-                     old_type,	         /* old type */
-                     new_type ))         /* new type */
-                )
-            HMPI_GOTO_ERROR(FAIL, "couldn't create MPI struct type", mpi_code);
-
-        if (MPI_SUCCESS != (mpi_code= MPI_Type_free (&old_type[0])))
-            HMPI_GOTO_ERROR(FAIL, "MPI_Type_free failed", mpi_code);
-    }
-    else {
-        *new_type = inner_type;
-    }
-
+    *new_type = inner_type;
     if (MPI_SUCCESS != (mpi_code= MPI_Type_commit( new_type )))
         HMPI_GOTO_ERROR(FAIL, "MPI_Type_commit failed", mpi_code);
 
@@ -490,12 +451,12 @@ empty:
 done:
     /* Release selection iterator */
     if(sel_iter_init) {
-        if (H5S_select_iter_release(&sel_iter)<0)
+        if (H5S_SELECT_ITER_RELEASE(&sel_iter)<0)
             HDONE_ERROR (H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator");
     } /* end if */
 
 #ifdef H5Smpi_DEBUG
-    HDfprintf(stderr, "Leave %s, count=%Hu  is_derived_type=%d\n",
+    HDfprintf(stderr, "Leave %s, count=%ld  is_derived_type=%d\n",
 		FUNC, *count, *is_derived_type );
 #endif
     FUNC_LEAVE_NOAPI(ret_value);
@@ -525,7 +486,7 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-static herr_t
+herr_t
 H5S_mpio_space_type( const H5S_t *space, size_t elmt_size,
 		     /* out: */
 		     MPI_Datatype *new_type,
@@ -541,10 +502,10 @@ H5S_mpio_space_type( const H5S_t *space, size_t elmt_size,
     assert (space);
 
     /* Creat MPI type based on the kind of selection */
-    switch (space->extent.type) {
+    switch (H5S_GET_EXTENT_TYPE(space)) {
         case H5S_SCALAR:
         case H5S_SIMPLE:
-            switch(space->select.type) {
+            switch(H5S_GET_SELECT_TYPE(space)) {
                 case H5S_SEL_NONE:
                     if ( H5S_mpio_none_type( space, elmt_size,
                         /* out: */ new_type, count, extra_offset, is_derived_type ) <0)
@@ -577,7 +538,6 @@ H5S_mpio_space_type( const H5S_t *space, size_t elmt_size,
         case H5S_COMPLEX:
             /* not yet implemented */
             HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "complex data spaces are not supported yet");
-            break;
 
         default:
             assert("unknown data space type" && 0);
@@ -588,281 +548,347 @@ done:
     FUNC_LEAVE_NOAPI(ret_value);
 }
 
+
 
 /*-------------------------------------------------------------------------
- * Function:	H5S_mpio_spaces_xfer
+ * Function:	H5S_mpio_space_span_ type
  *
- * Purpose:	Use MPI-IO to transfer data efficiently
- *		directly between app buffer and file.
+ * Purpose:	Translate an HDF5 dataspace selection into a general
+                MPI derivewd datatype built with span-tree.
+ *             
+ *		Currently handle only hyperslab and "all" selections.
  *
  * Return:	non-negative on success, negative on failure.
  *
- * Programmer:	rky 980813
+ * Outputs:	*new_type	  the MPI type corresponding to the selection
+ *		*count		  how many objects of the new_type in selection
+ *				  (useful if this is the buffer type for xfer)
+ *		*extra_offset     Number of bytes of offset within dataset
+ *		*is_derived_type  0 if MPI primitive type, 1 if derived
  *
- * Notes:
- *      For collective data transfer only since this would eventually call
- *      H5FD_mpio_setup to do setup to eveually call MPI_File_set_view in
- *      H5FD_mpio_read or H5FD_mpio_write.  MPI_File_set_view is a collective
- *      call.  Letting independent data transfer use this route would result in
- *      hanging.
- *
- *      The preconditions for calling this routine are located in the
- *      H5S_mpio_opt_possible() routine, which determines whether this routine
- *      can be called for a given dataset transfer.
+ * Programmer:	KY
  *
  * Modifications:
- *	rky 980918
- *	Added must_convert parameter to let caller know we can't optimize
- *	the xfer.
  *
- *	Albert Cheng, 001123
- *	Include the MPI_type freeing as part of cleanup code.
- *
- *      QAK - 2002/04/02
- *      Removed the must_convert parameter and move preconditions to
- *      H5S_mpio_opt_possible() routine
- *
- *      QAK - 2002/06/17
- *      Removed 'disp' parameter from H5FD_mpio_setup routine and use the
- *      address of the dataset in MPI_File_set_view() calls, as necessary.
- *
- *      QAK - 2002/06/18
- *      Removed 'dc_plist' parameter, since it was not used.  Also, switch to
- *      getting the 'extra_offset' setting for each selection.
+ *      Quincey Koziol, June 18, 2002
+ *      Added 'extra_offset' parameter
  *
  *-------------------------------------------------------------------------
  */
+herr_t
+H5S_mpio_space_span_type( const H5S_t *space, size_t elmt_size,
+		     /* out: */
+		     MPI_Datatype *new_type,
+		     size_t *count,
+		     hsize_t *extra_offset,
+		     hbool_t *is_derived_type )
+{
+    herr_t	ret_value = SUCCEED;
+
+    FUNC_ENTER_NOAPI_NOINIT(H5S_mpio_space_span_type);
+
+    /* Check args */
+    assert (space);
+
+    /* Creat MPI type based on the kind of selection */
+    switch (H5S_GET_EXTENT_TYPE(space)) {
+        case H5S_SCALAR:
+        case H5S_SIMPLE:
+            switch(H5S_GET_SELECT_TYPE(space)) {
+                case H5S_SEL_NONE:
+                    if ( H5S_mpio_none_type( space, elmt_size,
+                        /* out: */ new_type, count, extra_offset, is_derived_type ) <0)
+                        HGOTO_ERROR(H5E_DATASPACE, H5E_BADTYPE, FAIL,"couldn't convert \"all\" selection to MPI type");
+                    break;
+
+                case H5S_SEL_ALL:
+                    if ( H5S_mpio_all_type( space, elmt_size,
+                        /* out: */ new_type, count, extra_offset, is_derived_type ) <0)
+                        HGOTO_ERROR(H5E_DATASPACE, H5E_BADTYPE, FAIL,"couldn't convert \"all\" selection to MPI type");
+                    break;
+
+                case H5S_SEL_POINTS:
+                    /* not yet implemented */
+                    ret_value = FAIL;
+                    break;
+
+                case H5S_SEL_HYPERSLABS:
+                    if(H5S_mpio_span_hyper_type( space, elmt_size,
+                            /* out: */ new_type, count, extra_offset, is_derived_type )<0)
+                        HGOTO_ERROR(H5E_DATASPACE, H5E_BADTYPE, FAIL,"couldn't convert \"all\" selection to MPI type");
+                    break;
+
+                default:
+                    assert("unknown selection type" && 0);
+                    break;
+            } /* end switch */
+            break;
+
+        case H5S_COMPLEX:
+            /* not yet implemented */
+            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "complex data spaces are not supported yet");
+
+        default:
+            assert("unknown data space type" && 0);
+            break;
+    }
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value);
+}
+
+
+/* The following codes have been used by Kent to test
+   general collective derived datatype functionality. 
+   It should NOT be called by other routines except with
+   macro #ifdef KENT #endif
+   Nov. 11th, 2004 */
+
+   
 static herr_t
-H5S_mpio_spaces_xfer(H5F_t *f, const H5O_layout_t *layout, size_t elmt_size,
-                     const H5S_t *file_space, const H5S_t *mem_space,
-		     hid_t dxpl_id, void *_buf /*out*/,
-		     hbool_t do_write )
-{
-    haddr_t	 addr;                  /* Address of dataset (or selection) within file */
-    size_t	 mpi_buf_count, mpi_file_count;       /* Number of "objects" to transfer */
-    hsize_t	 mpi_buf_offset, mpi_file_offset;       /* Offset within dataset where selection (ie. MPI type) begins */
-    MPI_Datatype mpi_buf_type, mpi_file_type;   /* MPI types for buffer (memory) and file */
-    hbool_t	 mbt_is_derived=0,      /* Whether the buffer (memory) type is derived and needs to be free'd */
-		 mft_is_derived=0;      /* Whether the file type is derived and needs to be free'd */
-    hbool_t	 plist_is_setup=0;      /* Whether the dxpl has been customized */
-    uint8_t	*buf=(uint8_t *)_buf;   /* Alias for pointer arithmetic */
-    int         mpi_code;               /* MPI return code */
-    herr_t	 ret_value = SUCCEED;   /* Return value */
+H5S_mpio_span_hyper_type( const H5S_t *space, size_t elmt_size,
+		     /* out: */
+		     MPI_Datatype *new_type,
+		     size_t *count,
+		     hsize_t *extra_offset,
+			  hbool_t *is_derived_type ){
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_mpio_spaces_xfer);
+  MPI_Datatype          span_type;
+  H5S_hyper_span_t      *ospan;
+  H5S_hyper_span_info_t *odown;
+  hsize_t               *size;
+  int                   rank;
+  herr_t                ret_value = SUCCEED;
 
-    /* Check args */
-    assert (f);
-    assert (layout);
-    assert (file_space);
-    assert (mem_space);
-    assert (buf);
-    assert (IS_H5FD_MPIO(f));
-    /* Make certain we have the correct type of property list */
-    assert(TRUE==H5P_isa_class(dxpl_id,H5P_DATASET_XFER));
+   MPI_Aint              extent,lb;
 
-    /* create the MPI buffer type */
-    if (H5S_mpio_space_type( mem_space, elmt_size,
-			       /* out: */
-			       &mpi_buf_type,
-			       &mpi_buf_count,
-			       &mpi_buf_offset,
-			       &mbt_is_derived )<0)
-    	HGOTO_ERROR(H5E_DATASPACE, H5E_BADTYPE, FAIL,"couldn't create MPI buf type");
+  FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_mpio_span_hyper_type);
 
-    /* create the MPI file type */
-    if ( H5S_mpio_space_type( file_space, elmt_size,
-			       /* out: */
-			       &mpi_file_type,
-			       &mpi_file_count,
-			       &mpi_file_offset,
-			       &mft_is_derived )<0)
-    	HGOTO_ERROR(H5E_DATASPACE, H5E_BADTYPE, FAIL,"couldn't create MPI file type");
+  printf("coming to hyper type \n");
+  /* Check args */
+    assert (space);
+    /* assert(sizeof(MPI_Aint) >= sizeof(elmt_size));?? */
 
-    /* Use the absolute base address of the dataset (or chunk, eventually) as
-     * the address to read from.  This should be used as the diplacement for
-     * a call to MPI_File_set_view() in the read or write call.
-     */
-    addr = f->shared->base_addr + layout->addr + mpi_file_offset;
-#ifdef H5Smpi_DEBUG
-    HDfprintf(stderr, "spaces_xfer: addr=%a\n", addr );
+    /* Only for simple extent 
+    rank = space->extent.u.simple.rank;
+    */
+    rank = space->extent.rank;
+
+    /*    size = HDcalloc((size_t)rank,sizeof(hsize_t));  */ 
+    if (0==elmt_size)
+        goto empty;
+    size = space->extent.size;
+    
+
+    odown = space->select.sel_info.hslab->span_lst;
+    if(odown == NULL) 
+      goto empty;
+    ospan = odown->head;
+    if(ospan == NULL)
+      goto empty;
+    
+    obtain_datatype(space->extent.size,ospan,&span_type,elmt_size,rank);
+    MPI_Type_commit(&span_type);
+
+      MPI_Type_lb(span_type,&lb);
+      printf("lb %d\n",lb);
+      MPI_Type_extent(span_type,&extent);
+      printf("extent %d\n",extent);
+    *new_type = span_type;
+     /* fill in the remaining return values */
+    *count = 1;		
+    *extra_offset = 0;
+    *is_derived_type = 1;
+    printf("before freeing size\n");
+    /*    HDfree(size);*/
+    printf("after freeing size\n");
+    HGOTO_DONE(SUCCEED);
+
+empty:
+    /* special case: empty hyperslab */
+    *new_type = MPI_BYTE;
+    *count = 0;
+    *extra_offset = 0;
+    *is_derived_type = 0;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value);
+  }
+
+
+static herr_t obtain_datatype(const hsize_t size[], H5S_hyper_span_t* span,MPI_Datatype *span_type,
+			      size_t elmt_size,int dimindex) {
+  
+  int                   innercount,outercount;
+  MPI_Datatype          bas_type;
+  MPI_Datatype          temp_type;
+  MPI_Datatype          tempinner_type;
+  MPI_Datatype          *inner_type;
+  int                   *blocklen;
+  MPI_Aint              *disp;
+  MPI_Aint              stride;
+  MPI_Aint              extent,lb;
+  H5S_hyper_span_info_t *down;
+  H5S_hyper_span_t      *tspan;
+#ifdef H5_HAVE_MPI2
+  MPI_Aint              sizeaint,sizedtype;
+#endif /* H5_HAVE_MPI2 */
+  hsize_t               total_lowd,total_lowd1;
+  int                   i;
+  int                   ret;
+
+  assert(span);
+
+  inner_type = NULL;
+  down = NULL;
+  tspan= NULL;
+  down  = span->down;
+  tspan = span;
+
+  outercount = 0;
+  while(tspan) {
+    if(tspan) {
+      HDfprintf(stdout, " span->low %Hd\n",tspan->low);
+      HDfprintf(stdout, " span->high %Hd\n",tspan->high); 
+      HDfprintf(stdout, " span->nelm %Hu\n",tspan->nelem);
+    }
+    tspan = tspan->next;
+    outercount ++;
+  }
+
+  if(outercount == 0) {
+    span_type = NULL;
+    return 0;
+  }
+  printf("outercount %d\n",outercount);
+#ifdef H5_HAVE_MPI2
+  printf("coming into HAVE_MPI2\n");
+  fflush(stdout);
+
+  MPI_Type_extent(MPI_Aint,&sizeaint);
+  MPI_Type_extent(MPI_Datatype,&sizedtype);
+  printf("coming into HAVE_MPI2 type extent\n");
+  fflush(stdout);
+
+  blocklen  = (int *)HDcalloc((size_t)outercount,sizeof(int));
+  disp = (MPI_Aint *)HDcalloc((size_t)outercount,sizeaint);
+  inner_type   = (MPI_Datatype *)HDcalloc((size_t)outercount,sizedtype);
+#else
+  printf("coming to MPI2 else \n");
+  blocklen  = (int *)HDcalloc((size_t)outercount,sizeof(int));
+  disp = (MPI_Aint *)HDcalloc((size_t)outercount,sizeof(int));
+  inner_type   = (MPI_Datatype *)HDcalloc((size_t)outercount,sizeof(int));
+  printf("end of calloc \n");
 #endif
+  
+  
+  tspan = span;
+  outercount = 0;
 
-    /*
-     * Pass buf type, file type to the file driver. Request an MPI type
-     * transfer (instead of an elementary byteblock transfer).
-     */
-    if(H5FD_mpio_setup(dxpl_id, mpi_buf_type, mpi_file_type)<0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set MPI-I/O properties");
-    plist_is_setup=1;
-
-    /* Adjust the buffer pointer to the beginning of the selection */
-    buf+=mpi_buf_offset;
-
-    /* transfer the data */
-    if (do_write) {
-    	if (H5FD_write(f->shared->lf, H5FD_MEM_DRAW, dxpl_id, addr, mpi_buf_count, buf) <0)
-	    HGOTO_ERROR(H5E_IO, H5E_WRITEERROR, FAIL,"MPI write failed");
-    } else {
-    	if ( H5FD_read (f->shared->lf, H5FD_MEM_DRAW, dxpl_id, addr, mpi_buf_count, buf) <0)
-	    HGOTO_ERROR(H5E_IO, H5E_READERROR, FAIL,"MPI read failed");
+  if(down == NULL){
+    printf("coming to down = NULL \n");
+    if(dimindex > 1) printf("wrong area \n");
+    MPI_Type_contiguous((int)elmt_size,MPI_BYTE,&bas_type);
+    MPI_Type_commit(&bas_type);
+    printf("after type commit \n");
+    while(tspan){
+      disp[outercount]      = (MPI_Aint)elmt_size * tspan->low;
+      blocklen[outercount]  = tspan->nelem;
+      tspan                 = tspan->next;
+      outercount ++;
     }
+    /*    printf("outercount %d\n",outercount);
+	  printf("after while loop \n");*/
+    ret = MPI_Type_hindexed(outercount,blocklen,disp,bas_type,span_type);
+    if(ret < 0) printf("type hindexed doesn't work\n");
+    printf("after hindexed \n");
 
-done:
-    /* Reset the dxpl settings */
-    if(plist_is_setup) {
-        if(H5FD_mpio_teardown(dxpl_id)<0)
-    	    HDONE_ERROR(H5E_DATASPACE, H5E_CANTFREE, FAIL, "unable to reset dxpl values");
-    } /* end if */
+     
+  }
+  else {/* dimindex is the rank of the dimension */
 
-    /* free the MPI buf and file types */
-    if (mbt_is_derived) {
-	if (MPI_SUCCESS != (mpi_code= MPI_Type_free( &mpi_buf_type )))
-            HMPI_DONE_ERROR(FAIL, "MPI_Type_free failed", mpi_code);
+    if(dimindex <2) printf("something is wrong \n");
+    /* Calculate the total bytes of the lower dimension */
+    total_lowd  = 1;  /* one dimension down */
+    total_lowd1 = 1; /* two dimensions down */
+
+    for ( i = dimindex-1; i > 0; i--)
+         total_lowd = total_lowd * size[i];
+
+    for ( i = dimindex-1; i > 1; i--)
+         total_lowd1 = total_lowd1 * size[i];
+     
+     HDfprintf(stdout, " one dimension down size %Hu\n",total_lowd);
+     HDfprintf(stdout, " two dimension down size %Hu\n",total_lowd1);   
+   
+     while(tspan){
+/* Displacement should be in byte and should have dimension information */
+/* First using MPI Type vector to build derived data type for this span only */
+/* Need to calculate the disp in byte for this dimension. */
+      /* Calculate the total bytes of the lower dimension */
+
+      disp[outercount]      = tspan->low*total_lowd*elmt_size;
+      blocklen[outercount]  = 1;
+      printf("displacement the 0 rank %d\n",disp[0]);
+      /* generating inner derived datatype by using MPI_Type_hvector */
+      obtain_datatype(size,tspan->down->head,&temp_type,elmt_size,dimindex-1);
+      ret= MPI_Type_commit(&temp_type);
+      if(ret < 0) printf("cannot commit temp_type\n");
+/*      inner_type[count] = temp_type; */
+
+#ifdef H5_HAVE_MPI2
+      MPI_Type_get_extent(temp_type,&lb,&extent);
+#else
+      MPI_Type_lb(temp_type,&lb);
+      printf("lb %d\n",lb);
+      MPI_Type_extent(temp_type,&extent);
+      printf("extent %d\n",extent);
+#endif
+     /* building the inner vector datatype */
+      /* The following calculation of stride is wrong since stride is calculated
+         from the first element of the block to the first element of the next
+         block. */
+      /*stride = total_lowd1 * (size[dimindex-1]*elmt_size-extent-lb);*/
+       stride     = total_lowd*elmt_size;
+      innercount  = tspan->nelem;
+      printf("stride %d\n",stride);
+      printf("innercount %d\n",innercount);
+      fflush(stdout);
+	ret = MPI_Type_hvector(innercount,1,stride,temp_type,&tempinner_type);
+
+      /* MPI_Type_contiguous(2,temp_type,&tempinner_type);*/
+      if(ret < 0) printf("wrong vector \n");
+      MPI_Type_commit(&tempinner_type);
+      printf("after tempinner_type commit\n");
+      MPI_Type_free(&temp_type);
+      printf("after free \n");
+      inner_type[outercount] = tempinner_type;
+      outercount ++;
+      tspan = tspan->next;
     }
-    if (mft_is_derived) {
-	if (MPI_SUCCESS != (mpi_code= MPI_Type_free( &mpi_file_type )))
-            HMPI_DONE_ERROR(FAIL, "MPI_Type_free failed", mpi_code);
+    
+    /* building the whole vector datatype */
+    MPI_Type_struct(outercount,blocklen,disp,inner_type,span_type);
+    printf("after type struct \n");
+  }
+
+  
+  if(inner_type != NULL){
+    if(down != NULL) {
+    for(i=0;i<outercount;i++)
+             MPI_Type_free(&inner_type[i]);
+      
     }
+  }
 
-    FUNC_LEAVE_NOAPI(ret_value);
+  if(inner_type != NULL)
+    HDfree(inner_type);
+  if(blocklen != NULL)
+  HDfree(blocklen);
+  if(disp != NULL)
+  HDfree(disp);
+  printf("after obtaining type \n");
+  return 0;
 }
 
-
-/*-------------------------------------------------------------------------
- * Function:	H5S_mpio_spaces_read
- *
- * Purpose:	MPI-IO function to read directly from app buffer to file.
- *
- * Return:	non-negative on success, negative on failure.
- *
- * Programmer:	rky 980813
- *
- * Modifications:
- *
- * rky 980918
- * Added must_convert parameter to let caller know we can't optimize the xfer.
- *
- *      QAK - 2002/04/02
- *      Removed the must_convert parameter and move preconditions to
- *      H5S_mpio_opt_possible() routine
- *
- *-------------------------------------------------------------------------
- */
-herr_t
-H5S_mpio_spaces_read(H5F_t *f, const H5O_layout_t *layout,
-    H5P_genplist_t UNUSED *dc_plist, const H5D_storage_t UNUSED *store, size_t elmt_size,
-    const H5S_t *file_space, const H5S_t *mem_space, hid_t dxpl_id,
-    void *buf/*out*/)
-{
-    herr_t ret_value;
-
-    FUNC_ENTER_NOAPI(H5S_mpio_spaces_read, FAIL);
-
-    ret_value = H5S_mpio_spaces_xfer(f, layout, elmt_size,
-				     file_space, mem_space, dxpl_id,
-				     buf, 0/*read*/);
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
-
-
-/*-------------------------------------------------------------------------
- * Function:	H5S_mpio_spaces_write
- *
- * Purpose:	MPI-IO function to write directly from app buffer to file.
- *
- * Return:	non-negative on success, negative on failure.
- *
- * Programmer:	rky 980813
- *
- * Modifications:
- *
- * rky 980918
- * Added must_convert parameter to let caller know we can't optimize the xfer.
- *
- *      QAK - 2002/04/02
- *      Removed the must_convert parameter and move preconditions to
- *      H5S_mpio_opt_possible() routine
- *
- *-------------------------------------------------------------------------
- */
-herr_t
-H5S_mpio_spaces_write(H5F_t *f, H5O_layout_t *layout,
-    H5P_genplist_t UNUSED *dc_plist, const H5D_storage_t UNUSED *store, size_t elmt_size,
-    const H5S_t *file_space, const H5S_t *mem_space, hid_t dxpl_id,
-    const void *buf)
-{
-    herr_t ret_value;
-
-    FUNC_ENTER_NOAPI(H5S_mpio_spaces_write, FAIL);
-
-    /*OKAY: CAST DISCARDS CONST QUALIFIER*/
-    ret_value = H5S_mpio_spaces_xfer(f, layout, elmt_size,
-				     file_space, mem_space, dxpl_id,
-				     (void*)buf, 1/*write*/);
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
-
-
-/*-------------------------------------------------------------------------
- * Function:	H5S_mpio_opt_possible
- *
- * Purpose:	Checks if an direct I/O transfer is possible between memory and
- *                  the file.
- *
- * Return:	Success:        Non-negative: TRUE or FALSE
- *		Failure:	Negative
- *
- * Programmer:	Quincey Koziol
- *              Wednesday, April 3, 2002
- *
- * Modifications:
- *
- *-------------------------------------------------------------------------
- */
-htri_t
-H5S_mpio_opt_possible( const H5S_t *mem_space, const H5S_t *file_space, const unsigned flags)
-{
-    htri_t c1,c2;               /* Flags whether a selection is optimizable */
-    htri_t ret_value=TRUE;
-
-    FUNC_ENTER_NOAPI(H5S_mpio_opt_possible, FAIL);
-
-    /* Check args */
-    assert(mem_space);
-    assert(file_space);
-
-    /* Check whether these are both simple or scalar dataspaces */
-    if (!((H5S_SIMPLE==mem_space->extent.type || H5S_SCALAR==mem_space->extent.type)
-         && (H5S_SIMPLE==file_space->extent.type || H5S_SCALAR==file_space->extent.type)))
-        HGOTO_DONE(FALSE);
-
-    /* Check whether both selections are "regular" */
-    c1=(*file_space->select.is_regular)(file_space);
-    c2=(*mem_space->select.is_regular)(mem_space);
-    if(c1==FAIL || c2==FAIL)
-        HGOTO_ERROR(H5E_DATASPACE, H5E_BADRANGE, FAIL, "invalid check for single selection blocks");
-    if(c1==FALSE || c2==FALSE)
-        HGOTO_DONE(FALSE);
-
-    /* Can't currently handle point selections */
-    if (H5S_SEL_POINTS==mem_space->select.type || H5S_SEL_POINTS==file_space->select.type)
-        HGOTO_DONE(FALSE);
-
-    /* Dataset storage must be contiguous currently */
-    if ((flags&H5S_CONV_STORAGE_MASK)!=H5S_CONV_STORAGE_CONTIGUOUS)
-        HGOTO_DONE(FALSE);
-
-    /* Parallel I/O conversion flag must be set */
-    if(!(flags&H5S_CONV_PAR_IO_POSSIBLE))
-        HGOTO_DONE(FALSE);
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
-} /* H5S_mpio_opt_possible() */
-
 #endif  /* H5_HAVE_PARALLEL */
