@@ -21,9 +21,6 @@
 
 #define H5S_PACKAGE		/*suppress error about including H5Spkg	  */
 
-/* Pablo information */
-/* (Put before include files to avoid problems with inline functions) */
-#define PABLO_MASK      H5Snone_mask
 
 #include "H5private.h"
 #include "H5Eprivate.h"
@@ -32,19 +29,66 @@
 #include "H5Vprivate.h"
 #include "H5Dprivate.h"
 
-/* Interface initialization */
-#define INTERFACE_INIT  NULL
-static int             interface_initialize_g = 0;
-
 /* Static function prototypes */
-static herr_t H5S_none_iter_coords(const H5S_sel_iter_t *iter, hssize_t *coords);
-static herr_t H5S_none_iter_block(const H5S_sel_iter_t *iter, hssize_t *start, hssize_t *end);
+
+/* Selection callbacks */
+static herr_t H5S_none_copy(H5S_t *dst, const H5S_t *src, hbool_t share_selection);
+static herr_t H5S_none_get_seq_list(const H5S_t *space, unsigned flags,
+    H5S_sel_iter_t *iter, size_t maxseq, size_t maxbytes,
+    size_t *nseq, size_t *nbytes, hsize_t *off, size_t *len);
+static herr_t H5S_none_release(H5S_t *space);
+static htri_t H5S_none_is_valid(const H5S_t *space);
+static hssize_t H5S_none_serial_size(const H5S_t *space);
+static herr_t H5S_none_serialize(const H5S_t *space, uint8_t *buf);
+static herr_t H5S_none_deserialize(H5S_t *space, const uint8_t *buf);
+static herr_t H5S_none_bounds(const H5S_t *space, hsize_t *start, hsize_t *end);
+static htri_t H5S_none_is_contiguous(const H5S_t *space);
+static htri_t H5S_none_is_single(const H5S_t *space);
+static htri_t H5S_none_is_regular(const H5S_t *space);
+static herr_t H5S_none_iter_init(H5S_sel_iter_t *iter, const H5S_t *space);
+
+/* Selection iteration callbacks */
+static herr_t H5S_none_iter_coords(const H5S_sel_iter_t *iter, hsize_t *coords);
+static herr_t H5S_none_iter_block(const H5S_sel_iter_t *iter, hsize_t *start, hsize_t *end);
 static hsize_t H5S_none_iter_nelmts(const H5S_sel_iter_t *iter);
 static htri_t H5S_none_iter_has_next_block(const H5S_sel_iter_t *iter);
 static herr_t H5S_none_iter_next(H5S_sel_iter_t *sel_iter, size_t nelem);
 static herr_t H5S_none_iter_next_block(H5S_sel_iter_t *sel_iter);
 static herr_t H5S_none_iter_release(H5S_sel_iter_t *sel_iter);
 
+/* Selection properties for "none" selections */
+const H5S_select_class_t H5S_sel_none[1] = {{
+    H5S_SEL_NONE,
+
+    /* Methods on selection */
+    H5S_none_copy,
+    H5S_none_get_seq_list,
+    H5S_none_release,
+    H5S_none_is_valid,
+    H5S_none_serial_size,
+    H5S_none_serialize,
+    H5S_none_deserialize,
+    H5S_none_bounds,
+    H5S_none_is_contiguous,
+    H5S_none_is_single,
+    H5S_none_is_regular,
+    H5S_none_iter_init,
+}};
+
+/* Iteration properties for "none" selections */
+static const H5S_sel_iter_class_t H5S_sel_iter_none[1] = {{
+    H5S_SEL_NONE,
+
+    /* Methods on selection iterator */
+    H5S_none_iter_coords,
+    H5S_none_iter_block,
+    H5S_none_iter_nelmts,
+    H5S_none_iter_has_next_block,
+    H5S_none_iter_next,
+    H5S_none_iter_next_block,
+    H5S_none_iter_release,
+}};
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5S_none_iter_init
@@ -61,27 +105,18 @@ static herr_t H5S_none_iter_release(H5S_sel_iter_t *sel_iter);
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_none_iter_init (H5S_sel_iter_t *iter, const H5S_t UNUSED *space, size_t UNUSED elmt_size)
+H5S_none_iter_init (H5S_sel_iter_t *iter, const H5S_t UNUSED *space)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_iter_init, FAIL);
+    FUNC_ENTER_NOAPI_NOFUNC(H5S_none_iter_init);
 
     /* Check args */
-    assert (space && H5S_SEL_NONE==space->select.type);
+    assert (space && H5S_SEL_NONE==H5S_GET_SELECT_TYPE(space));
     assert (iter);
 
-    /* Initialize methods for selection iterator */
-    iter->iter_coords=H5S_none_iter_coords;
-    iter->iter_block=H5S_none_iter_block;
-    iter->iter_nelmts=H5S_none_iter_nelmts;
-    iter->iter_has_next_block=H5S_none_iter_has_next_block;
-    iter->iter_next=H5S_none_iter_next;
-    iter->iter_next_block=H5S_none_iter_next_block;
-    iter->iter_release=H5S_none_iter_release;
+    /* Initialize type of selection iterator */
+    iter->type=H5S_sel_iter_none;
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }   /* H5S_none_iter_init() */
 
 
@@ -101,7 +136,7 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5S_none_iter_coords (const H5S_sel_iter_t UNUSED *iter, hssize_t UNUSED *coords)
+H5S_none_iter_coords (const H5S_sel_iter_t UNUSED *iter, hsize_t UNUSED *coords)
 {
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_iter_coords);
 
@@ -129,7 +164,7 @@ H5S_none_iter_coords (const H5S_sel_iter_t UNUSED *iter, hssize_t UNUSED *coords
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5S_none_iter_block (const H5S_sel_iter_t UNUSED *iter, hssize_t UNUSED *start, hssize_t UNUSED *end)
+H5S_none_iter_block (const H5S_sel_iter_t UNUSED *iter, hsize_t UNUSED *start, hsize_t UNUSED *end)
 {
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_iter_block);
 
@@ -303,52 +338,50 @@ H5S_none_iter_release (H5S_sel_iter_t UNUSED * iter)
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-herr_t
+static herr_t
 H5S_none_release (H5S_t UNUSED * space)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_release, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_release);
 
     /* Check args */
     assert (space);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }   /* H5S_none_release() */
 
 
 /*--------------------------------------------------------------------------
  NAME
-    H5S_none_npoints
+    H5S_none_copy
  PURPOSE
-    Compute number of elements in current selection
+    Copy a selection from one dataspace to another
  USAGE
-    hsize_t H5S_none_npoints(space)
-        H5S_t *space;       IN: Pointer to dataspace
+    herr_t H5S_none_copy(dst, src)
+        H5S_t *dst;  OUT: Pointer to the destination dataspace
+        H5S_t *src;  IN: Pointer to the source dataspace
  RETURNS
-    The number of elements in selection on success, 0 on failure
+    Non-negative on success/Negative on failure
  DESCRIPTION
-    Compute number of elements in current selection.  For "none" selections,
-    this is always 0.
+    Copies the 'none' selection information from the source
+    dataspace to the destination dataspace.
  GLOBAL VARIABLES
  COMMENTS, BUGS, ASSUMPTIONS
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-hsize_t
-H5S_none_npoints (const H5S_t UNUSED *space)
+static herr_t
+H5S_none_copy(H5S_t *dst, const H5S_t UNUSED *src, hbool_t UNUSED share_selection)
 {
-    hsize_t ret_value=0;   /* Return value */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_copy);
 
-    FUNC_ENTER_NOAPI(H5S_none_npoints, 0);
+    assert(src);
+    assert(dst);
 
-    /* Check args */
-    assert (space);
+    /* Set number of elements in selection */
+    dst->select.num_elem=0;
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}   /* H5S_none_npoints() */
+    FUNC_LEAVE_NOAPI(SUCCEED);
+} /* end H5S_none_copy() */
 
 
 /*--------------------------------------------------------------------------
@@ -371,17 +404,14 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-htri_t
+static htri_t
 H5S_none_is_valid (const H5S_t UNUSED *space)
 {
-    htri_t ret_value=TRUE;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_is_valid, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_is_valid);
 
     assert(space);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(TRUE);
 } /* end H5S_none_is_valid() */
 
 
@@ -404,12 +434,10 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-hssize_t
+static hssize_t
 H5S_none_serial_size (const H5S_t UNUSED *space)
 {
-    hssize_t ret_value;    /* return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_serial_size, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_serial_size);
 
     assert(space);
 
@@ -417,10 +445,7 @@ H5S_none_serial_size (const H5S_t UNUSED *space)
      *  <type (4 bytes)> + <version (4 bytes)> + <padding (4 bytes)> + 
      *      <length (4 bytes)> = 16 bytes
      */
-    ret_value=16;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(16);
 } /* end H5S_none_serial_size() */
 
 
@@ -443,23 +468,20 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-herr_t
+static herr_t
 H5S_none_serialize (const H5S_t *space, uint8_t *buf)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_serialize, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_serialize);
 
     assert(space);
 
     /* Store the preamble information */
-    UINT32ENCODE(buf, (uint32_t)space->select.type);  /* Store the type of selection */
+    UINT32ENCODE(buf, (uint32_t)H5S_GET_SELECT_TYPE(space));  /* Store the type of selection */
     UINT32ENCODE(buf, (uint32_t)1);  /* Store the version number */
     UINT32ENCODE(buf, (uint32_t)0);  /* Store the un-used padding */
     UINT32ENCODE(buf, (uint32_t)0);  /* Store the additional information length */
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 }   /* H5S_none_serialize() */
 
 
@@ -482,12 +504,12 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-herr_t
+static herr_t
 H5S_none_deserialize (H5S_t *space, const uint8_t UNUSED *buf)
 {
     herr_t ret_value;  /* return value */
 
-    FUNC_ENTER_NOAPI(H5S_none_deserialize, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT(H5S_none_deserialize);
 
     assert(space);
 
@@ -508,8 +530,8 @@ done:
  USAGE
     herr_t H5S_none_bounds(space, start, end)
         H5S_t *space;           IN: Dataspace pointer of selection to query
-        hssize_t *start;         OUT: Starting coordinate of bounding box
-        hssize_t *end;           OUT: Opposite coordinate of bounding box
+        hsize_t *start;         OUT: Starting coordinate of bounding box
+        hsize_t *end;           OUT: Opposite coordinate of bounding box
  RETURNS
     Non-negative on success, negative on failure
  DESCRIPTION
@@ -525,19 +547,16 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-herr_t
-H5S_none_bounds(const H5S_t UNUSED *space, hssize_t UNUSED *start, hssize_t UNUSED *end)
+static herr_t
+H5S_none_bounds(const H5S_t UNUSED *space, hsize_t UNUSED *start, hsize_t UNUSED *end)
 {
-    herr_t ret_value=FAIL;      /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_bounds, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_bounds);
 
     assert(space);
     assert(start);
     assert(end);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(FAIL);
 }   /* H5Sget_none_bounds() */
 
 
@@ -559,17 +578,14 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-htri_t
+static htri_t
 H5S_none_is_contiguous(const H5S_t UNUSED *space)
 {
-    htri_t ret_value=FALSE;      /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_is_contiguous, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_is_contiguous);
 
     assert(space);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(FALSE);
 }   /* H5S_none_is_contiguous() */
 
 
@@ -591,17 +607,14 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-htri_t
+static htri_t
 H5S_none_is_single(const H5S_t UNUSED *space)
 {
-    htri_t ret_value=FALSE;      /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_is_single, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_is_single);
 
     assert(space);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(FALSE);
 }   /* H5S_none_is_single() */
 
 
@@ -624,18 +637,15 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-htri_t
+static htri_t
 H5S_none_is_regular(const H5S_t UNUSED *space)
 {
-    htri_t ret_value=TRUE;      /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_none_is_regular, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_is_regular);
 
     /* Check args */
     assert(space);
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(TRUE);
 }   /* H5S_none_is_regular() */
 
 
@@ -666,24 +676,14 @@ herr_t H5S_select_none (H5S_t *space)
     assert(space);
 
     /* Remove current selection first */
-    if((*space->select.release)(space)<0)
+    if(H5S_SELECT_RELEASE(space)<0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDELETE, FAIL, "can't release hyperslab");
 
-    /* Set selection type */
-    space->select.type=H5S_SEL_NONE;
+    /* Set number of elements in selection */
+    space->select.num_elem=0;
 
-    /* Set selection methods */
-    space->select.get_seq_list=H5S_none_get_seq_list;
-    space->select.get_npoints=H5S_none_npoints;
-    space->select.release=H5S_none_release;
-    space->select.is_valid=H5S_none_is_valid;
-    space->select.serial_size=H5S_none_serial_size;
-    space->select.serialize=H5S_none_serialize;
-    space->select.bounds=H5S_none_bounds;
-    space->select.is_contiguous=H5S_none_is_contiguous;
-    space->select.is_single=H5S_none_is_single;
-    space->select.is_regular=H5S_none_is_regular;
-    space->select.iter_init=H5S_none_iter_init;
+    /* Set selection type */
+    space->select.type=H5S_sel_none;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value);
@@ -733,17 +733,16 @@ done:
  PURPOSE
     Create a list of offsets & lengths for a selection
  USAGE
-    herr_t H5S_all_get_seq_list(space,flags,iter,elem_size,maxseq,maxbytes,nseq,nbytes,off,len)
+    herr_t H5S_all_get_seq_list(space,flags,iter,maxseq,maxelem,nseq,nelem,off,len)
         H5S_t *space;           IN: Dataspace containing selection to use.
         unsigned flags;         IN: Flags for extra information about operation
         H5S_sel_iter_t *iter;   IN/OUT: Selection iterator describing last
                                     position of interest in selection.
-        size_t elem_size;       IN: Size of an element
         size_t maxseq;          IN: Maximum number of sequences to generate
-        size_t maxbytes;        IN: Maximum number of bytes to include in the
+        size_t maxelem;         IN: Maximum number of elements to include in the
                                     generated sequences
         size_t *nseq;           OUT: Actual number of sequences generated
-        size_t *nbytes;         OUT: Actual number of bytes in sequences generated
+        size_t *nelem;          OUT: Actual number of elements in sequences generated
         hsize_t *off;           OUT: Array of offsets
         size_t *len;            OUT: Array of lengths
  RETURNS
@@ -759,32 +758,28 @@ done:
  EXAMPLES
  REVISION LOG
 --------------------------------------------------------------------------*/
-herr_t
+static herr_t
 H5S_none_get_seq_list(const H5S_t UNUSED *space, unsigned UNUSED flags, H5S_sel_iter_t UNUSED *iter,
-    size_t UNUSED elem_size, size_t UNUSED maxseq, size_t UNUSED maxbytes, size_t *nseq, size_t *nbytes,
+    size_t UNUSED maxseq, size_t UNUSED maxelem, size_t *nseq, size_t *nelem,
     hsize_t UNUSED *off, size_t UNUSED *len)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI (H5S_none_get_seq_list, FAIL);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_none_get_seq_list);
 
     /* Check args */
     assert(space);
     assert(iter);
-    assert(elem_size>0);
     assert(maxseq>0);
-    assert(maxbytes>0);
+    assert(maxelem>0);
     assert(nseq);
-    assert(nbytes);
+    assert(nelem);
     assert(off);
     assert(len);
 
-    /* "none" selection don't generate sequences of bytes */
+    /* "none" selections don't generate sequences of bytes */
     *nseq=0;
 
-    /* The don't use any bytes, either */
-    *nbytes=0;
+    /* They don't use any elements, either */
+    *nelem=0;
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(SUCCEED);
 } /* end H5S_all_get_seq_list() */
