@@ -19,27 +19,25 @@
 
 #define H5T_PACKAGE		/*suppress error about including H5Tpkg	  */
 
+/* Interface initialization */
+#define H5_INTERFACE_INIT_FUNC	H5T_init_enum_interface
+
+
 #include "H5private.h"		/*generic functions			  */
 #include "H5Eprivate.h"		/*error handling			  */
-#include "H5FLprivate.h"	/*Free Lists	  */
+#include "H5FLprivate.h"	/*Free Lists				  */
 #include "H5Iprivate.h"		/*ID functions		   		  */
 #include "H5MMprivate.h"	/*memory management			  */
 #include "H5Tpkg.h"		/*data-type functions			  */
 
-#define PABLO_MASK	H5Tenum_mask
-
-/* Interface initialization */
-static int interface_initialize_g = 0;
-#define INTERFACE_INIT H5T_init_enum_interface
-static herr_t H5T_init_enum_interface(void);
-
-/* Declare extern the free list for H5T_t's */
+/* Declare extern the free lists for H5T_t's and H5T_shared_t's */
 H5FL_EXTERN(H5T_t);
+H5FL_EXTERN(H5T_shared_t);
 
 /* Static local functions */
-static char *H5T_enum_nameof(H5T_t *dt, void *value, char *name/*out*/,
+static char *H5T_enum_nameof(const H5T_t *dt, const void *value, char *name/*out*/,
 			      size_t size);
-static herr_t H5T_enum_valueof(H5T_t *dt, const char *name,
+static herr_t H5T_enum_valueof(const H5T_t *dt, const char *name,
 				void *value/*out*/);
 
 
@@ -59,9 +57,9 @@ DESCRIPTION
 static herr_t
 H5T_init_enum_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_init_enum_interface);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_init_enum_interface)
 
-    FUNC_LEAVE_NOAPI(H5T_init());
+    FUNC_LEAVE_NOAPI(H5T_init())
 } /* H5T_init_enum_interface() */
 
 
@@ -89,22 +87,22 @@ H5Tenum_create(hid_t parent_id)
     H5T_t	*dt = NULL;		/*new enumeration data type	*/
     hid_t	ret_value;	        /*return value			*/
     
-    FUNC_ENTER_API(H5Tenum_create, FAIL);
+    FUNC_ENTER_API(H5Tenum_create, FAIL)
     H5TRACE1("i","i",parent_id);
 
     /* Check args */
-    if (NULL==(parent=H5I_object_verify(parent_id,H5I_DATATYPE)) || H5T_INTEGER!=parent->type)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an integer data type");
+    if (NULL==(parent=H5I_object_verify(parent_id,H5I_DATATYPE)) || H5T_INTEGER!=parent->shared->type)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an integer data type")
 
     /* Build new type */
     if((dt=H5T_enum_create(parent))==NULL)
-	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "cannot create enum type");
+	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "cannot create enum type")
     /* Atomize the type */
     if ((ret_value=H5I_register(H5I_DATATYPE, dt))<0)
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "unable to register data type atom");
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "unable to register data type atom")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -127,25 +125,29 @@ done:
  *-------------------------------------------------------------------------
  */
 H5T_t *
-H5T_enum_create(H5T_t *parent)
+H5T_enum_create(const H5T_t *parent)
 {
     H5T_t	*ret_value;		/*new enumeration data type	*/
     
-    FUNC_ENTER_NOAPI(H5T_enum_create, NULL);
+    FUNC_ENTER_NOAPI(H5T_enum_create, NULL)
 
     assert(parent);
 
     /* Build new type */
     if (NULL==(ret_value = H5FL_CALLOC(H5T_t)))
-	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
-    ret_value->type = H5T_ENUM;
-    ret_value->parent = H5T_copy(parent, H5T_COPY_ALL);
-    assert(ret_value->parent);
-    ret_value->size = ret_value->parent->size;
+        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+    if (NULL==(ret_value->shared=H5FL_CALLOC(H5T_shared_t))) {
+        H5FL_FREE(H5T_t, ret_value);
+        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+    }
+    ret_value->shared->type = H5T_ENUM;
+    ret_value->shared->parent = H5T_copy(parent, H5T_COPY_ALL);
+    assert(ret_value->shared->parent);
+    ret_value->shared->size = ret_value->shared->parent->shared->size;
     ret_value->ent.header = HADDR_UNDEF;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -171,30 +173,30 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5Tenum_insert(hid_t type, const char *name, void *value)
+H5Tenum_insert(hid_t type, const char *name, const void *value)
 {
     H5T_t	*dt=NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
     
-    FUNC_ENTER_API(H5Tenum_insert, FAIL);
+    FUNC_ENTER_API(H5Tenum_insert, FAIL)
     H5TRACE3("e","isx",type,name,value);
 
     /* Check args */
     if (NULL==(dt=H5I_object_verify(type,H5I_DATATYPE)))
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type");
-    if (H5T_ENUM!=dt->type)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type")
+    if (H5T_ENUM!=dt->shared->type)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type")
     if (!name || !*name)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name specified");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name specified")
     if (!value)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value specified");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value specified")
 
     /* Do work */
     if (H5T_enum_insert(dt, name, value)<0)
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to insert new enumeration member");
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to insert new enumeration member")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -218,48 +220,48 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5T_enum_insert(H5T_t *dt, const char *name, void *value)
+H5T_enum_insert(const H5T_t *dt, const char *name, const void *value)
 {
-    int		i;
+    unsigned	i;
     char	**names=NULL;
     uint8_t	*values=NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
     
-    FUNC_ENTER_NOAPI(H5T_enum_insert, FAIL);
+    FUNC_ENTER_NOAPI(H5T_enum_insert, FAIL)
 
     assert(dt);
     assert(name && *name);
     assert(value);
 
     /* The name and value had better not already exist */
-    for (i=0; i<dt->u.enumer.nmembs; i++) {
-	if (!HDstrcmp(dt->u.enumer.name[i], name))
-	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "name redefinition");
-	if (!HDmemcmp(dt->u.enumer.value+i*dt->size, value, dt->size))
-	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "value redefinition");
+    for (i=0; i<dt->shared->u.enumer.nmembs; i++) {
+	if (!HDstrcmp(dt->shared->u.enumer.name[i], name))
+	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "name redefinition")
+	if (!HDmemcmp(dt->shared->u.enumer.value+i*dt->shared->size, value, dt->shared->size))
+	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "value redefinition")
     }
 
     /* Increase table sizes */
-    if (dt->u.enumer.nmembs >= dt->u.enumer.nalloc) {
-	int n = MAX(32, 2*dt->u.enumer.nalloc);
-	if (NULL==(names=H5MM_realloc(dt->u.enumer.name, n*sizeof(char*))))
-	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed");
-	dt->u.enumer.name = names;
+    if (dt->shared->u.enumer.nmembs >= dt->shared->u.enumer.nalloc) {
+	unsigned n = MAX(32, 2*dt->shared->u.enumer.nalloc);
+	if (NULL==(names=H5MM_realloc(dt->shared->u.enumer.name, n*sizeof(char*))))
+	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
+	dt->shared->u.enumer.name = names;
 
-	if (NULL==(values=H5MM_realloc(dt->u.enumer.value, n*dt->size)))
-	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed");
-	dt->u.enumer.value = values;
-	dt->u.enumer.nalloc = n;
+	if (NULL==(values=H5MM_realloc(dt->shared->u.enumer.value, n*dt->shared->size)))
+	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
+	dt->shared->u.enumer.value = values;
+	dt->shared->u.enumer.nalloc = n;
     }
 
     /* Insert new member at end of member arrays */
-    dt->u.enumer.sorted = H5T_SORT_NONE;
-    i = dt->u.enumer.nmembs++;
-    dt->u.enumer.name[i] = H5MM_xstrdup(name);
-    HDmemcpy(dt->u.enumer.value+i*dt->size, value, dt->size);
+    dt->shared->u.enumer.sorted = H5T_SORT_NONE;
+    i = dt->shared->u.enumer.nmembs++;
+    dt->shared->u.enumer.name[i] = H5MM_xstrdup(name);
+    HDmemcpy(dt->shared->u.enumer.value+i*dt->shared->size, value, dt->shared->size);
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -281,27 +283,27 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5Tget_member_value(hid_t type, int membno, void *value/*out*/)
+H5Tget_member_value(hid_t type, unsigned membno, void *value/*out*/)
 {
     H5T_t	*dt=NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
     
-    FUNC_ENTER_API(H5Tget_member_value, FAIL);
-    H5TRACE3("e","iIsx",type,membno,value);
+    FUNC_ENTER_API(H5Tget_member_value, FAIL)
+    H5TRACE3("e","iIux",type,membno,value);
 
     if (NULL==(dt=H5I_object_verify(type,H5I_DATATYPE)))
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type");
-    if (H5T_ENUM!=dt->type)
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "operation not defined for data type class");
-    if (membno<0 || membno>=dt->u.enumer.nmembs)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid member number");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type")
+    if (H5T_ENUM!=dt->shared->type)
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "operation not defined for data type class")
+    if (membno>=dt->shared->u.enumer.nmembs)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid member number")
     if (!value)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "null value buffer");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "null value buffer")
 
     if (H5T_get_member_value(dt, membno, value)<0)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get member value");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get member value")
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -324,19 +326,19 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5T_get_member_value(H5T_t *dt, int membno, void *value/*out*/)
+H5T_get_member_value(const H5T_t *dt, unsigned membno, void *value/*out*/)
 {
     herr_t      ret_value=SUCCEED;       /* Return value */
     
-    FUNC_ENTER_NOAPI(H5T_get_member_value, FAIL);
+    FUNC_ENTER_NOAPI(H5T_get_member_value, FAIL)
 
     assert(dt);
     assert(value);
 
-    HDmemcpy(value, dt->u.enumer.value + membno*dt->size, dt->size);
+    HDmemcpy(value, dt->shared->u.enumer.value + membno*dt->shared->size, dt->shared->size);
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -364,29 +366,29 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5Tenum_nameof(hid_t type, void *value, char *name/*out*/, size_t size)
+H5Tenum_nameof(hid_t type, const void *value, char *name/*out*/, size_t size)
 {
     H5T_t	*dt = NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_API(H5Tenum_nameof, FAIL);
+    FUNC_ENTER_API(H5Tenum_nameof, FAIL)
     H5TRACE4("e","ixxz",type,value,name,size);
 
     /* Check args */
     if (NULL==(dt=H5I_object_verify(type,H5I_DATATYPE)))
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type");
-    if (H5T_ENUM!=dt->type)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type")
+    if (H5T_ENUM!=dt->shared->type)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type")
     if (!value)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value supplied");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value supplied")
     if (!name)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name buffer supplied");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name buffer supplied")
 
     if (NULL==H5T_enum_nameof(dt, value, name, size))
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "nameof query failed");
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "nameof query failed")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -412,37 +414,44 @@ done:
  *              Monday, January  4, 1999
  *
  * Modifications:
- *
+ *              Raymond Lu
+ *              Wednesday, Febuary 9, 2005
+ *              Made a copy of original datatype and do sorting and search
+ *              on that copy, to protect the original order of members.
  *-------------------------------------------------------------------------
  */
 static char *
-H5T_enum_nameof(H5T_t *dt, void *value, char *name/*out*/, size_t size)
+H5T_enum_nameof(const H5T_t *dt, const void *value, char *name/*out*/, size_t size)
 {
-    int	lt, md, rt;		/*indices for binary search	*/
-    int	cmp=(-1);		/*comparison result		*/
-    char *ret_value;            /* Return value */
+    unsigned	lt, md=0, rt;		/*indices for binary search	*/
+    int	        cmp=(-1);		/*comparison result		*/
+    H5T_t       *copied_dt = NULL;      /*do sorting in copied datatype */
+    char        *ret_value;             /* Return value                 */
     
-    FUNC_ENTER_NOAPI(H5T_enum_nameof, NULL);
+    FUNC_ENTER_NOAPI(H5T_enum_nameof, NULL)
 
     /* Check args */
-    assert(dt && H5T_ENUM==dt->type);
+    assert(dt && H5T_ENUM==dt->shared->type);
     assert(value);
     assert(name || 0==size);
     if (name && size>0) *name = '\0';
 
     /* Sanity check */
-    if (dt->u.enumer.nmembs == 0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, NULL, "datatype has no members");
+    if (dt->shared->u.enumer.nmembs == 0)
+        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, NULL, "datatype has no members")
 
-    /* Do a binary search over the values to find the correct one */
-    H5T_sort_value(dt, NULL);
+    /* Do a binary search over the values to find the correct one.  Do sorting
+     * and search on the copied datatype to protect the original order. */
+    if (NULL==(copied_dt=H5T_copy(dt, H5T_COPY_ALL)))
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, NULL, "unable to copy data type");
+    if(H5T_sort_value(copied_dt, NULL)<0)
+        HGOTO_ERROR(H5E_INTERNAL, H5E_CANTCOMPARE, NULL, "value sort failed")
     lt = 0;
-    rt = dt->u.enumer.nmembs;
-    md = -1;
+    rt = copied_dt->shared->u.enumer.nmembs;
 
     while (lt<rt) {
 	md = (lt+rt)/2;
-	cmp = HDmemcmp(value, dt->u.enumer.value+md*dt->size, dt->size);
+	cmp = HDmemcmp(value, copied_dt->shared->u.enumer.value+md*copied_dt->shared->size, copied_dt->shared->size);
 	if (cmp<0) {
 	    rt = md;
 	} else if (cmp>0) {
@@ -453,20 +462,23 @@ H5T_enum_nameof(H5T_t *dt, void *value, char *name/*out*/, size_t size)
     }
     /* Value was not yet defined. This fixes bug # 774, 2002/06/05 EIP */
     if (cmp!=0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, NULL, "value is currently not defined");
+        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, NULL, "value is currently not defined")
 
     /* Save result name */
-    if (!name && NULL==(name=H5MM_malloc(HDstrlen(dt->u.enumer.name[md])+1)))
+    if (!name && NULL==(name=H5MM_malloc(HDstrlen(copied_dt->shared->u.enumer.name[md])+1)))
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
-    HDstrncpy(name, dt->u.enumer.name[md], size);
-    if (HDstrlen(dt->u.enumer.name[md])>=size)
-	HGOTO_ERROR(H5E_DATATYPE, H5E_NOSPACE, NULL, "name has been truncated");
+    HDstrncpy(name, copied_dt->shared->u.enumer.name[md], size);
+    if (HDstrlen(copied_dt->shared->u.enumer.name[md])>=size)
+	HGOTO_ERROR(H5E_DATATYPE, H5E_NOSPACE, NULL, "name has been truncated")
 
     /* Set return value */
     ret_value=name;
 
+    if (H5T_close(copied_dt)<0)
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTCLOSEOBJ, NULL, "unable to close data type");
+
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -495,24 +507,24 @@ H5Tenum_valueof(hid_t type, const char *name, void *value/*out*/)
     H5T_t	*dt = NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_API(H5Tenum_valueof, FAIL);
+    FUNC_ENTER_API(H5Tenum_valueof, FAIL)
     H5TRACE3("e","isx",type,name,value);
 
     /* Check args */
     if (NULL==(dt=H5I_object_verify(type,H5I_DATATYPE)))
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type");
-    if (H5T_ENUM!=dt->type)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data type")
+    if (H5T_ENUM!=dt->shared->type)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an enumeration data type")
     if (!name || !*name)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name")
     if (!value)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value buffer");
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no value buffer")
 
     if (H5T_enum_valueof(dt, name, value)<0)
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "valueof query failed");
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "valueof query failed")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -532,36 +544,43 @@ done:
  *              Monday, January  4, 1999
  *
  * Modifications:
- *
+ *              Raymond Lu
+ *              Wednesday, Febuary 9, 2005
+ *              Made a copy of original datatype and do sorting and search
+ *              on that copy, to protect the original order of members.
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5T_enum_valueof(H5T_t *dt, const char *name, void *value/*out*/)
+H5T_enum_valueof(const H5T_t *dt, const char *name, void *value/*out*/)
 {
-    int	lt, md, rt;		/*indices for binary search	*/
-    int	cmp=(-1);		/*comparison result		*/
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    unsigned	lt, md=0, rt;		/*indices for binary search	*/
+    int	        cmp=(-1);		/*comparison result		*/
+    H5T_t       *copied_dt = NULL;      /*do sorting in copied datatype */
+    herr_t      ret_value=SUCCEED;      /* Return value */
     
-    FUNC_ENTER_NOAPI(H5T_enum_valueof, FAIL);
+    FUNC_ENTER_NOAPI(H5T_enum_valueof, FAIL)
 
     /* Check args */
-    assert(dt && H5T_ENUM==dt->type);
+    assert(dt && H5T_ENUM==dt->shared->type);
     assert(name && *name);
     assert(value);
 
     /* Sanity check */
-    if (dt->u.enumer.nmembs == 0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "datatype has no members");
+    if (dt->shared->u.enumer.nmembs == 0)
+        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "datatype has no members")
 
-    /* Do a binary search over the names to find the correct one */
-    H5T_sort_name(dt, NULL);
+    /* Do a binary search over the names to find the correct one.  Do sorting
+     * and search on the copied datatype to protect the original order. */
+    if (NULL==(copied_dt=H5T_copy(dt, H5T_COPY_ALL)))
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to copy data type");
+    if(H5T_sort_name(copied_dt, NULL)<0)
+        HGOTO_ERROR(H5E_INTERNAL, H5E_CANTCOMPARE, FAIL, "value sort failed")
     lt = 0;
-    rt = dt->u.enumer.nmembs;
-    md = -1;
+    rt = copied_dt->shared->u.enumer.nmembs;
 
     while (lt<rt) {
 	md = (lt+rt)/2;
-	cmp = HDstrcmp(name, dt->u.enumer.name[md]);
+	cmp = HDstrcmp(name, copied_dt->shared->u.enumer.name[md]);
 	if (cmp<0) {
 	    rt = md;
 	} else if (cmp>0) {
@@ -572,11 +591,13 @@ H5T_enum_valueof(H5T_t *dt, const char *name, void *value/*out*/)
     }
     /* Value was not yet defined. This fixes bug # 774, 2002/06/05 EIP */
     if (cmp!=0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "string doesn't exist in the enumeration type");
+        HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "string doesn't exist in the enumeration type")
 
-    HDmemcpy(value, dt->u.enumer.value+md*dt->size, dt->size);
+    HDmemcpy(value, copied_dt->shared->u.enumer.value+md*copied_dt->shared->size, copied_dt->shared->size);
+
+    if (H5T_close(copied_dt)<0)
+	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTCLOSEOBJ, FAIL, "unable to close data type");
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
-
