@@ -66,9 +66,11 @@
 /*
  * C9x integer types
  */
+#ifndef __cplusplus
 #ifdef H5_HAVE_STDINT_H
 #   include <stdint.h>
 #endif
+#endif
 
 /*
  * The `struct stat' data type for stat() and fstat(). This is a Posix file
@@ -156,16 +158,6 @@
 #   define R_OK 04
 #endif
 
-/*
- * Pablo support files.
- */
-#ifdef H5_HAVE_PABLO
-#   define IOTRACE
-#   define HDFIOTRACE
-#   include "HDFIOTrace.h"
-#   include "ProcIDs.h"
-#endif
-
 /*
  * MPE Instrumentation support
  * Do not #if the following header file because it contains
@@ -287,7 +279,7 @@
 #   define FALSE 0
 #endif
 #ifndef TRUE
-#   define TRUE (!FALSE)
+#   define TRUE 1
 #endif
 
 /*
@@ -434,6 +426,7 @@
 #ifndef LLONG_MAX
 #   define LLONG_MAX	((long_long)(((unsigned long_long)1		      \
 				      <<(8*sizeof(long_long)-1))-1))
+#   define LLONG_MIN    ((long_long)(-LLONG_MAX)-1)
 #   define ULLONG_MAX	((unsigned long_long)((long_long)(-1)))
 #endif
 #ifndef SIZET_MAX
@@ -449,15 +442,6 @@
 #endif /* H5_HAVE_LARGE_HSIZET */
 #define HSSIZET_MIN	(~(HSSIZET_MAX))
 
-/*
- * Some compilers have problems declaring auto variables that point
- * to string constants.	 Use the CONSTR() macro so it's easy to fix
- * those compilers.
- */
-#ifndef CONSTR
-#  define CONSTR(VAR,STR) static const char VAR[]=STR
-#endif
-
 /*
  * A macro to portably increment enumerated types.
  */
@@ -489,16 +473,16 @@
  * A macro for detecting over/under-flow when assigning between types
  */
 #ifndef NDEBUG
-#define H5_ASSIGN_OVERFLOW(var,expr,vartype,casttype)   \
+#define H5_ASSIGN_OVERFLOW(var,expr,exprtype,vartype)   \
 {                                                       \
-    vartype _tmp_overflow=(vartype)(expr);              \
-    casttype _tmp_overflow2=(casttype)(_tmp_overflow);  \
-    assert((casttype)_tmp_overflow==_tmp_overflow2);    \
+    exprtype _tmp_overflow=(exprtype)(expr);              \
+    vartype _tmp_overflow2=(vartype)(_tmp_overflow);  \
+    assert((vartype)_tmp_overflow==_tmp_overflow2);    \
     (var)=_tmp_overflow2;                               \
 }
 #else /* NDEBUG */
-#define H5_ASSIGN_OVERFLOW(var,expr,vartype,casttype)   \
-    (var)=(casttype)(expr);
+#define H5_ASSIGN_OVERFLOW(var,expr,exprtype,vartype)   \
+    (var)=(vartype)(expr);
 #endif /* NDEBUG */
 
 /*
@@ -578,6 +562,9 @@ H5_DLL void H5_bandwidth(char *buf/*out*/, double nbytes, double nseconds);
 #endif /* __MWERKS __ */
 #define HDexp(X)		exp(X)
 #define HDfabs(X)		fabs(X)
+/* use ABS() because fabsf() fabsl() are not common yet. */
+#define HDfabsf(X)		ABS(X)
+#define HDfabsl(X)		ABS(X)
 #define HDfclose(F)		fclose(F)
 /* fcntl() variable arguments */
 #define HDfdopen(N,S)		fdopen(N,S)
@@ -587,7 +574,11 @@ H5_DLL void H5_bandwidth(char *buf/*out*/, double nbytes, double nseconds);
 #define HDfgetc(F)		fgetc(F)
 #define HDfgetpos(F,P)		fgetpos(F,P)
 #define HDfgets(S,N,F)		fgets(S,N,F)
+#ifdef WIN32
+#define HDfileno(F)		_fileno(F)
+#else /* WIN32 */
 #define HDfileno(F)		fileno(F)
+#endif /* WIN32 */
 #define HDfloor(X)		floor(X)
 #define HDfmod(X,Y)		fmod(X,Y)
 #define HDfopen(S,M)		fopen(S,M)
@@ -617,15 +608,18 @@ H5_DLL int HDfprintf (FILE *stream, const char *fmt, ...);
 /* definitions related to the file stat utilities */
 #ifdef WIN32
      #ifdef __MWERKS__
-     #define HDfstat(F,B)            fstat(F,B)
+     #define HDfstat(F,B)               fstat(F,B)
      typedef struct stat		h5_stat_t;
+     typedef off_t                      h5_stat_size_t;
      #else /*MSVC*/
      #define HDfstat(F,B)		_fstati64(F,B)
      typedef struct _stati64		h5_stat_t;
+     typedef __int64                    h5_stat_size_t;
      #endif
 #else
 #define HDfstat(F,B)            fstat(F,B)
 typedef struct stat		h5_stat_t;
+typedef off_t                   h5_stat_size_t;
 #endif
 
 #define HDftell(F)		ftell(F)
@@ -711,7 +705,7 @@ typedef struct stat		h5_stat_t;
 #define HDmkfifo(S,M)		mkfifo(S,M)
 #define HDmktime(T)		mktime(T)
 #define HDmodf(X,Y)		modf(X,Y)
-#ifdef O_BINARY
+#ifdef _O_BINARY
 #define HDopen(S,F,M)		open(S,F|_O_BINARY,M)
 #else
 #define HDopen(S,F,M)		open(S,F,M)
@@ -767,11 +761,7 @@ typedef struct stat		h5_stat_t;
 #define HDsinh(X)		sinh(X)
 #define HDsleep(N)		sleep(N)
 #ifdef H5_HAVE_SNPRINTF
-#ifdef __WATCOMC__
-#   define HDsnprintf		_snprintf /*varargs*/
-#else /* __WATCOMC__ */
 #   define HDsnprintf		snprintf /*varargs*/
-#endif /* __WATCOMC__ */
 #endif
 /* sprintf() variable arguments */
 #define HDsqrt(X)		sqrt(X)
@@ -839,7 +829,11 @@ H5_DLL int64_t HDstrtoll (const char *s, const char **rest, int base);
 #define HDumask(N)		umask(N)
 #define HDuname(S)		uname(S)
 #define HDungetc(C,F)		ungetc(C,F)
+#ifdef WIN32
+#define HDunlink(S)             _unlink(S)
+#else
 #define HDunlink(S)		unlink(S)
+#endif
 #define HDutime(S,T)		utime(S,T)
 #define HDva_arg(A,T)		va_arg(A,T)
 #define HDva_end(A)		va_end(A)
@@ -977,9 +971,13 @@ extern H5_debug_t		H5_debug_g;
 #define H5TRACE9(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8) RTYPE=R;                                               \
                                            CALLTIME=H5_trace(NULL,FUNC,T,#A0,A0,#A1,A1,#A2,A2,#A3,A3,   \
                                                              #A4,A4,#A5,A5,#A6,A6,#A7,A7,#A8,A8)
-#define H5TRACE10(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8,A9) RTYPE=R;                                               \
+#define H5TRACE10(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8,A9) RTYPE=R;                                           \
                                            CALLTIME=H5_trace(NULL,FUNC,T,#A0,A0,#A1,A1,#A2,A2,#A3,A3,   \
                                                              #A4,A4,#A5,A5,#A6,A6,#A7,A7,#A8,A8,#A9,A9)
+#define H5TRACE11(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10) RTYPE=R;                                       \
+                                           CALLTIME=H5_trace(NULL,FUNC,T,#A0,A0,#A1,A1,#A2,A2,#A3,A3,   \
+                                                             #A4,A4,#A5,A5,#A6,A6,#A7,A7,#A8,A8,#A9,A9, \
+                                                             #A10,A10)
 #define H5TRACE_RETURN(V)		   if (RTYPE) {                                                 \
 					      H5_trace(&CALLTIME,FUNC,RTYPE,NULL,V);                    \
 					      RTYPE=NULL;                                               \
@@ -997,10 +995,11 @@ extern H5_debug_t		H5_debug_g;
 #define H5TRACE8(R,T,A0,A1,A2,A3,A4,A5,A6,A7)           /*void*/
 #define H5TRACE9(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8)        /*void*/
 #define H5TRACE10(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8,A9)    /*void*/
+#define H5TRACE11(R,T,A0,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10) /*void*/
 #define H5TRACE_RETURN(V)		                /*void*/
 #endif
 
-H5_DLL double H5_trace(double *calltime, const char *func, const char *type, ...);
+H5_DLL double H5_trace(const double *calltime, const char *func, const char *type, ...);
 
 
 /*-------------------------------------------------------------------------
@@ -1022,37 +1021,6 @@ H5_DLL double H5_trace(double *calltime, const char *func, const char *type, ...
  *
  * Modifications:
  *
- *	Robb Matzke, 4 Aug 1997
- *	The `interface_init_func' can be the null pointer.  Changed
- *	HGOTO_ERROR() to HRETURN_ERROR() since no clean-up needs to occur
- *	when an error is detected at this point since this must be the
- *	first executable statement in a function.  This allows functions
- *	to omit the `done:' label when convenient to do so.
- *
- *	Robb Matzke, 4 Aug 1997
- *	The pablo mask comes from the constant PABLO_MASK defined on
- *	a per-file basis.  The `pablo_func_id' is generated from the
- *	`func_name' argument by prepending an `ID_' to the name.  The
- *	pablo function identifier should be saved in a local variable
- *	so FUNC_LEAVE() can access it.
- *
- *	Robb Matzke, 4 Aug 1997
- *	It is safe to call this function even inside various library
- *	initializing functions.	 Infinite recursion is no longer a
- *	danger.
- *
- *	Robb Matzke, 3 Dec 1997
- *	The interface initialization function is no longer passed as an
- *	argument unless the `FUNC_ENTER_INIT' form is called.  Instead, the
- *	function comes from the `INTERFACE_INIT' constant which must be
- *	defined in every source file.
- *
- *	Robb Matzke, 17 Jun 1998
- *	Added auto variable RTYPE which is initialized by the tracing macros.
- *
- *	Quincey Koziol, 28 May 2002
- *	Split FUNC_ENTER macro into FUNC_ENTER_API, FUNC_ENTER_NOAPI and
- *      FUNC_ENTER_NOAPI_NOINIT.
  *-------------------------------------------------------------------------
  */
 
@@ -1113,7 +1081,6 @@ extern H5_api_t H5_g;
 /* extern global variables */
 extern hbool_t H5_libinit_g;    /* Has the library been initialized? */
 
-
 /* Macros for accessing the global variables */
 #define H5_INIT_GLOBAL H5_libinit_g
 
@@ -1135,18 +1102,44 @@ extern hbool_t H5_libinit_g;    /* Has the library been initialized? */
 extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
 #endif
 
+/* Check if the function name is correct (if the compiler supports __FUNCTION__) */
+#ifdef H5_HAVE_FUNCTION
+#define H5_CHECK_FUNCNAME(func_name) \
+    assert(!HDstrcmp(#func_name, __FUNCTION__))
+#else /* H5_HAVE_FUNCTION */
+#define H5_CHECK_FUNCNAME(func_name) \
+    assert(func_name)
+#endif /* H5_HAVE_FUNCTION */
+
+/* Macros for defining interface initialization routines */
+#ifdef H5_INTERFACE_INIT_FUNC
+static int		H5_interface_initialize_g = 0;
+static herr_t		H5_INTERFACE_INIT_FUNC(void);
+#define H5_INTERFACE_INIT(err)						      \
+   /* Initialize this interface or bust */				      \
+   if (!H5_interface_initialize_g) {					      \
+      H5_interface_initialize_g = 1;					      \
+      if (H5_INTERFACE_INIT_FUNC()<0) {					      \
+         H5_interface_initialize_g = 0;				              \
+         HGOTO_ERROR (H5E_FUNC, H5E_CANTINIT, err,		              \
+            "interface initialization failed")		                      \
+      }								              \
+   }
+#else /* H5_INTERFACE_INIT_FUNC */
+#define H5_INTERFACE_INIT(err)
+#endif /* H5_INTERFACE_INIT_FUNC */
+
+
 #define FUNC_ENTER_COMMON_NOFUNC(func_name,asrt)                              \
-   PABLO_SAVE (ID_ ## func_name)  					      \
-									      \
    /* Check API status */               				      \
    assert(asrt);				                              \
 									      \
-   /* Start tracing */                  				      \
-   PABLO_TRACE_ON (PABLO_MASK, pablo_func_id)
+   /* Check function name */               				      \
+   H5_CHECK_FUNCNAME(func_name);
 
 #define FUNC_ENTER_COMMON(func_name,asrt)                                     \
-   CONSTR (FUNC, #func_name);						      \
-   FUNC_ENTER_COMMON_NOFUNC(func_name,asrt);                                  \
+    static const char FUNC[]=#func_name;                                      \
+    FUNC_ENTER_COMMON_NOFUNC(func_name,asrt);
 
 /* Threadsafety initialization code for API routines */
 #define FUNC_ENTER_API_THREADSAFE                                             \
@@ -1172,7 +1165,7 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
     FUNC_ENTER_API_VARS(func_name)                                            \
     FUNC_ENTER_COMMON(func_name,H5_IS_API(#func_name));                       \
     FUNC_ENTER_API_THREADSAFE;                                                \
-    FUNC_ENTER_API_COMMON(func_name,INTERFACE_INIT,err);                      \
+    FUNC_ENTER_API_COMMON(func_name,err);		                      \
     /* Clear thread error stack entering public functions */		      \
     H5E_clear();						              \
     {
@@ -1185,7 +1178,7 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
     FUNC_ENTER_API_VARS(func_name)                                            \
     FUNC_ENTER_COMMON(func_name,H5_IS_API(#func_name));                       \
     FUNC_ENTER_API_THREADSAFE;                                                \
-    FUNC_ENTER_API_COMMON(func_name,INTERFACE_INIT,err);                      \
+    FUNC_ENTER_API_COMMON(func_name,err);		                      \
     {
 
 /*
@@ -1205,7 +1198,13 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
 /* Use this macro for all "normal" non-API functions */
 #define FUNC_ENTER_NOAPI(func_name,err) {                                     \
     FUNC_ENTER_COMMON(func_name,!H5_IS_API(#func_name));                      \
-    FUNC_ENTER_NOAPI_INIT(func_name,INTERFACE_INIT,err)                       \
+    FUNC_ENTER_NOAPI_INIT(func_name,err)		                      \
+    {
+
+/* Use this macro for all non-API functions which don't issue errors */
+#define FUNC_ENTER_NOAPI_NOFUNC(func_name) {                                  \
+    FUNC_ENTER_COMMON_NOFUNC(func_name,!H5_IS_API(#func_name));               \
+    FUNC_ENTER_NOAPI_INIT(func_name,err)		                      \
     {
 
 /*
@@ -1242,47 +1241,35 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
  * Use this macro for non-API functions which fall into these categories:
  *      - functions which shouldn't push their name on the function stack
  *              (so far, just the H5FS routines themselves)
+ * 
+ * This macro is used for functions which fit the above categories _and_
+ * also don't use the 'FUNC' variable (i.e. don't push errors on the error stack)
+ * 
  */
-#define FUNC_ENTER_NOAPI_NOFS(func_name) {                                    \
-    FUNC_ENTER_COMMON(func_name,!H5_IS_API(#func_name));                      \
+#define FUNC_ENTER_NOAPI_NOFUNC_NOFS(func_name) {                             \
+    FUNC_ENTER_COMMON_NOFUNC(func_name,!H5_IS_API(#func_name));               \
     {
 
-#define FUNC_ENTER_API_COMMON(func_name,interface_init_func,err)       	      \
+#define FUNC_ENTER_API_COMMON(func_name,err)			       	      \
    /* Initialize the library */           				      \
    if (!(H5_INIT_GLOBAL)) {                                                   \
        H5_INIT_GLOBAL = TRUE;                                                 \
        if (H5_init_library()<0)  					      \
           HGOTO_ERROR (H5E_FUNC, H5E_CANTINIT, err,		              \
-            "library initialization failed");		                      \
+            "library initialization failed")		                      \
    }								              \
                                                                               \
-   /* Initialize this interface or bust */				      \
-   if (!interface_initialize_g) {					      \
-      interface_initialize_g = 1;					      \
-      if (interface_init_func &&					      \
-              ((herr_t(*)(void))interface_init_func)()<0) {		      \
-         interface_initialize_g = 0;				              \
-         HGOTO_ERROR (H5E_FUNC, H5E_CANTINIT, err,		              \
-            "interface initialization failed");		                      \
-      }								              \
-   }                                                                          \
+   /* Initialize the interface, if appropriate */		              \
+   H5_INTERFACE_INIT(err)						      \
                                                                               \
    /* Push the name of this function on the function stack */                 \
    H5_PUSH_FUNC(func_name);                                                   \
                                                                               \
    BEGIN_MPE_LOG(func_name)
 
-#define FUNC_ENTER_NOAPI_INIT(func_name,interface_init_func,err)       	      \
-   /* Initialize this interface or bust */				      \
-   if (!interface_initialize_g) {					      \
-      interface_initialize_g = 1;					      \
-      if (interface_init_func &&					      \
-              ((herr_t(*)(void))interface_init_func)()<0) {		      \
-         interface_initialize_g = 0;				              \
-         HGOTO_ERROR (H5E_FUNC, H5E_CANTINIT, err,		              \
-            "interface initialization failed");		                      \
-      }								              \
-   }                                                                          \
+#define FUNC_ENTER_NOAPI_INIT(func_name,err)			       	      \
+   /* Initialize the interface, if appropriate */		              \
+   H5_INTERFACE_INIT(err)						      \
                                                                               \
    /* Push the name of this function on the function stack */                 \
    H5_PUSH_FUNC(func_name);
@@ -1299,12 +1286,12 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
  *	The pablo mask comes from the constant PABLO_MASK defined on a
  *	per-file basis.	 The pablo_func_id comes from an auto variable
  *	defined by FUNC_ENTER.
+ *      PABLO was removed on January 20, 2005 EIP
  *
  *-------------------------------------------------------------------------
  */
 #define FUNC_LEAVE_API(ret_value)                                             \
         FINISH_MPE_LOG;                                                       \
-        PABLO_TRACE_OFF (PABLO_MASK, pablo_func_id);			      \
         H5TRACE_RETURN(ret_value);					      \
         H5_POP_FUNC;                                                          \
         FUNC_LEAVE_API_THREADSAFE                                             \
@@ -1313,14 +1300,12 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
 }} /*end scope from beginning of FUNC_ENTER*/
 
 #define FUNC_LEAVE_NOAPI(ret_value)                                           \
-        PABLO_TRACE_OFF (PABLO_MASK, pablo_func_id);			      \
         H5_POP_FUNC;                                                          \
         return (ret_value);						      \
     } /*end scope from end of FUNC_ENTER*/                                    \
 } /*end scope from beginning of FUNC_ENTER*/
 
 #define FUNC_LEAVE_NOAPI_VOID                                                 \
-        PABLO_TRACE_OFF (PABLO_MASK, pablo_func_id);			      \
         H5_POP_FUNC;                                                          \
         return;						                      \
     } /*end scope from end of FUNC_ENTER*/                                    \
@@ -1332,24 +1317,10 @@ extern hbool_t H5_MPEinit_g;   /* Has the MPE Library been initialized? */
  *              (so far, just the H5FS routines themselves)
  */
 #define FUNC_LEAVE_NOAPI_NOFS(ret_value)                                      \
-        PABLO_TRACE_OFF (PABLO_MASK, pablo_func_id);			      \
         return (ret_value);						      \
     } /*end scope from end of FUNC_ENTER*/                                    \
 } /*end scope from beginning of FUNC_ENTER*/
 
-/*
- * The FUNC_ENTER() and FUNC_LEAVE() macros make calls to Pablo functions
- * through one of these two sets of macros.
- */
-#ifdef H5_HAVE_PABLO
-#  define PABLO_SAVE(func_id)	const int pablo_func_id = func_id;
-#  define PABLO_TRACE_ON(m, f)	TRACE_ON(m,f)
-#  define PABLO_TRACE_OFF(m, f) TRACE_OFF(m,f)
-#else
-#  define PABLO_SAVE(func_id)	/*void */
-#  define PABLO_TRACE_ON(m, f)	/*void */
-#  define PABLO_TRACE_OFF(m, f) /*void */
-#endif
 
 /* Macro for "glueing" together items, for re-scanning macros */
 #define H5_GLUE(x,y)       x##y
@@ -1369,7 +1340,6 @@ H5_DLL int H5I_term_interface(void);
 H5_DLL int H5P_term_interface(void);
 H5_DLL int H5R_term_interface(void);
 H5_DLL int H5S_term_interface(void);
-H5_DLL int H5TN_term_interface(void);
 H5_DLL int H5T_term_interface(void);
 H5_DLL int H5Z_term_interface(void);
 
