@@ -28,6 +28,8 @@ const char *FILENAME[] = {
 
 #define DIM0    100
 #define DIM1    200
+#define DIM3    20
+
 
 int	ipoints2[DIM0][DIM1], icheck2[DIM0][DIM1];
 short	spoints2[DIM0][DIM1], scheck2[DIM0][DIM1];
@@ -137,7 +139,7 @@ test_atomic_dtype(hid_t file)
     free(tmp);
     
     /* Convert to the integer type */
-    if(H5Tconvert(native_type, H5T_NATIVE_INT, (hsize_t)(DIM0*DIM1), icheck2, NULL, H5P_DEFAULT)<0)
+    if(H5Tconvert(native_type, H5T_NATIVE_INT, (DIM0*DIM1), icheck2, NULL, H5P_DEFAULT)<0)
         TEST_ERROR;
     
     /* Check that the values read are the same as the values written */
@@ -152,8 +154,9 @@ test_atomic_dtype(hid_t file)
 	}
     }
 
-    H5Dclose(dataset);
-    H5Tclose(dtype);
+    if(H5Dclose(dataset)<0) TEST_ERROR;
+    if(H5Tclose(native_type)<0) TEST_ERROR;
+    if(H5Tclose(dtype)<0) TEST_ERROR;
 
     /*------------------ Test different data types ----------------*/
 
@@ -175,6 +178,7 @@ test_atomic_dtype(hid_t file)
         TEST_ERROR;
         
     if(H5Dclose(dataset)<0) TEST_ERROR;
+    if(H5Tclose(native_type)<0) TEST_ERROR;
     if(H5Tclose(dtype)<0) TEST_ERROR;
 
 
@@ -196,6 +200,7 @@ test_atomic_dtype(hid_t file)
         TEST_ERROR;
         
     if(H5Dclose(dataset)<0) TEST_ERROR;
+    if(H5Tclose(native_type)<0) TEST_ERROR;
     if(H5Tclose(dtype)<0) TEST_ERROR;
     
     
@@ -217,6 +222,7 @@ test_atomic_dtype(hid_t file)
         TEST_ERROR;
         
     if(H5Dclose(dataset)<0) TEST_ERROR;
+    if(H5Tclose(native_type)<0) TEST_ERROR;
     if(H5Tclose(dtype)<0) TEST_ERROR;
        
        
@@ -238,6 +244,7 @@ test_atomic_dtype(hid_t file)
         TEST_ERROR;
         
     if(H5Dclose(dataset)<0) TEST_ERROR;
+    if(H5Tclose(native_type)<0) TEST_ERROR;
     if(H5Tclose(dtype)<0) TEST_ERROR;
     
     
@@ -456,7 +463,7 @@ test_compound_dtype2(hid_t file)
     memcpy(check, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
 
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), check, bkg, H5P_DEFAULT))
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), check, bkg, H5P_DEFAULT))
         TEST_ERROR;
 
     free(bkg);
@@ -638,7 +645,7 @@ test_compound_dtype(hid_t file)
     memcpy(check, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
     
-    if (H5Tconvert(native_type, tid2, (hsize_t)(DIM0*DIM1), check, bkg, H5P_DEFAULT)<0)
+    if (H5Tconvert(native_type, tid2, (DIM0*DIM1), check, bkg, H5P_DEFAULT)<0)
         TEST_ERROR;
                 
     free(bkg);
@@ -832,7 +839,7 @@ test_compound_dtype3(hid_t file)
     memcpy(check, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
 
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), check, bkg, H5P_DEFAULT))
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), check, bkg, H5P_DEFAULT))
         TEST_ERROR;
 
     free(bkg);
@@ -1022,7 +1029,7 @@ test_compound_opaque(hid_t file)
     HDmemcpy(check, tmp, DIM0*DIM1*H5Tget_size(native_type));
     HDfree(tmp);
 
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), check, bkg, H5P_DEFAULT))
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), check, bkg, H5P_DEFAULT))
         TEST_ERROR;
 
     HDfree(bkg);
@@ -1166,7 +1173,7 @@ test_enum_dtype(hid_t file)
     memcpy(scheck2, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
    
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), scheck2, NULL, H5P_DEFAULT)<0)
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), scheck2, NULL, H5P_DEFAULT)<0)
         TEST_ERROR;
 
     /* Check that the values read are the same as the values written */
@@ -1305,7 +1312,7 @@ test_array_dtype(hid_t file)
     memcpy(check, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
     
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), check, NULL, H5P_DEFAULT)<0)
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), check, NULL, H5P_DEFAULT)<0)
         TEST_ERROR;
 
     /* Check that the values read are the same as the values written */
@@ -1427,7 +1434,7 @@ test_array_dtype2(hid_t file)
     memcpy(icheck3, tmp, DIM0*DIM1*H5Tget_size(native_type));
     free(tmp);
 
-    if (H5Tconvert(native_type, tid_m, (hsize_t)(DIM0*DIM1), icheck3, NULL, H5P_DEFAULT)<0)
+    if (H5Tconvert(native_type, tid_m, (DIM0*DIM1), icheck3, NULL, H5P_DEFAULT)<0)
         TEST_ERROR;
 
     /* Check that the values read are the same as the values written */
@@ -1577,7 +1584,7 @@ test_vl_dtype(hid_t file)
             tmp=malloc(t2->len*sizeof(unsigned int));
             memcpy(tmp, t2->p, t2->len*H5Tget_size(nat_super_type));
 
-            if (H5Tconvert(nat_super_type, H5T_NATIVE_UINT, (hsize_t)t2->len, tmp, NULL, H5P_DEFAULT))
+            if (H5Tconvert(nat_super_type, H5T_NATIVE_UINT, t2->len, tmp, NULL, H5P_DEFAULT))
                 TEST_ERROR;
 
             for(k=0; k<t2->len; k++) {
@@ -2018,7 +2025,7 @@ test_refer_dtype2(hid_t file)
     hid_t               dtype, native_type;
     hsize_t		dims1[] = {1},
             	        dims2[] = {SPACE2_DIM1, SPACE2_DIM2};
-    hssize_t	        start[SPACE2_RANK];     /* Starting location of hyperslab */
+    hsize_t	        start[SPACE2_RANK];     /* Starting location of hyperslab */
     hsize_t		stride[SPACE2_RANK];    /* Stride of hyperslab */
     hsize_t		count[SPACE2_RANK];     /* Element count of hyperslab */
     hsize_t		block[SPACE2_RANK];     /* Block size of hyperslab */
@@ -2284,7 +2291,7 @@ test_bitfield_dtype(hid_t file)
     hid_t		type=-1, space=-1, dset=-1;
     hid_t               dataset, dtype, native_type;
     size_t		i;
-    unsigned char	wbuf[32], rbuf[32];
+    unsigned char	wbuf[32];
     hsize_t		nelmts;
     
     TESTING("bitfield datatype");
@@ -2310,26 +2317,17 @@ test_bitfield_dtype(hid_t file)
 
     if((dtype=H5Dget_type(dataset))<0) TEST_ERROR;
     
-    if((native_type=H5Tget_native_type(dtype, H5T_DIR_DEFAULT))<0)
+    H5E_BEGIN_TRY {
+        native_type=H5Tget_native_type(dtype, H5T_DIR_DEFAULT);
+    } H5E_END_TRY;
+    if(native_type>0) {
+        H5_FAILED();
+        puts("  Bit field isn't supported.  Should have failed.");
         TEST_ERROR;
-
-    if(!H5Tequal(native_type, type)) TEST_ERROR;
-
-    if (H5Dread(dataset, native_type, H5S_ALL, H5S_ALL, H5P_DEFAULT, rbuf)<0)
-        TEST_ERROR;
-    
-    for(i=0; i<sizeof(rbuf); i++) {
-	if (rbuf[i] != wbuf[i]) {
-	    H5_FAILED();
-	    printf("    Read different values than written.\n");
-            printf("    At index %u\n", (unsigned)i);
-	    goto error;
-        }
     }
-    
+
     if (H5Tclose(type)<0) TEST_ERROR;
     if (H5Tclose(dtype)<0) TEST_ERROR;
-    if (H5Tclose(native_type)<0) TEST_ERROR;
     if (H5Dclose(dataset)<0) TEST_ERROR;
 
     PASSED();                                                 
@@ -2337,7 +2335,209 @@ test_bitfield_dtype(hid_t file)
                                                                       
 error:                                                       
     return -1;
-} /* test_opaque_dtype */
+} /* test_bitfield_dtype */
+
+
+
+
+
+/*-------------------------------------------------------------------------
+ * Function: test_ninteger
+ *
+ * Purpose: Test the native integer function; made to check the case
+ * like the Cray SV1, where the size of short is 8 but precision is 32
+ *
+ * Return: Success: 0
+ *  Failure: -1
+ *
+ * Programmer: pvn@ncsa.uiuc.edu
+ *  September 3, 2004
+ *
+ * Modifications:
+ *             
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+test_ninteger(void)
+{
+    hid_t     fid1=(-1);             /* file ID */
+    hid_t     fid2=(-1);             /* file ID */
+    hid_t     did1=(-1);             /* dataset ID */
+    hid_t     did2=(-1);             /* dataset ID */
+    hid_t     sid1=(-1);             /* dataspace ID */ 
+    hid_t     dcpl1=(-1);            /* dataset creation property list ID */
+    hid_t     dcpl2=(-1);            /* dataset creation property list ID */
+    hid_t     tid1=(-1);             /* file datatype */
+    hid_t     tid2=(-1);             /* file datatype */
+    hid_t     nid1=(-1);             /* native datatype */
+    hid_t     nid2=(-1);             /* native datatype */
+    hsize_t   dims[1]={DIM3};        /* dataspace dimensions */
+    hsize_t   nelmts;                /* number of elements in dataset */
+    int       rank=1;                /* rank of dataset */
+    int       buf[DIM3];
+    int       chk[DIM3];
+    int       i;
+   
+    for (i=0; i<DIM3; i++){
+     buf[i]=i;
+    }
+    
+    TESTING("native integer ");
+
+   /*-------------------------------------------------------------------------
+    * step1: create a file 
+    *-------------------------------------------------------------------------
+    */  
+    /* create a file using default properties */
+    if ((fid1=H5Fcreate("tstint1.h5",H5F_ACC_TRUNC,H5P_DEFAULT,H5P_DEFAULT))<0) 
+     goto error;
+
+    /* create a data space */
+    if ((sid1 = H5Screate_simple(rank,dims,NULL))<0) goto error;
+
+    /* create dcpl  */
+    if((dcpl1 = H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
+   
+    /* create a dataset */
+    if ((did1 = H5Dcreate(fid1,"dset",H5T_NATIVE_INT,sid1,dcpl1)) <0) goto error;
+
+    /* write */
+    if(H5Dwrite(did1,H5T_NATIVE_INT,H5S_ALL,H5S_ALL,H5P_DEFAULT,buf)<0)
+     goto error;
+
+    /* close  */
+    if (H5Sclose (sid1)<0) goto error;
+    if (H5Pclose (dcpl1)<0) goto error;
+    if (H5Dclose (did1)<0) goto error;
+    if (H5Fclose (fid1)<0) goto error;
+
+   /*-------------------------------------------------------------------------
+    * step 2: open and create another file copying the data from file1
+    *-------------------------------------------------------------------------
+    */ 
+    
+     /* open */
+    if ((fid1=H5Fopen("tstint1.h5",H5F_ACC_RDONLY,H5P_DEFAULT))<0) 
+     goto error;
+
+    /* open dataset */
+    if ((did1=H5Dopen(fid1,"dset"))<0) 
+     goto error;
+    
+    if ((sid1=H5Dget_space(did1))<0) 
+     goto error;
+
+    /* get dcpl */
+    if ((dcpl1=H5Dget_create_plist(did1))<0) 
+     goto error;
+
+    /* get file datatype */
+    if ((tid1=H5Dget_type (did1))<0) 
+     goto error;
+
+    /* get native datatype */
+    if ((nid1=H5Tget_native_type(tid1,H5T_DIR_DEFAULT))<0)
+     goto error;
+
+    /* get size */
+    if (H5Tget_size(nid1)==0)
+     goto error;
+
+    /* get rank */
+    if ((rank=H5Sget_simple_extent_ndims(sid1))<0)
+     goto error;
+    HDmemset(dims, 0, sizeof dims);
+
+    /* get dimension */
+    if (H5Sget_simple_extent_dims(sid1,dims,NULL)<0)
+     goto error;
+    nelmts=1;
+    for (i=0; i<rank; i++) 
+     nelmts*=dims[i];
+     
+    /* read */
+    if (H5Dread(did1,nid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,chk)<0)
+     goto error;
+
+    /* create a file using default properties */
+    if ((fid2=H5Fcreate("tstint2.h5",H5F_ACC_TRUNC,H5P_DEFAULT,H5P_DEFAULT))<0) 
+     goto error;
+
+    /* create a dataset using the native type */
+    if ((did2 = H5Dcreate(fid2,"dset",nid1,sid1,dcpl1)) <0) goto error;
+
+    /* write */
+    if(H5Dwrite(did2,nid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,chk)<0)
+     goto error;
+
+    /* get dcpl */
+    if ((dcpl2=H5Dget_create_plist(did2))<0) 
+     goto error;
+
+    /* get file datatype */
+    if ((tid2=H5Dget_type (did2))<0) 
+     goto error;
+
+    /* get native datatype */
+    if ((nid2=H5Tget_native_type(tid2,H5T_DIR_DEFAULT))<0)
+     goto error;
+
+    /* check */
+    if (H5Tget_precision(nid1)!=H5Tget_precision(nid2)) {
+     printf("    Precision differ.\n");
+     goto error;
+    }
+
+    /* compare dataset creation property lists */
+    if(H5Pequal(dcpl1,dcpl2)<=0) {
+     printf("    Property lists differ.\n");
+     goto error;
+    }
+
+    /* check */
+    for (i = 0; i < DIM3; i++) {
+     if (buf[i] != chk[i]) {
+      H5_FAILED();
+      printf("    Read different values than written.\n");
+      printf("    At index %d\n", i);
+      goto error;
+     }
+    }
+    
+    /* close  */
+    if (H5Sclose (sid1)<0) goto error;
+    if (H5Pclose (dcpl1)<0) goto error;
+    if (H5Pclose (dcpl2)<0) goto error;
+    if (H5Tclose (tid1)<0) goto error;
+    if (H5Tclose (tid2)<0) goto error;
+    if (H5Tclose (nid1)<0) goto error;
+    if (H5Tclose (nid2)<0) goto error;
+    if (H5Dclose (did1)<0) goto error;
+    if (H5Dclose (did2)<0) goto error;
+    if (H5Fclose (fid1)<0) goto error;
+    if (H5Fclose (fid2)<0) goto error;
+    
+    
+    PASSED();
+    return 0;
+    
+error:
+    H5E_BEGIN_TRY {
+     H5Pclose(dcpl1);
+     H5Pclose(dcpl2);
+     H5Tclose(tid1);
+     H5Tclose(tid2);
+     H5Tclose(nid1);
+     H5Tclose(nid2);
+     H5Dclose(did1);
+     H5Dclose(did2);
+     H5Sclose(sid1);
+     H5Fclose(fid1);
+     H5Fclose(fid2);
+    } H5E_END_TRY;
+    return -1;
+} /* end test_ninteger() */
+
 
 
 /*-------------------------------------------------------------------------
@@ -2364,7 +2564,7 @@ main(void)
     
     h5_fixname(FILENAME[0], fapl, filename, sizeof filename);
     if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0)
-	goto error;
+	    goto error;
 
     nerrors += test_atomic_dtype(file)<0 	?1:0;
     nerrors += test_compound_dtype(file)<0 	?1:0;
@@ -2381,6 +2581,7 @@ main(void)
     nerrors += test_refer_dtype2(file)<0 	?1:0;
     nerrors += test_opaque_dtype(file)<0 	?1:0;
     nerrors += test_bitfield_dtype(file)<0 	?1:0;
+    nerrors += test_ninteger()<0  ?1:0;
 
     if (H5Fclose(file)<0) goto error;
     if (nerrors) goto error;
