@@ -32,36 +32,36 @@
 /* Use %ld to print the value because long should cover most cases. */
 /* Used to make certain a return value _is_not_ a value */
 #define CHECK(ret, val, where) do {					      \
-    if (GetTestVerbosity()>=VERBO_HI) print_func("   Call to routine: %15s at line %4d "	      \
+    if (GetTestVerbosity()>=VERBO_HI) print_func("   Call to routine: %15s at line %4d " \
 				"in %s returned %ld \n",		      \
 				where, (int)__LINE__, __FILE__,		      \
 				(long)(ret));				      \
-    if ((ret) == (val)) {							      \
-	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "	      \
-		   "in %s\n", where, (long)(ret), (int)__LINE__, __FILE__);     \
+    if ((ret) == (val)) {						      \
+	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "     \
+		   "in %s\n", where, (long)(ret), (int)__LINE__, __FILE__);   \
 	H5Eprint (stdout);						      \
     }									      \
 } while(0)
 
 #define CHECK_I(ret,where) {						      \
-   if (GetTestVerbosity()>=VERBO_HI) {							      \
+   if (GetTestVerbosity()>=VERBO_HI) {						      \
       print_func("   Call to routine: %15s at line %4d in %s returned %ld\n", \
                  (where), (int)__LINE__, __FILE__, (long)(ret));	      \
    }									      \
    if ((ret)<0) {							      \
-      TestErrPrintf ("*** UNEXPECTED RETURN from %s is %ld line %4d in %s\n",    \
+      TestErrPrintf ("*** UNEXPECTED RETURN from %s is %ld line %4d in %s\n", \
                   (where), (long)(ret), (int)__LINE__, __FILE__);	      \
       H5Eprint (stdout);						      \
    }									      \
 }
 
 #define CHECK_PTR(ret,where) {						      \
-   if (GetTestVerbosity()>=VERBO_HI) {							      \
+   if (GetTestVerbosity()>=VERBO_HI) {					      \
       print_func("   Call to routine: %15s at line %4d in %s returned %p\n",  \
                  (where), (int)__LINE__, __FILE__, (ret));		      \
    }									      \
    if (!(ret)) {							      \
-      TestErrPrintf ("*** UNEXPECTED RETURN from %s is NULL line %4d in %s\n",   \
+      TestErrPrintf ("*** UNEXPECTED RETURN from %s is NULL line %4d in %s\n", \
                   (where), (int)__LINE__, __FILE__);			      \
       H5Eprint (stdout);						      \
    }									      \
@@ -69,27 +69,28 @@
 
 /* Used to make certain a return value _is_ a value */
 #define VERIFY(x, val, where) do {					      \
-    if (GetTestVerbosity()>=VERBO_HI) {							      \
+    if (GetTestVerbosity()>=VERBO_HI) {					      \
 	print_func("   Call to routine: %15s at line %4d in %s had value "    \
-		   "%ld \n", (where), (int)__LINE__, __FILE__, (long)(x));	      \
+		   "%ld \n", (where), (int)__LINE__, __FILE__, (long)(x));    \
     }									      \
     if ((x) != (val)) {							      \
-	TestErrPrintf("*** UNEXPECTED VALUE from %s should be %ld, but is %ld at line %4d "	      \
-		   "in %s\n", (where), (long)(val), (long)(x), (int)__LINE__, __FILE__);	      \
+	TestErrPrintf("*** UNEXPECTED VALUE from %s should be %ld, but is %ld at line %4d " \
+		   "in %s\n", (where), (long)(val), (long)(x), (int)__LINE__, __FILE__); \
 	H5Eprint (stdout);						      \
     }									      \
 } while(0)
 
 /* Used to document process through a test and to check for errors */
 #define RESULT(ret,func) do {						      \
-    if (GetTestVerbosity()>VERBO_MED) {							      \
+    if (GetTestVerbosity()>VERBO_MED) {					      \
 	print_func("   Call to routine: %15s at line %4d in %s returned "     \
-		   "%ld\n", func, (int)__LINE__, __FILE__, (long)(ret));	      \
+		   "%ld\n", func, (int)__LINE__, __FILE__, (long)(ret));      \
     }									      \
-    if (GetTestVerbosity()>=VERBO_HI) H5Eprint(stdout);					      \
-    if ((ret) == FAIL) {							      \
-	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "	      \
-		   "in %s\n", func, (long)(ret), (int)__LINE__, __FILE__);      \
+    if (GetTestVerbosity()>=VERBO_HI)					      \
+        H5Eprint(stdout);					              \
+    if ((ret) == FAIL) {						      \
+	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "     \
+		   "in %s\n", func, (long)(ret), (int)__LINE__, __FILE__);    \
 	H5Eprint (stdout);						      \
     }									      \
 } while(0)
@@ -107,25 +108,8 @@
 extern "C" {
 #endif
 
-/* Routines for operating on the list of tests (for the "all in one" tests) */
-H5TEST_DLL void TestUsage(void);
-H5TEST_DLL void AddTest(const char *TheName, void (*TheCall) (void),
-		     void (*Cleanup) (void), const char *TheDescr);
-H5TEST_DLL void TestInfo(const char *ProgName);
-H5TEST_DLL void TestParseCmdLine(int argc, char *argv[], int *Summary, int *CleanUp);
-H5TEST_DLL void PerformTests(void);
-H5TEST_DLL void TestSummary(void);
-H5TEST_DLL void TestCleanup(void);
-H5TEST_DLL void TestInit(void);
-H5TEST_DLL int  GetTestVerbosity(void);
-H5TEST_DLL int  SetTestVerbosity(int newval);
-H5TEST_DLL void ParseTestVerbosity(char *argv);
-H5TEST_DLL int  GetTestNumErrs(void);
-H5TEST_DLL int  TestErrPrintf(const char *format, ...);
-
 /* Prototypes for the test routines */
 void                    test_metadata(void);
-void                    test_tbbt(void);
 void                    test_tst(void);
 void                    test_heap(void);
 void                    test_refstr(void);
@@ -144,6 +128,7 @@ void                    test_array(void);
 void                    test_genprop(void);
 void			test_configure(void);
 void			test_misc(void);
+void			test_skiplist(void);
 
 /* Prototypes for the cleanup routines */
 void                    cleanup_metadata(void);
