@@ -12,8 +12,6 @@
  * access to either file, you may request a copy from hdfhelp@ncsa.uiuc.edu. *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-/* $Id: tgenprop.c,v 1.26.2.1 2004/01/10 02:41:21 koziol Exp $ */
-
 /***********************************************************
 *
 * Test program:	 tgenprop
@@ -87,9 +85,8 @@ test_genprop_basic_class(void)
     /* Check class name */
     name = H5Pget_class_name(cid1);
     CHECK_PTR(name, "H5Pget_class_name");
-    if(HDstrcmp(name,CLASS1_NAME)!=0) {
+    if(HDstrcmp(name,CLASS1_NAME)!=0)
         TestErrPrintf("Class names don't match!, name=%s, CLASS1_NAME=%s\n",name,CLASS1_NAME);
-    } /* end if */
     free(name);
 
     /* Check class parent */
@@ -119,9 +116,8 @@ test_genprop_basic_class(void)
     /* Check class name */
     name = H5Pget_class_name(cid1);
     CHECK_PTR(name, "H5Pget_class_name");
-    if(HDstrcmp(name,CLASS2_NAME)!=0) {
+    if(HDstrcmp(name,CLASS2_NAME)!=0)
         TestErrPrintf("Class names don't match!, name=%s, CLASS2_NAME=%s\n",name,CLASS2_NAME);
-    } /* end if */
     free(name);
 
     /* Check class parent */
@@ -682,9 +678,8 @@ test_genprop_basic_list_prop(void)
     /* Check values of temporary properties (set with regular values) */
     ret = H5Pget(lid1,PROP3_NAME,&prop3_value);
     CHECK_I(ret, "H5Pget");
-    if(memcmp(&prop3_value,PROP3_DEF_VALUE,PROP3_SIZE)!=0) {
+    if(HDmemcmp(&prop3_value,PROP3_DEF_VALUE,PROP3_SIZE)!=0)
         TestErrPrintf("Property #3 doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
     ret = H5Pget(lid1,PROP4_NAME,&prop4_value);
     CHECK_I(ret, "H5Pget");
     VERIFY(prop4_value, *PROP4_DEF_VALUE, "H5Pget");
@@ -1058,12 +1053,10 @@ test_genprop_list_callback(void)
 
     /* Verify creation callback information for properties tracked */
     VERIFY(prop1_cb_info.crt_count, 1, "H5Pcreate");
-    if(HDstrcmp(prop1_cb_info.crt_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.crt_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.crt_value,PROP1_DEF_VALUE,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.crt_value,PROP1_DEF_VALUE,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Check values of permanent properties (set with default values) */
     ret = H5Pget(lid1,PROP1_NAME,&prop1_value);
@@ -1076,9 +1069,8 @@ test_genprop_list_callback(void)
     /* Check values of temporary properties (set with regular values) */
     ret = H5Pget(lid1,PROP3_NAME,&prop3_value);
     CHECK_I(ret, "H5Pget");
-    if(memcmp(&prop3_value,PROP3_DEF_VALUE,PROP3_SIZE)!=0) {
+    if(HDmemcmp(&prop3_value,PROP3_DEF_VALUE,PROP3_SIZE)!=0)
         TestErrPrintf("Property #3 doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
     ret = H5Pget(lid1,PROP4_NAME,&prop4_value);
     CHECK_I(ret, "H5Pget");
     VERIFY(prop4_value, *PROP4_DEF_VALUE, "H5Pget");
@@ -1086,12 +1078,10 @@ test_genprop_list_callback(void)
     /* Verify get callback information for properties tracked */
     VERIFY(prop1_cb_info.get_count, 1, "H5Pget");
     VERIFY(prop1_cb_info.get_plist_id, lid1, "H5Pget");
-    if(HDstrcmp(prop1_cb_info.get_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.get_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.get_value,PROP1_DEF_VALUE,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.get_value,PROP1_DEF_VALUE,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Set value of property #1 to different value */
     ret = H5Pset(lid1,PROP1_NAME,&prop1_new_value);
@@ -1100,12 +1090,10 @@ test_genprop_list_callback(void)
     /* Verify set callback information for properties tracked */
     VERIFY(prop1_cb_info.set_count, 1, "H5Pset");
     VERIFY(prop1_cb_info.set_plist_id, lid1, "H5Pset");
-    if(HDstrcmp(prop1_cb_info.set_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.set_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.set_value,&prop1_new_value,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.set_value,&prop1_new_value,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Check new value of tracked properties */
     ret = H5Pget(lid1,PROP1_NAME,&prop1_value);
@@ -1115,12 +1103,10 @@ test_genprop_list_callback(void)
     /* Verify get callback information again for properties tracked */
     VERIFY(prop1_cb_info.get_count, 2, "H5Pget");
     VERIFY(prop1_cb_info.get_plist_id, lid1, "H5Pget");
-    if(HDstrcmp(prop1_cb_info.get_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.get_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.get_value,&prop1_new_value,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.get_value,&prop1_new_value,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Delete property #2 */
     ret = H5Premove(lid1,PROP2_NAME);
@@ -1129,12 +1115,10 @@ test_genprop_list_callback(void)
     /* Verify delete callback information for properties tracked */
     VERIFY(prop2_cb_info.del_count, 1, "H5Premove");
     VERIFY(prop2_cb_info.del_plist_id, lid1, "H5Premove");
-    if(HDstrcmp(prop2_cb_info.del_name,PROP2_NAME)!=0) {
+    if(HDstrcmp(prop2_cb_info.del_name,PROP2_NAME)!=0)
         TestErrPrintf("Property #2 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop2_cb_info.del_value,PROP2_DEF_VALUE,PROP2_SIZE)!=0) {
+    if(HDmemcmp(prop2_cb_info.del_value,PROP2_DEF_VALUE,PROP2_SIZE)!=0)
         TestErrPrintf("Property #2 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Copy first list */
     lid2 = H5Pcopy(lid1);
@@ -1142,12 +1126,10 @@ test_genprop_list_callback(void)
 
     /* Verify copy callback information for properties tracked */
     VERIFY(prop1_cb_info.cop_count, 1, "H5Pcopy");
-    if(HDstrcmp(prop1_cb_info.cop_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.cop_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.cop_value,&prop1_new_value,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.cop_value,&prop1_new_value,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Verify that the class creation callback occurred */
     VERIFY(cop_cb_struct.count, 1, "H5Pcopy");
@@ -1159,12 +1141,10 @@ test_genprop_list_callback(void)
 
     /* Verify close callback information for properties tracked */
     VERIFY(prop1_cb_info.cls_count, 1, "H5Pclose");
-    if(HDstrcmp(prop1_cb_info.cls_name,PROP1_NAME)!=0) {
+    if(HDstrcmp(prop1_cb_info.cls_name,PROP1_NAME)!=0)
         TestErrPrintf("Property #1 name doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
-    if(HDmemcmp(prop1_cb_info.cls_value,&prop1_new_value,PROP1_SIZE)!=0) {
+    if(HDmemcmp(prop1_cb_info.cls_value,&prop1_new_value,PROP1_SIZE)!=0)
         TestErrPrintf("Property #1 value doesn't match!, line=%d\n",__LINE__);
-    } /* end if */
 
     /* Close second list */
     ret = H5Pclose(lid2);
@@ -1432,10 +1412,9 @@ test_genprop_path(void)
     /* Get full path for first class */
     path=H5P_get_class_path_test(cid1);
     CHECK_PTR(path, "H5P_get_class_path_test");
-    if(HDstrcmp(path,CLASS1_PATH)!=0) {
+    if(HDstrcmp(path,CLASS1_PATH)!=0)
         TestErrPrintf("Class names don't match!, path=%s, CLASS1_PATH=%s\n",path,CLASS1_PATH);
-    } /* end if */
-    free(path);
+    HDfree(path);
 
     /* Create another new generic class, derived from first class */
     cid2 = H5Pcreate_class(cid1,CLASS2_NAME,NULL,NULL,NULL,NULL,NULL,NULL);
@@ -1448,9 +1427,8 @@ test_genprop_path(void)
     /* Get full path for second class */
     path=H5P_get_class_path_test(cid2);
     CHECK_PTR(path, "H5P_get_class_path_test");
-    if(HDstrcmp(path,CLASS2_PATH)!=0) {
+    if(HDstrcmp(path,CLASS2_PATH)!=0)
         TestErrPrintf("Class names don't match!, path=%s, CLASS2_PATH=%s\n",path,CLASS2_PATH);
-    } /* end if */
 
     /* Open a copy of the class with the path name */
     cid3 = H5P_open_class_path_test(path);
@@ -1509,10 +1487,9 @@ test_genprop_refcount(void)
     /* Check class name */
     name = H5Pget_class_name(cid1);
     CHECK_PTR(name, "H5Pget_class_name");
-    if(HDstrcmp(name,CLASS1_NAME)!=0) {
+    if(HDstrcmp(name,CLASS1_NAME)!=0)
         TestErrPrintf("Class names don't match!, name=%s, CLASS1_NAME=%s\n",name,CLASS1_NAME);
-    } /* end if */
-    free(name);
+    HDfree(name);
 
     /* Close class */
     ret = H5Pclose_class(cid1);
@@ -1529,10 +1506,9 @@ test_genprop_refcount(void)
     /* Check class name */
     name = H5Pget_class_name(cid1);
     CHECK_PTR(name, "H5Pget_class_name");
-    if(HDstrcmp(name,CLASS1_NAME)!=0) {
+    if(HDstrcmp(name,CLASS1_NAME)!=0)
         TestErrPrintf("Class names don't match!, name=%s, CLASS1_NAME=%s\n",name,CLASS1_NAME);
-    } /* end if */
-    free(name);
+    HDfree(name);
 
     /* Close list */
     ret = H5Pclose(lid1);
@@ -1541,10 +1517,9 @@ test_genprop_refcount(void)
     /* Check class name */
     name = H5Pget_class_name(cid1);
     CHECK_PTR(name, "H5Pget_class_name");
-    if(HDstrcmp(name,CLASS1_NAME)!=0) {
+    if(HDstrcmp(name,CLASS1_NAME)!=0)
         TestErrPrintf("Class names don't match!, name=%s, CLASS1_NAME=%s\n",name,CLASS1_NAME);
-    } /* end if */
-    free(name);
+    HDfree(name);
 
     /* Close class */
     ret = H5Pclose_class(cid1);
