@@ -178,7 +178,7 @@ static void test_iter_group(void)
     CHECK(ret, FAIL, "H5Fclose");
 
     /* Sort the dataset names */
-    qsort(dnames,NDATASETS,sizeof(char *),iter_strcmp);
+    HDqsort(dnames,NDATASETS,sizeof(char *),iter_strcmp);
 
 
     /* Iterate through the datasets in the root group in various ways */
@@ -197,11 +197,17 @@ static void test_iter_group(void)
         VERIFY(num_membs,NDATASETS+2,"H5Gget_num_objs");
   
         for(i=0; i< (int)num_membs; i++) {
-            ret = H5Gget_objname_by_idx(root_group, (hsize_t)i, dataset_name, NAMELEN);
+#ifdef H5_WANT_H5_V1_4_COMPAT
+            int obj_type;         /* Type of object in file */
+#else /*H5_WANT_H5_V1_4_COMPAT*/
+            H5G_obj_t obj_type;         /* Type of object in file */
+#endif /*H5_WANT_H5_V1_4_COMPAT*/
+
+            ret = (herr_t)H5Gget_objname_by_idx(root_group, (hsize_t)i, dataset_name, NAMELEN);
             CHECK(ret, FAIL, "H5Gget_objname_by_idx");
             
-            ret = (herr_t)H5Gget_objtype_by_idx(root_group, (hsize_t)i);
-            CHECK(ret, FAIL, "H5Gget_objtype_by_idx");
+            obj_type = H5Gget_objtype_by_idx(root_group, (hsize_t)i);
+            CHECK(obj_type, H5G_UNKNOWN, "H5Gget_objtype_by_idx");
         }
     
         H5E_BEGIN_TRY {
@@ -229,7 +235,7 @@ static void test_iter_group(void)
             H5G_obj_t obj_type;         /* Type of object in file */
 #endif /*H5_WANT_H5_V1_4_COMPAT*/
 
-            ret = H5Gget_objname_by_idx(file, (hsize_t)i, dataset_name, NAMELEN);
+            ret = (herr_t)H5Gget_objname_by_idx(file, (hsize_t)i, dataset_name, NAMELEN);
             CHECK(ret, FAIL, "H5Gget_objname_by_idx");
             
             obj_type = H5Gget_objtype_by_idx(file, (hsize_t)i);
@@ -267,9 +273,8 @@ static void test_iter_group(void)
     /* Test all objects in group, when callback always returns 0 */
     info.command=RET_ZERO;
     idx=0;
-    if((ret=H5Giterate(file,"/",&idx,giter_cb,&info))>0) {
+    if((ret=H5Giterate(file,"/",&idx,giter_cb,&info))>0)
         TestErrPrintf("Group iteration function didn't return zero correctly!\n");
-    }
 
     /* Test all objects in group, when callback always returns 1 */
     /* This also tests the "restarting" ability, because the index changes */
@@ -287,29 +292,24 @@ static void test_iter_group(void)
 
         /* Verify that the correct name is retrieved */
         if(idx<=NDATASETS) {
-            if(strcmp(info.name,dnames[idx-1])!=0) {
+            if(HDstrcmp(info.name,dnames[idx-1])!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for dataset #%d!\n",idx);
-            } /* end if */
         } /* end if */
         else if(idx==(NDATASETS+1)) {
-            if(strcmp(info.name,"dtype")!=0) {
+            if(HDstrcmp(info.name,"dtype")!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for group!\n");
-            } /* end if */
         } /* end if */
         else if(idx==(NDATASETS+2)) {
-            if(strcmp(info.name,"grp")!=0) {
+            if(HDstrcmp(info.name,"grp")!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for group!\n");
-            } /* end if */
         } /* end if */
-        else {
+        else
             TestErrPrintf("Group iteration function walked too far!\n");
-        } /* end else */
     }
     VERIFY(ret,-1,"H5Giterate");
 
-    if(i!=(NDATASETS+2)) {
+    if(i!=(NDATASETS+2))
         TestErrPrintf("Group iteration function didn't perform multiple iterations correctly!\n");
-    } /* end if */
 
     /* Test all objects in group, when callback changes return value */
     /* This also tests the "restarting" ability, because the index changes */
@@ -327,36 +327,31 @@ static void test_iter_group(void)
 
         /* Verify that the correct name is retrieved */
         if(idx<=NDATASETS) {
-            if(strcmp(info.name,dnames[idx-1])!=0) {
+            if(HDstrcmp(info.name,dnames[idx-1])!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for dataset #%d!\n",idx);
-            } /* end if */
         } /* end if */
         else if(idx==(NDATASETS+1)) {
-            if(strcmp(info.name,"dtype")!=0) {
+            if(HDstrcmp(info.name,"dtype")!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for group!\n");
-            } /* end if */
         } /* end if */
         else if(idx==(NDATASETS+2)) {
-            if(strcmp(info.name,"grp")!=0) {
+            if(HDstrcmp(info.name,"grp")!=0)
                 TestErrPrintf("Group iteration function didn't return one correctly for group!\n");
-            } /* end if */
         } /* end if */
-        else {
+        else
             TestErrPrintf("Group iteration function walked too far!\n");
-        } /* end else */
     }
     VERIFY(ret,-1,"H5Giterate");
 
-    if(i!=42 || idx!=52) {
+    if(i!=42 || idx!=52)
         TestErrPrintf("Group iteration function didn't perform multiple iterations correctly!\n");
-    } /* end if */
 
     ret=H5Fclose(file);
     CHECK(ret, FAIL, "H5Fclose");
 
     /* Free the dataset names */
     for(i=0; i< NDATASETS; i++)
-        free(dnames[i]);
+        HDfree(dnames[i]);
 
 } /* test_iter_group() */
 
@@ -477,9 +472,8 @@ static void test_iter_attr(void)
     /* Test all attributes on dataset, when callback always returns 0 */
     info.command=RET_ZERO;
     idx=0;
-    if((ret=H5Aiterate(dataset,&idx,aiter_cb,&info))>0) {
+    if((ret=H5Aiterate(dataset,&idx,aiter_cb,&info))>0)
         TestErrPrintf("Attribute iteration function didn't return zero correctly!\n");
-    }
 
     /* Test all attributes on dataset, when callback always returns 1 */
     /* This also tests the "restarting" ability, because the index changes */
@@ -496,14 +490,12 @@ static void test_iter_attr(void)
         VERIFY(idx,(unsigned)i,"H5Aiterate");
 
         /* Verify that the correct name is retrieved */
-        if(strcmp(info.name,anames[idx-1])!=0) {
+        if(HDstrcmp(info.name,anames[idx-1])!=0)
             TestErrPrintf("Attribute iteration function didn't return one correctly!\n");
-        } /* end if */
     }
     VERIFY(ret,-1,"H5Aiterate");
-    if(i!=50 || idx!=50) {
+    if(i!=50 || idx!=50)
         TestErrPrintf("Group iteration function didn't perform multiple iterations correctly!\n");
-    } /* end if */
 
 
     /* Test all attributes on dataset, when callback changes return value */
@@ -521,14 +513,12 @@ static void test_iter_attr(void)
         VERIFY(idx,(unsigned)i+10,"H5Aiterate");
 
         /* Verify that the correct name is retrieved */
-        if(strcmp(info.name,anames[idx-1])!=0) {
+        if(HDstrcmp(info.name,anames[idx-1])!=0)
             TestErrPrintf("Attribute iteration function didn't return changing correctly!\n");
-        } /* end if */
     }
     VERIFY(ret,-1,"H5Aiterate");
-    if(i!=40 || idx!=50) {
+    if(i!=40 || idx!=50)
         TestErrPrintf("Group iteration function didn't perform multiple iterations correctly!\n");
-    } /* end if */
 
     ret=H5Fclose(file);
     CHECK(ret, FAIL, "H5Fclose");
@@ -538,7 +528,7 @@ static void test_iter_attr(void)
 
     /* Free the attribute names */
     for(i=0; i< NATTR; i++)
-        free(anames[i]);
+        HDfree(anames[i]);
 
 } /* test_iter_attr() */
 
@@ -563,7 +553,7 @@ herr_t giter_cb2(hid_t loc_id, const char *name, void *opdata)
     herr_t		ret;		/* Generic return value		*/
     H5G_stat_t statbuf;
 
-    if(strcmp(name,test_info->name)) {
+    if(HDstrcmp(name,test_info->name)) {
         TestErrPrintf("name=%s, test_info=%s\n",name,test_info->name);
         return(-1);
     } /* end if */
@@ -774,14 +764,13 @@ static void test_grp_memb_funcs(void)
     CHECK(ret, FAIL, "H5Fclose");
 
     /* Sort the dataset names */
-    qsort(dnames,NDATASETS+2,sizeof(char *),iter_strcmp);
-
+    HDqsort(dnames,NDATASETS+2,sizeof(char *),iter_strcmp);
 
     /* Iterate through the datasets in the root group in various ways */
     file=H5Fopen(DATAFILE, H5F_ACC_RDONLY, H5P_DEFAULT);
     CHECK(file, FAIL, "H5Fopen");
 
-    /* These twp functions, H5Gget_num_objs and H5Gget_objname_by_idx, actually
+    /* These two functions, H5Gget_num_objs and H5Gget_objname_by_idx, actually
      * iterate through B-tree for group members in internal library design.
      */
     root_group = H5Gopen(file, "/");
@@ -792,11 +781,17 @@ static void test_grp_memb_funcs(void)
     VERIFY(num_membs,NDATASETS+2,"H5Gget_num_objs");
   
     for(i=0; i< (int)num_membs; i++) {
+#ifdef H5_WANT_H5_V1_4_COMPAT
+        int obj_type;         /* Type of object in file */
+#else /*H5_WANT_H5_V1_4_COMPAT*/
+        H5G_obj_t obj_type;         /* Type of object in file */
+#endif /*H5_WANT_H5_V1_4_COMPAT*/
+
         /* Test with NULL for name, to query length */
         name_len = H5Gget_objname_by_idx(root_group, (hsize_t)i, NULL, NAMELEN);
         CHECK(name_len, FAIL, "H5Gget_objname_by_idx");
         
-        ret = H5Gget_objname_by_idx(root_group, (hsize_t)i, dataset_name, NAMELEN);
+        ret = (herr_t)H5Gget_objname_by_idx(root_group, (hsize_t)i, dataset_name, (size_t)(name_len+1));
         CHECK(ret, FAIL, "H5Gget_objname_by_idx");
 
         /* Double-check that the length is the same */
@@ -804,17 +799,17 @@ static void test_grp_memb_funcs(void)
         
         /* Keep a copy of the dataset names around for later */
         obj_names[i]=HDstrdup(dataset_name);
-        CHECK(obj_names[i], NULL, "strdup");           
+        CHECK(obj_names[i], NULL, "strdup");
         
-        ret = (herr_t)H5Gget_objtype_by_idx(root_group, (hsize_t)i);
-        CHECK(ret, FAIL, "H5Gget_objtype_by_idx");
+        obj_type = H5Gget_objtype_by_idx(root_group, (hsize_t)i);
+        CHECK(obj_type, H5G_UNKNOWN, "H5Gget_objtype_by_idx");
 
         if(!HDstrcmp(dataset_name, "grp"))
-            VERIFY(ret, H5G_GROUP, "H5Gget_objname_by_idx");
+            VERIFY(obj_type, H5G_GROUP, "H5Gget_objname_by_idx");
         if(!HDstrcmp(dataset_name, "dtype"))
-            VERIFY(ret, H5G_TYPE, "H5Gget_objname_by_idx");
+            VERIFY(obj_type, H5G_TYPE, "H5Gget_objname_by_idx");
         if(!HDstrncmp(dataset_name, "Dataset", 7))
-            VERIFY(ret, H5G_DATASET, "H5Gget_objname_by_idx");
+            VERIFY(obj_type, H5G_DATASET, "H5Gget_objname_by_idx");
     }
     
     H5E_BEGIN_TRY {
@@ -846,6 +841,80 @@ static void test_grp_memb_funcs(void)
 
 } /* test_grp_memb_funcs() */
 
+/****************************************************************
+**
+**  test_links(): Test soft and hard link iteration
+** 
+****************************************************************/
+static void test_links(void)
+{
+    hid_t file;             /* File ID */
+    char obj_name[NAMELEN]; /* Names of the object in group */
+    ssize_t name_len;       /* Length of object's name */
+    herr_t ret;		    /* Generic return value */
+    hid_t    gid, gid1;
+    hsize_t i;
+#ifdef H5_WANT_H5_V1_4_COMPAT
+    int       obj_type;     /* Type of object */
+#else /*H5_WANT_H5_V1_4_COMPAT*/
+    H5G_obj_t obj_type;     /* Type of object */
+#endif /*H5_WANT_H5_V1_4_COMPAT*/
+    hsize_t   nobjs;        /* Number of objects */
+
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing Soft and Hard Link Iteration Functionality\n"));
+
+    /* Create the test file with the datasets */
+    file = H5Fcreate(DATAFILE, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(file, FAIL, "H5Fcreate");
+
+    /* create groups */
+    gid = H5Gcreate (file, "/g1", 0);
+    CHECK(gid, FAIL, "H5Gcreate");
+    
+    gid1 = H5Gcreate (file, "/g1/g1.1", 0);
+    CHECK(gid1, FAIL, "H5Gcreate");
+
+    /* create soft and hard links to the group "/g1". */
+    ret = H5Glink (gid, H5G_LINK_SOFT, "something", "softlink");
+    CHECK(ret, FAIL, "H5Glink");
+
+    ret = H5Glink (gid, H5G_LINK_HARD, "/g1", "hardlink");
+    CHECK(ret, FAIL, "H5Glink");
+
+    ret = H5Gget_num_objs(gid, &nobjs);
+    CHECK(ret, FAIL, "H5Gget_num_objs");
+    VERIFY(nobjs,3,"H5Gget_num_objs");
+
+    /* Test these two functions, H5Gget_num_objs and H5Gget_objname_by_idx */
+    for(i=0; i<nobjs; i++) {
+        /* Get object name */
+        name_len = H5Gget_objname_by_idx(gid, i, obj_name, NAMELEN);
+        CHECK(name_len, FAIL, "H5Gget_objname_by_idx");
+        
+        obj_type = H5Gget_objtype_by_idx(gid, i);
+        CHECK(obj_type, H5G_UNKNOWN, "H5Gget_objtype_by_idx");
+
+        if(!HDstrcmp(obj_name, "g1.1"))
+            VERIFY(obj_type, H5G_GROUP, "H5Gget_objname_by_idx");
+        else if(!HDstrcmp(obj_name, "hardlink"))
+            VERIFY(obj_type, H5G_GROUP, "H5Gget_objname_by_idx");
+        else if(!HDstrcmp(obj_name, "softlink"))
+            VERIFY(obj_type, H5G_LINK, "H5Gget_objname_by_idx");
+        else
+            CHECK(0, 0, "unknown object name");
+    }
+
+    ret=H5Gclose(gid);
+    CHECK(ret, FAIL, "H5Gclose");
+
+    ret=H5Gclose(gid1);
+    CHECK(ret, FAIL, "H5Gclose");
+
+    ret=H5Fclose(file);
+    CHECK(ret, FAIL, "H5Fclose");
+} /* test_links() */
+
 /****************************************************************
 **
 **  test_iterate(): Main iteration testing routine.
@@ -862,6 +931,7 @@ test_iterate(void)
     test_iter_group_large(); /* Test group iteration for large # of objects */
     test_iter_attr();        /* Test attribute iteration */
     test_grp_memb_funcs();   /* Test group member information functions */
+    test_links();            /* Test soft and hard link iteration */
 }   /* test_iterate() */
 
 
