@@ -12,8 +12,6 @@
  * access to either file, you may request a copy from hdfhelp@ncsa.uiuc.edu. *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-/* $Id: tvltypes.c,v 1.25.2.4 2004/01/10 02:41:34 koziol Exp $ */
-
 /***********************************************************
 *
 * Test program:	 tvltypes
@@ -177,24 +175,25 @@ test_vltypes_vlen_atomic(void)
     hvl_t wdata2[SPACE1_DIM1];  /* Information to write */
     hvl_t rdata[SPACE1_DIM1];   /* Information read in */
     hvl_t fill;                 /* Fill value */
-    hid_t		fid1;		/* HDF5 File IDs		*/
-    hid_t		dataset;	/* Dataset ID			*/
-    hid_t		sid1;       /* Dataspace ID			*/
-    hid_t		tid1;       /* Datatype ID			*/
-    hid_t       dcpl_pid;   /* Dataset creation property list ID */
-    hid_t       xfer_pid;   /* Dataset transfer property list ID */
-    hsize_t		dims1[] = {SPACE1_DIM1};
-    hsize_t     size;       /* Number of bytes which will be used */
-    unsigned       i,j;        /* counting variables */
-    size_t         mem_used=0; /* Memory used during allocation */
-    herr_t		ret;		/* Generic return value		*/
+    hid_t fid1;		/* HDF5 File IDs		*/
+    hid_t dataset;	/* Dataset ID			*/
+    hid_t sid1;         /* Dataspace ID			*/
+    hid_t sid2;         /* ID of bad dataspace (no extent set) */
+    hid_t tid1;         /* Datatype ID			*/
+    hid_t dcpl_pid;     /* Dataset creation property list ID */
+    hid_t xfer_pid;     /* Dataset transfer property list ID */
+    hsize_t dims1[] = {SPACE1_DIM1};
+    hsize_t size;       /* Number of bytes which will be used */
+    unsigned i,j;       /* counting variables */
+    size_t mem_used=0;  /* Memory used during allocation */
+    herr_t ret;		/* Generic return value		*/
 
     /* Output message about test being performed */
     MESSAGE(5, ("Testing Basic Atomic VL Datatype Functionality\n"));
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+1)*sizeof(unsigned int));
+        wdata[i].p=HDmalloc((i+1)*sizeof(unsigned int));
         wdata[i].len=i+1;
         for(j=0; j<(i+1); j++)
             ((unsigned int *)wdata[i].p)[j]=i*10+j;
@@ -224,11 +223,9 @@ test_vltypes_vlen_atomic(void)
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(rdata[i].len!=0 || rdata[i].p!=NULL) {
+    for(i=0; i<SPACE1_DIM1; i++)
+        if(rdata[i].len!=0 || rdata[i].p!=NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d].len=%u, rdata[%d].p=%p\n",(int)i,(unsigned)rdata[i].len,(int)i,rdata[i].p);
-        } /* end if */
-    } /* end for */
     
     /* Write "nil" data to disk */
     ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata2);
@@ -239,11 +236,9 @@ test_vltypes_vlen_atomic(void)
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(rdata[i].len!=0 || rdata[i].p!=NULL) {
+    for(i=0; i<SPACE1_DIM1; i++)
+        if(rdata[i].len!=0 || rdata[i].p!=NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d].len=%u, rdata[%d].p=%p\n",(int)i,(unsigned)rdata[i].len,(int)i,rdata[i].p);
-        } /* end if */
-    } /* end for */
     
     /* Write dataset to disk */
     ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
@@ -275,11 +270,9 @@ test_vltypes_vlen_atomic(void)
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(rdata[i].len!=0 || rdata[i].p!=NULL) {
+    for(i=0; i<SPACE1_DIM1; i++)
+        if(rdata[i].len!=0 || rdata[i].p!=NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d].len=%u, rdata[%d].p=%p\n",(int)i,(unsigned)rdata[i].len,(int)i,rdata[i].p);
-        } /* end if */
-    } /* end for */
     
     /* Write "nil" data to disk */
     ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata2);
@@ -290,11 +283,9 @@ test_vltypes_vlen_atomic(void)
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(rdata[i].len!=0 || rdata[i].p!=NULL) {
+    for(i=0; i<SPACE1_DIM1; i++)
+        if(rdata[i].len!=0 || rdata[i].p!=NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d].len=%u, rdata[%d].p=%p\n",(int)i,(unsigned)rdata[i].len,(int)i,rdata[i].p);
-        } /* end if */
-    } /* end for */
     
     /* Write data to disk */
     ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
@@ -405,6 +396,10 @@ test_vltypes_vlen_atomic(void)
     tid1 = H5Dget_type(dataset);
     CHECK(tid1, FAIL, "H5Dget_type");
 
+    /* Create a "bad" dataspace with no extent set */
+    sid2 = H5Screate(H5S_SIMPLE);
+    CHECK(sid2, FAIL, "H5Screate");
+
     /* Change to the custom memory allocation routines for reading VL data */
     xfer_pid=H5Pcreate(H5P_DATASET_XFER);
     CHECK(xfer_pid, FAIL, "H5Pcreate");
@@ -416,6 +411,12 @@ test_vltypes_vlen_atomic(void)
     ret=H5Dvlen_get_buf_size(dataset,tid1,sid1,&size);
     CHECK(ret, FAIL, "H5Dvlen_get_buf_size");
 
+    /* Try to call H5Dvlen_get_buf with bad dataspace */
+    H5E_BEGIN_TRY {
+    ret=H5Dvlen_get_buf_size(dataset,tid1,sid2,&size);
+    } H5E_END_TRY
+    VERIFY(ret, FAIL, "H5Dvlen_get_buf_size");
+
     /* 10 elements allocated = 1 + 2 + 3 + 4 elements for each array position */
     VERIFY(size,((SPACE1_DIM1*(SPACE1_DIM1+1))/2)*sizeof(unsigned int),"H5Dvlen_get_buf_size");
 
@@ -441,6 +442,13 @@ test_vltypes_vlen_atomic(void)
         } /* end for */
     } /* end for */
 
+    /* Try to reclaim read data using "bad" dataspace with no extent
+     * Should fail */
+    H5E_BEGIN_TRY {
+    ret=H5Dvlen_reclaim(tid1,sid2,xfer_pid,rdata);
+    } H5E_END_TRY
+    VERIFY(ret, FAIL, "H5Dvlen_reclaim");
+
     /* Reclaim the read VL data */
     ret=H5Dvlen_reclaim(tid1,sid1,xfer_pid,rdata);
     CHECK(ret, FAIL, "H5Dvlen_reclaim");
@@ -501,7 +509,7 @@ rewrite_vltypes_vlen_atomic(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+increment)*sizeof(unsigned int));
+        wdata[i].p=HDmalloc((i+increment)*sizeof(unsigned int));
         wdata[i].len=i+increment;
         for(j=0; j<(i+increment); j++)
             ((unsigned int *)wdata[i].p)[j]=i*20+j;
@@ -662,7 +670,7 @@ test_vltypes_vlen_compound(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+1)*sizeof(s1));
+        wdata[i].p=HDmalloc((i+1)*sizeof(s1));
         wdata[i].len=i+1;
         for(j=0; j<(i+1); j++) {
             ((s1 *)wdata[i].p)[j].i=i*10+j;
@@ -808,7 +816,7 @@ rewrite_vltypes_vlen_compound(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+increment)*sizeof(s1));
+        wdata[i].p=HDmalloc((i+increment)*sizeof(s1));
         wdata[i].len=i+increment;
         for(j=0; j<(i+increment); j++) {
             ((s1 *)wdata[i].p)[j].i=i*40+j;
@@ -942,12 +950,8 @@ test_vltypes_compound_vlen_vlen(void)
     hid_t	dataset;	        /* Dataset ID			*/
     hid_t	sid1;                   /* Dataspace ID			*/
     hid_t	tid1, tid2, tid3;       /* Datatype IDs         */
-    hid_t       xfer_pid;               /* Dataset transfer property list ID */
-    hid_t       dcpl_pid;               /* Dataset creation property list ID */
     hsize_t	dims1[] = {SPACE3_DIM1};
-    hsize_t     size;                   /* Number of bytes which will be used */
     unsigned    i,j,k;                  /* counting variables */
-    size_t      mem_used=0;             /* Memory used during allocation */
     hvl_t       *t1, *t2;               /* Temporary pointer to VL information */
     herr_t	ret;		        /* Generic return value		*/
 
@@ -958,10 +962,10 @@ test_vltypes_compound_vlen_vlen(void)
     for(i=0; i<SPACE3_DIM1; i++) {
         wdata[i].i=i*10;
         wdata[i].f=(float)((i*20)/3.0);
-        wdata[i].v.p=malloc((i+L1_INCM)*sizeof(hvl_t));
+        wdata[i].v.p=HDmalloc((i+L1_INCM)*sizeof(hvl_t));
         wdata[i].v.len=i+L1_INCM;
         for(t1=(wdata[i].v).p,j=0; j<(i+L1_INCM); j++, t1++) {
-            t1->p=malloc((j+L2_INCM)*sizeof(unsigned int));
+            t1->p=HDmalloc((j+L2_INCM)*sizeof(unsigned int));
             t1->len=j+L2_INCM;
             for(k=0; k<j+L2_INCM; k++)
                 ((unsigned int*)t1->p)[k] = i*100 + j*10 + k;
@@ -1124,7 +1128,7 @@ test_vltypes_compound_vlen_atomic(void)
     for(i=0; i<SPACE1_DIM1; i++) {
         wdata[i].i=i*10;
         wdata[i].f=(float)((i*20)/3.0);
-        wdata[i].v.p=malloc((i+1)*sizeof(unsigned int));
+        wdata[i].v.p=HDmalloc((i+1)*sizeof(unsigned int));
         wdata[i].v.len=i+1;
         for(j=0; j<(i+1); j++)
             ((unsigned int *)wdata[i].v.p)[j]=i*10+j;
@@ -1238,11 +1242,9 @@ test_vltypes_compound_vlen_atomic(void)
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(rdata[i].i!=0 || rdata[i].f!=0.0 || rdata[i].v.len!=0 || rdata[i].v.p!=NULL) {
+    for(i=0; i<SPACE1_DIM1; i++)
+        if(rdata[i].i!=0 || rdata[i].f!=0.0 || rdata[i].v.len!=0 || rdata[i].v.p!=NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d].i=%d, rdata[%d].f=%f, rdata[%d].v.len=%u, rdata[%d].v.p=%p\n",(int)i,rdata[i].i,(int)i,rdata[i].f,(int)i,(unsigned)rdata[i].v.len,(int)i,rdata[i].v.p);
-        } /* end if */
-    } /* end for */
     
     /* Write dataset to disk */
     ret=H5Dwrite(dataset,tid2,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
@@ -1344,7 +1346,7 @@ rewrite_vltypes_compound_vlen_atomic(void)
     for(i=0; i<SPACE1_DIM1; i++) {
         wdata[i].i=i*40;
         wdata[i].f=(float)((i*50)/3.0);
-        wdata[i].v.p=malloc((i+increment)*sizeof(unsigned int));
+        wdata[i].v.p=HDmalloc((i+increment)*sizeof(unsigned int));
         wdata[i].v.len=i+increment;
         for(j=0; j<(i+increment); j++)
             ((unsigned int *)wdata[i].v.p)[j]=i*60+j;
@@ -1511,14 +1513,14 @@ test_vltypes_vlen_vlen_atomic(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+1)*sizeof(hvl_t));
+        wdata[i].p=HDmalloc((i+1)*sizeof(hvl_t));
         if(wdata[i].p==NULL) {
             TestErrPrintf("Cannot allocate memory for VL data! i=%u\n",i);
             return;
         } /* end if */
         wdata[i].len=i+1;
         for(t1=wdata[i].p,j=0; j<(i+1); j++, t1++) {
-            t1->p=malloc((j+1)*sizeof(unsigned int));
+            t1->p=HDmalloc((j+1)*sizeof(unsigned int));
             if(t1->p==NULL) {
                 TestErrPrintf("Cannot allocate memory for VL data! i=%u, j=%u\n",i,j);
                 return;
@@ -1702,14 +1704,14 @@ rewrite_longer_vltypes_vlen_vlen_atomic(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+increment)*sizeof(hvl_t));
+        wdata[i].p=HDmalloc((i+increment)*sizeof(hvl_t));
         if(wdata[i].p==NULL) {
             TestErrPrintf("Cannot allocate memory for VL data! i=%u\n",i);
             return;
         } /* end if */
         wdata[i].len=i+increment;
         for(t1=wdata[i].p,j=0; j<(i+increment); j++, t1++) {
-            t1->p=malloc((j+1)*sizeof(unsigned int));
+            t1->p=HDmalloc((j+1)*sizeof(unsigned int));
             if(t1->p==NULL) {
                 TestErrPrintf("Cannot allocate memory for VL data! i=%u, j=%u\n",i,j);
                 return;
@@ -1878,14 +1880,14 @@ rewrite_shorter_vltypes_vlen_vlen_atomic(void)
 
     /* Allocate and initialize VL data to write */
     for(i=0; i<SPACE1_DIM1; i++) {
-        wdata[i].p=malloc((i+increment)*sizeof(hvl_t));
+        wdata[i].p=HDmalloc((i+increment)*sizeof(hvl_t));
         if(wdata[i].p==NULL) {
             TestErrPrintf("Cannot allocate memory for VL data! i=%u\n",i);
             return;
         } /* end if */
         wdata[i].len=i+increment;
         for(t1=wdata[i].p,j=0; j<(i+increment); j++, t1++) {
-            t1->p=malloc((j+1)*sizeof(unsigned int));
+            t1->p=HDmalloc((j+1)*sizeof(unsigned int));
             if(t1->p==NULL) {
                 TestErrPrintf("Cannot allocate memory for VL data! i=%u, j=%u\n",i,j);
                 return;
