@@ -29,6 +29,10 @@ const char *FILENAME[] = {
     "lunlink",
     "filespace",
     "slashes",
+    "resurrect_set",
+    "resurrect_type",
+    "resurrect_group",
+    "unlink_chunked",
     NULL
 };
 
@@ -42,6 +46,7 @@ const char *FILENAME[] = {
 #define DATASET2NAME    "dataset2"
 #define ATTRNAME        "attribute"
 #define TYPENAME        "datatype"
+#define TYPE2NAME       "datatype2"
 #define FILESPACE_NDIMS 3
 #define FILESPACE_DIM0  20
 #define FILESPACE_DIM1  20
@@ -752,7 +757,7 @@ test_filespace(void)
 
         /* Set buffer to different random numbers each time */
         for (v=0, tmp_data=data; v<(FILESPACE_DIM0*FILESPACE_DIM1*FILESPACE_DIM2); v++)
-            *tmp_data++ = HDrandom();
+            *tmp_data++ = (int)HDrandom();
 
         /* Write the buffer to the dataset */
         if (H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data)<0) TEST_ERROR;
@@ -1303,7 +1308,7 @@ static int test_create_unlink(const char *msg, hid_t fapl)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_create_unlink() */
 
 
@@ -1373,7 +1378,7 @@ test_link_slashes(void)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_link_slashes() */
 
 
@@ -1443,7 +1448,7 @@ test_unlink_slashes(void)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_unlink_slashes() */
 
 /*
@@ -1532,7 +1537,7 @@ test_unlink_rightleaf(hid_t fid)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_unlink_rightleaf() */
 
 
@@ -1593,7 +1598,7 @@ test_unlink_rightnode(hid_t fid)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_unlink_rightnode() */
 
 
@@ -1797,9 +1802,313 @@ test_unlink_middlenode(hid_t fid)
     return 0;
 
 error:
-    return -1;
+    return 1;
 } /* end test_unlink_middlenode() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    test_resurrect_dataset
+ *
+ * Purpose:     Tests deleting a dataset while its still open and then
+ *              "resurrecting" it by creating a link to it again.
+ *
+ * Return:      Success:        0
+ *              Failure:        number of errors
+ *
+ * Programmer:  Quincey Koziol
+ *              Wednesday, July 14, 2004
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+test_resurrect_dataset(void)
+{
+    hid_t       f=-1, s=-1, d=-1, fapl=-1;
+    char	filename[1024];
+
+    TESTING("Resurrecting dataset after deletion");
+
+    /* Create file */
+    fapl = h5_fileaccess();
+    h5_fixname(FILENAME[6], fapl, filename, sizeof filename);
+
+    /* Create the file */
+    if((f = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) TEST_ERROR;
+
+    /* Create a dataset in the file */
+    if((s = H5Screate(H5S_SCALAR))<0) TEST_ERROR;
+    if((d = H5Dcreate(f, DATASETNAME, H5T_NATIVE_INT, s, H5P_DEFAULT))<0) TEST_ERROR;
+    if(H5Sclose(s)<0) TEST_ERROR;
+
+    /* Unlink the dataset while it's open (will mark it for deletion when closed) */
+    if(H5Gunlink(f, DATASETNAME)<0) TEST_ERROR;
+
+    /* Check that dataset name is NULL */
+    if(H5Iget_name(d, NULL, 0) != 0) TEST_ERROR;
+
+    /* Re-link the dataset to the group hierarchy (shouldn't get deleted now) */
+    if(H5Glink2(d, ".", H5G_LINK_HARD, f, DATASET2NAME)<0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Dclose(d)<0) TEST_ERROR;
+    if(H5Fclose(f)<0) TEST_ERROR;
+
+    /* Re-open the file */
+    if((f=H5Fopen(filename, H5F_ACC_RDONLY, fapl))<0) TEST_ERROR;
+
+    /* Attempt to open the dataset under the new name */
+    if((d=H5Dopen(f,DATASET2NAME))<0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Dclose(d)<0) TEST_ERROR;
+    if(H5Fclose(f)<0) TEST_ERROR;
+    if(H5Pclose(fapl)<0) TEST_ERROR;
+
+    PASSED();
+    return 0;
+
+error:
+    H5E_BEGIN_TRY {
+	H5Sclose(s);
+	H5Dclose(d);
+	H5Fclose(f);
+	H5Pclose(fapl);
+    } H5E_END_TRY;
+    return 1;
+} /* end test_resurrect_dataset() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_resurrect_datatype
+ *
+ * Purpose:     Tests deleting a datatype while it's still open and then
+ *              "resurrecting" it by creating a link to it again.
+ *
+ * Return:      Success:        0
+ *              Failure:        number of errors
+ *
+ * Programmer:  James Laird
+ *              Wednesday, July 28, 2004
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+test_resurrect_datatype(void)
+{
+    hid_t       file=-1, type=-1, fapl=-1;
+    char        filename[1024];
+
+    TESTING("Resurrecting datatype after deletion");
+
+    /* Create file */
+    fapl = h5_fileaccess();
+    h5_fixname(FILENAME[7], fapl, filename, sizeof filename);
+
+    /* Create the file */
+    if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) TEST_ERROR;
+
+    /* Create a named datatype in the file */
+    if((type = H5Tcopy (H5T_NATIVE_INT))<0) TEST_ERROR;
+    if(H5Tcommit (file, TYPENAME, type)<0) TEST_ERROR;
+
+    /* Unlink the datatype while it's open (will mark it for deletion when closed) */
+    if(H5Gunlink(file, TYPENAME)<0) TEST_ERROR;
+
+    /* Check that datatype name is NULL */
+    if(H5Iget_name(type, NULL, 0) != 0) TEST_ERROR;
+
+    /* Re-link the datatype to the group hierarchy (shouldn't get deleted now) */
+    if(H5Glink2(type, ".", H5G_LINK_HARD, file, TYPE2NAME) < 0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Tclose(type)<0) TEST_ERROR;
+    if(H5Fclose(file)<0) TEST_ERROR;
+
+    /* Re-open the file */
+    if((file=H5Fopen(filename, H5F_ACC_RDONLY, fapl))<0) TEST_ERROR;
+
+    /* Attempt to open the datatype under the new name */
+    if((type=H5Topen(file,TYPE2NAME))<0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Tclose(type)<0) TEST_ERROR;
+    if(H5Fclose(file)<0) TEST_ERROR;
+
+    PASSED();
+    return 0;
+
+error:
+    H5E_BEGIN_TRY {
+	H5Tclose(type);
+	H5Fclose(file);
+    } H5E_END_TRY;
+    return 1;
+} /* end test_resurrect_datatype() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_resurrect_group
+ *
+ * Purpose:     Tests deleting a group while it's still open and then
+ *              "resurrecting" it by creating a link to it again.
+ *
+ * Return:      Success:        0
+ *              Failure:        number of errors
+ *
+ * Programmer:  James Laird
+ *              Wednesday, July 28, 2004
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+test_resurrect_group(void)
+{
+    hid_t       file=-1, group=-1, fapl=-1;
+    char        filename[1024];
+
+    TESTING("Resurrecting group after deletion");
+
+    /* Create file */
+    fapl = h5_fileaccess();
+    h5_fixname(FILENAME[8], fapl, filename, sizeof filename);
+
+    /* Create the file */
+    if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) TEST_ERROR;
+
+    /* Create a group in the file */
+    if((group = H5Gcreate (file, GROUPNAME, 0))<0) TEST_ERROR;
+
+    /* Unlink the group while it's open (will mark it for deletion when closed) */
+    if(H5Gunlink(file, GROUPNAME)<0) TEST_ERROR;
+
+    /* Check that group's name is NULL */
+    if(H5Iget_name(group, NULL, 0) != 0) TEST_ERROR;
+
+    /* Re-link the group into the group hierarchy (shouldn't get deleted now) */
+    if(H5Glink2(group, ".", H5G_LINK_HARD, file, GROUP2NAME)<0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Gclose(group)<0) TEST_ERROR;
+    if(H5Fclose(file)<0) TEST_ERROR;
+
+    /* Re-open the file */
+    if((file=H5Fopen(filename, H5F_ACC_RDONLY, fapl))<0) TEST_ERROR;
+
+    /* Attempt to open the datatype under the new name */
+    if((group=H5Gopen(file,GROUP2NAME))<0) TEST_ERROR;
+
+    /* Close things */
+    if(H5Gclose(group)<0) TEST_ERROR;
+    if(H5Fclose(file)<0) TEST_ERROR;
+
+    PASSED();
+    return 0;
+
+error:
+    H5E_BEGIN_TRY {
+	H5Gclose(group);
+	H5Fclose(file);
+    } H5E_END_TRY;
+    return 1;
+} /* end test_resurrect_group() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_unlink_chunked_dataset
+ *
+ * Purpose:     Tests deleting a chunked dataset 
+ *
+ * Return:      Success:        0
+ *              Failure:        number of errors
+ *
+ * Programmer:  Quincey Koziol
+ *              Monday, September 27, 2004
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+test_unlink_chunked_dataset(void)
+{
+    hid_t fapl_id=-1;
+    hid_t file_id=-1;
+    hid_t dset_id=-1;
+    hid_t space_id=-1;
+    hid_t dcpl_id=-1;
+    hsize_t dims[FILESPACE_NDIMS]={FILESPACE_DIM0,FILESPACE_DIM1,FILESPACE_DIM2};
+    hsize_t max_dims[FILESPACE_NDIMS]={H5S_UNLIMITED,H5S_UNLIMITED,H5S_UNLIMITED};
+    hsize_t chunk_dims[FILESPACE_NDIMS]={FILESPACE_CHUNK0,FILESPACE_CHUNK1,FILESPACE_CHUNK2};
+    char filename[1024];
+
+    TESTING("Unlinking chunked dataset");
+
+    /* Create file */
+    fapl_id = h5_fileaccess();
+    h5_fixname(FILENAME[7], fapl_id, filename, sizeof filename);
+
+    /* Create the file */
+    if((file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl_id))<0) TEST_ERROR;
+
+    /* Create the dataspace */
+    if((space_id=H5Screate_simple(FILESPACE_NDIMS,dims,max_dims))<0) TEST_ERROR;
+
+    /* Create the dataset creation filter */
+    if((dcpl_id=H5Pcreate(H5P_DATASET_CREATE))<0) TEST_ERROR;
+
+    /* Set to chunked storage */
+    if(H5Pset_chunk(dcpl_id,FILESPACE_NDIMS,chunk_dims)<0) TEST_ERROR;
+
+    /* Set to early space allocation */
+    if(H5Pset_alloc_time(dcpl_id,H5D_ALLOC_TIME_EARLY)<0) TEST_ERROR;
+
+    /* Create the dataset */
+    if((dset_id = H5Dcreate(file_id,DATASETNAME,H5T_NATIVE_INT,space_id,dcpl_id))<0) TEST_ERROR;
+
+    /* Close the dataspace */
+    if(H5Sclose(space_id)<0) TEST_ERROR;
+
+    /* Close the dataset creation property list */
+    if(H5Pclose(dcpl_id)<0) TEST_ERROR;
+
+    /* Close the dataset */
+    if(H5Dclose(dset_id)<0) TEST_ERROR;
+
+    /* Close the file */
+    if(H5Fclose(file_id)<0) TEST_ERROR;
+
+    /* Re-open the file */
+    if((file_id = H5Fopen(filename, H5F_ACC_RDWR, H5P_DEFAULT))<0) TEST_ERROR;
+
+    /* Delete the dataset */
+    if(H5Gunlink(file_id, DATASETNAME)<0) TEST_ERROR;
+
+    /* Close the file */
+    if(H5Fclose(file_id)<0) TEST_ERROR;
+
+    /* Close the file access property list */
+    if(H5Pclose(fapl_id)<0) TEST_ERROR;
+
+    PASSED();
+    return 0;
+
+error:
+    H5E_BEGIN_TRY {
+	H5Pclose(dcpl_id);
+	H5Sclose(space_id);
+	H5Dclose(dset_id);
+	H5Fclose(file_id);
+	H5Pclose(fapl_id);
+    } H5E_END_TRY;
+    return 1;
+} /* end test_unlink_chunked_dataset() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	main
@@ -1835,7 +2144,7 @@ main(void)
     double rdcc_w0;
 
     /* Set the random # seed */
-    HDsrandom((unsigned long)time(NULL));
+    HDsrandom((unsigned long)HDtime(NULL));
 
     /* Open */
     h5_reset();
@@ -1872,12 +2181,20 @@ main(void)
 
     nerrors += test_link_slashes();
     nerrors += test_unlink_slashes();
- 
+
     /* Test specific B-tree removal issues */
     nerrors += test_unlink_rightleaf(file);
     nerrors += test_unlink_rightnode(file);
     nerrors += test_unlink_middlenode(file);
+
+    /* Test "resurrecting" objects */
+    nerrors += test_resurrect_dataset();
+    nerrors += test_resurrect_datatype();
+    nerrors += test_resurrect_group();
  
+    /* Test unlinking chunked datasets */
+    nerrors += test_unlink_chunked_dataset();
+
     /* Close */
     if (H5Fclose(file)<0) TEST_ERROR;
     if (nerrors) {
@@ -1890,4 +2207,3 @@ main(void)
  error:
     return 1;
 }
-
