@@ -12,8 +12,6 @@
  * access to either file, you may request a copy from hdfhelp@ncsa.uiuc.edu. *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-/* $Id: t_dset.c,v 1.36.2.1 2004/01/23 00:10:34 acheng Exp $ */
-
 /*
  * Parallel tests for datasets
  */
@@ -45,7 +43,7 @@
  * ZCOL		same as BYCOL except process 0 gets 0 columns
  */
 static void
-slab_set(int mpi_rank, int mpi_size, hssize_t start[], hsize_t count[],
+slab_set(int mpi_rank, int mpi_size, hsize_t start[], hsize_t count[],
 	 hsize_t stride[], hsize_t block[], int mode)
 {
     switch (mode){
@@ -112,8 +110,8 @@ if (VERBOSE_MED) printf("slab_set wholeset\n");
 	break;
     }
 if (VERBOSE_MED){
-    printf("start[]=(%ld,%ld), count[]=(%lu,%lu), stride[]=(%lu,%lu), block[]=(%lu,%lu), total datapoints=%lu\n",
-	(long)start[0], (long)start[1], (unsigned long)count[0], (unsigned long)count[1],
+    printf("start[]=(%lu,%lu), count[]=(%lu,%lu), stride[]=(%lu,%lu), block[]=(%lu,%lu), total datapoints=%lu\n",
+	(unsigned long)start[0], (unsigned long)start[1], (unsigned long)count[0], (unsigned long)count[1],
 	(unsigned long)stride[0], (unsigned long)stride[1], (unsigned long)block[0], (unsigned long)block[1],
 	(unsigned long)(block[0]*block[1]*count[0]*count[1]));
     }
@@ -125,7 +123,7 @@ if (VERBOSE_MED){
  * Assume dimension rank is 2 and data is stored contiguous.
  */
 static void
-dataset_fill(hssize_t start[], hsize_t block[], DATATYPE * dataset)
+dataset_fill(hsize_t start[], hsize_t block[], DATATYPE * dataset)
 {
     DATATYPE *dataptr = dataset;
     hsize_t i, j;
@@ -144,7 +142,7 @@ dataset_fill(hssize_t start[], hsize_t block[], DATATYPE * dataset)
  * Print the content of the dataset.
  */
 static void
-dataset_print(hssize_t start[], hsize_t block[], DATATYPE * dataset)
+dataset_print(hsize_t start[], hsize_t block[], DATATYPE * dataset)
 {
     DATATYPE *dataptr = dataset;
     hsize_t i, j;
@@ -152,13 +150,13 @@ dataset_print(hssize_t start[], hsize_t block[], DATATYPE * dataset)
     /* print the column heading */
     printf("%-8s", "Cols:");
     for (j=0; j < block[1]; j++){
-	printf("%3ld ", (long)(start[1]+j));
+	printf("%3lu ", (unsigned long)(start[1]+j));
     }
     printf("\n");
 
     /* print the slab data */
     for (i=0; i < block[0]; i++){
-	printf("Row %2ld: ", (long)(i+start[0]));
+	printf("Row %2lu: ", (unsigned long)(i+start[0]));
 	for (j=0; j < block[1]; j++){
 	    printf("%03d ", *dataptr++);
 	}
@@ -170,7 +168,8 @@ dataset_print(hssize_t start[], hsize_t block[], DATATYPE * dataset)
 /*
  * Print the content of the dataset.
  */
-int dataset_vrfy(hssize_t start[], hsize_t count[], hsize_t stride[], hsize_t block[], DATATYPE *dataset, DATATYPE *original)
+int
+dataset_vrfy(hsize_t start[], hsize_t count[], hsize_t stride[], hsize_t block[], DATATYPE *dataset, DATATYPE *original)
 {
     hsize_t i, j;
     int vrfyerrs;
@@ -178,8 +177,8 @@ int dataset_vrfy(hssize_t start[], hsize_t count[], hsize_t stride[], hsize_t bl
     /* print it if VERBOSE_MED */
     if (VERBOSE_MED) {
 	printf("dataset_vrfy dumping:::\n");
-	printf("start(%ld, %ld), count(%lu, %lu), stride(%lu, %lu), block(%lu, %lu)\n",
-	    (long)start[0], (long)start[1], (unsigned long)count[0], (unsigned long)count[1],
+	printf("start(%lu, %lu), count(%lu, %lu), stride(%lu, %lu), block(%lu, %lu)\n",
+	    (unsigned long)start[0], (unsigned long)start[1], (unsigned long)count[0], (unsigned long)count[1],
 	    (unsigned long)stride[0], (unsigned long)stride[1], (unsigned long)block[0], (unsigned long)block[1]);
 	printf("original values:\n");
 	dataset_print(start, block, original);
@@ -192,9 +191,9 @@ int dataset_vrfy(hssize_t start[], hsize_t count[], hsize_t stride[], hsize_t bl
 	for (j=0; j < block[1]; j++){
 	    if (*dataset != *original){
 		if (vrfyerrs++ < MAX_ERR_REPORT || VERBOSE_MED){
-		    printf("Dataset Verify failed at [%ld][%ld](row %ld, col %ld): expect %d, got %d\n",
-			(long)i, (long)j,
-			(long)(i+start[0]), (long)(j+start[1]),
+		    printf("Dataset Verify failed at [%lu][%lu](row %lu, col %lu): expect %d, got %d\n",
+			(unsigned long)i, (unsigned long)j,
+			(unsigned long)(i+start[0]), (unsigned long)(j+start[1]),
 			*(original), *(dataset));
 		}
 		dataset++;
@@ -223,7 +222,7 @@ int dataset_vrfy(hssize_t start[], hsize_t count[], hsize_t stride[], hsize_t bl
  */
 
 void
-dataset_writeInd(char *filename)
+dataset_writeInd(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -234,8 +233,9 @@ dataset_writeInd(char *filename)
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
     hsize_t dims[RANK];   	/* dataset dim sizes */
     DATATYPE *data_array1 = NULL;	/* data buffer */
+    const char *filename;
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t   start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -245,6 +245,7 @@ dataset_writeInd(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Independent write test on file %s\n", filename);
 
@@ -368,7 +369,7 @@ MPI_Barrier(MPI_COMM_WORLD);
 
 /* Example of using the parallel HDF5 library to read a dataset */
 void
-dataset_readInd(char *filename)
+dataset_readInd(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -378,8 +379,9 @@ dataset_readInd(char *filename)
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
     DATATYPE *data_array1 = NULL;	/* data buffer */
     DATATYPE *data_origin1 = NULL; 	/* expected data buffer */
+    const char *filename;
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -389,6 +391,7 @@ dataset_readInd(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Independent read test on file %s\n", filename);
 
@@ -489,7 +492,7 @@ dataset_readInd(char *filename)
  */
 
 void
-dataset_writeAll(char *filename)
+dataset_writeAll(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -502,8 +505,9 @@ dataset_writeAll(char *filename)
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
     hsize_t dims[RANK];   	/* dataset dim sizes */
     DATATYPE *data_array1 = NULL;	/* data buffer */
+    const char *filename;
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -513,6 +517,7 @@ dataset_writeAll(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Collective write test on file %s\n", filename);
 
@@ -848,7 +853,7 @@ dataset_writeAll(char *filename)
  */
 
 void
-dataset_readAll(char *filename)
+dataset_readAll(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -859,8 +864,9 @@ dataset_readAll(char *filename)
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
     DATATYPE *data_array1 = NULL;	/* data buffer */
     DATATYPE *data_origin1 = NULL; 	/* expected data buffer */
+    const char *filename;
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -870,6 +876,7 @@ dataset_readAll(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Collective read test on file %s\n", filename);
 
@@ -1069,7 +1076,7 @@ dataset_readAll(char *filename)
  */
 
 void
-extend_writeInd(char *filename)
+extend_writeInd(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -1078,6 +1085,7 @@ extend_writeInd(char *filename)
     hid_t mem_dataspace;	/* memory dataspace ID */
     hid_t dataset1, dataset2;	/* Dataset ID */
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
+    const char *filename;
     hsize_t dims[RANK];   	/* dataset dim sizes */
     hsize_t max_dims[RANK] =
 		{H5S_UNLIMITED, H5S_UNLIMITED};	/* dataset maximum dim sizes */
@@ -1085,7 +1093,7 @@ extend_writeInd(char *filename)
     hsize_t	chunk_dims[RANK];		/* chunk sizes */
     hid_t	dataset_pl;			/* dataset create prop. list */
 
-    hssize_t	start[RANK];			/* for hyperslab setting */
+    hsize_t	start[RANK];			/* for hyperslab setting */
     hsize_t	count[RANK];			/* for hyperslab setting */
     hsize_t	stride[RANK];			/* for hyperslab setting */
     hsize_t 	block[RANK];			/* for hyperslab setting */
@@ -1096,6 +1104,7 @@ extend_writeInd(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Extend independent write test on file %s\n", filename);
 
@@ -1295,9 +1304,177 @@ extend_writeInd(char *filename)
     if (data_array1) free(data_array1);
 }
 
+/*
+ * Example of using the parallel HDF5 library to create an extendable dataset
+ * and perform I/O on it in a way that verifies that the chunk cache is
+ * bypassed for parallel I/O.
+ */
+
+void
+extend_writeInd2(void)
+{
+    const char *filename;
+    hid_t fid;                  /* HDF5 file ID */
+    hid_t fapl;			/* File access templates */
+    hid_t fs;   		/* File dataspace ID */
+    hid_t ms;   		/* Memory dataspace ID */
+    hid_t dataset;		/* Dataset ID */
+    hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
+    hsize_t orig_size=10;   	/* Original dataset dim size */
+    hsize_t new_size=20;   	/* Extended dataset dim size */
+    hsize_t one=1;
+    hsize_t max_size = H5S_UNLIMITED;	/* dataset maximum dim size */
+    hsize_t chunk_size = 16384;	/* chunk size */
+    hid_t dcpl;	       		/* dataset create prop. list */
+    int   written[10],          /* Data to write */
+        retrieved[10];          /* Data read in */
+    int mpi_size, mpi_rank;     /* MPI settings */
+    int i;                      /* Local index variable */
+    herr_t ret;         	/* Generic return value */
+
+    filename = GetTestParameters();
+    if (VERBOSE_MED)
+	printf("Extend independent write test #2 on file %s\n", filename);
+
+    /* set up MPI parameters */
+    MPI_Comm_size(MPI_COMM_WORLD,&mpi_size);
+    MPI_Comm_rank(MPI_COMM_WORLD,&mpi_rank);
+
+    /* -------------------
+     * START AN HDF5 FILE
+     * -------------------*/
+    /* setup file access template */
+    fapl = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
+    VRFY((fapl >= 0), "create_faccess_plist succeeded");
+
+    /* create the file collectively */
+    fid=H5Fcreate(filename,H5F_ACC_TRUNC,H5P_DEFAULT,fapl);
+    VRFY((fid >= 0), "H5Fcreate succeeded");
+
+    /* Release file-access template */
+    ret=H5Pclose(fapl);
+    VRFY((ret >= 0), "H5Pclose succeeded");
+
+
+    /* --------------------------------------------------------------
+     * Define the dimensions of the overall datasets and create them.
+     * ------------------------------------------------------------- */
+
+    /* set up dataset storage chunk sizes and creation property list */
+    dcpl = H5Pcreate(H5P_DATASET_CREATE);
+    VRFY((dcpl >= 0), "H5Pcreate succeeded");
+    ret = H5Pset_chunk(dcpl, 1, &chunk_size);
+    VRFY((ret >= 0), "H5Pset_chunk succeeded");
+
+    /* setup dimensionality object */
+    fs = H5Screate_simple (1, &orig_size, &max_size);
+    VRFY((fs >= 0), "H5Screate_simple succeeded");
+
+    /* create an extendible dataset collectively */
+    dataset = H5Dcreate(fid, DATASETNAME1, H5T_NATIVE_INT, fs, dcpl);
+    VRFY((dataset >= 0), "H5Dcreate succeeded");
+
+    /* release resource */
+    ret=H5Pclose(dcpl);
+    VRFY((ret >= 0), "H5Pclose succeeded");
+
+
+    /* -------------------------
+     * Test writing to dataset
+     * -------------------------*/
+    /* create a memory dataspace independently */
+    ms = H5Screate_simple(1, &orig_size, &max_size);
+    VRFY((ms >= 0), "H5Screate_simple succeeded");
+
+    /* put some trivial data in the data_array */
+    for (i=0; i<(int)orig_size; i++)
+        written[i] = i;
+    MESG("data array initialized");
+    if (VERBOSE_MED) {
+	MESG("writing at offset zero: ");
+        for (i=0; i<(int)orig_size; i++)
+            printf("%s%d", i?", ":"", written[i]);
+        printf("\n");
+    }
+    ret = H5Dwrite(dataset, H5T_NATIVE_INT, ms, fs, H5P_DEFAULT, written);
+    VRFY((ret >= 0), "H5Dwrite succeeded");
+
+    /* -------------------------
+     * Read initial data from dataset.
+     * -------------------------*/
+    ret = H5Dread(dataset, H5T_NATIVE_INT, ms, fs, H5P_DEFAULT, retrieved);
+    VRFY((ret >= 0), "H5Dread succeeded");
+    for (i=0; i<(int)orig_size; i++)
+        if(written[i]!=retrieved[i]) {
+            printf("Line #%d: written!=retrieved: written[%d]=%d, retrieved[%d]=%d\n",__LINE__,
+                i,written[i], i,retrieved[i]);
+            nerrors++;
+        }
+    if (VERBOSE_MED){
+	MESG("read at offset zero: ");
+        for (i=0; i<(int)orig_size; i++)
+            printf("%s%d", i?", ":"", retrieved[i]);
+        printf("\n");
+    }
+
+    /* -------------------------
+     * Extend the dataset & retrieve new dataspace
+     * -------------------------*/
+    ret =H5Dextend(dataset, &new_size);
+    VRFY((ret >= 0), "H5Dextend succeeded");
+    ret=H5Sclose(fs);
+    VRFY((ret >= 0), "H5Sclose succeeded");
+    fs = H5Dget_space(dataset);
+    VRFY((fs >= 0), "H5Dget_space succeeded");
+
+    /* -------------------------
+     * Write to the second half of the dataset
+     * -------------------------*/
+    for (i=0; i<(int)orig_size; i++)
+        written[i] = orig_size + i;
+    MESG("data array re-initialized");
+    if (VERBOSE_MED) {
+	MESG("writing at offset 10: ");
+        for (i=0; i<(int)orig_size; i++)
+            printf("%s%d", i?", ":"", written[i]);
+        printf("\n");
+    }
+    ret = H5Sselect_hyperslab(fs, H5S_SELECT_SET, &orig_size, NULL, &one, &orig_size);
+    VRFY((ret >= 0), "H5Sselect_hyperslab succeeded");
+    ret = H5Dwrite(dataset, H5T_NATIVE_INT, ms, fs, H5P_DEFAULT, written);
+    VRFY((ret >= 0), "H5Dwrite succeeded");
+
+    /* -------------------------
+     * Read the new data
+     * -------------------------*/
+    ret = H5Dread(dataset, H5T_NATIVE_INT, ms, fs, H5P_DEFAULT, retrieved);
+    VRFY((ret >= 0), "H5Dread succeeded");
+    for (i=0; i<(int)orig_size; i++)
+        if(written[i]!=retrieved[i]) {
+            printf("Line #%d: written!=retrieved: written[%d]=%d, retrieved[%d]=%d\n",__LINE__,
+                i,written[i], i,retrieved[i]);
+            nerrors++;
+        }
+    if (VERBOSE_MED){
+	MESG("read at offset 10: ");
+        for (i=0; i<(int)orig_size; i++)
+            printf("%s%d", i?", ":"", retrieved[i]);
+        printf("\n");
+    }
+
+
+    /* Close dataset collectively */
+    ret=H5Dclose(dataset);
+    VRFY((ret >= 0), "H5Dclose succeeded");
+
+    /* Close the file collectively */
+    ret = H5Fclose(fid);
+    VRFY((ret >= 0), "H5Fclose succeeded");
+}
+
 /* Example of using the parallel HDF5 library to read an extendible dataset */
 void
-extend_readInd(char *filename)
+extend_readInd(void)
 {
     hid_t fid;			/* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -1309,8 +1486,9 @@ extend_readInd(char *filename)
     DATATYPE *data_array1 = NULL;	/* data buffer */
     DATATYPE *data_array2 = NULL;	/* data buffer */
     DATATYPE *data_origin1 = NULL; 	/* expected data buffer */
+    const char *filename;
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -1320,6 +1498,7 @@ extend_readInd(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Extend independent read test on file %s\n", filename);
 
@@ -1474,7 +1653,7 @@ extend_readInd(char *filename)
  */
 
 void
-extend_writeAll(char *filename)
+extend_writeAll(void)
 {
     hid_t fid;                  /* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -1484,6 +1663,7 @@ extend_writeAll(char *filename)
     hid_t mem_dataspace;	/* memory dataspace ID */
     hid_t dataset1, dataset2;	/* Dataset ID */
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
+    const char *filename;
     hsize_t dims[RANK];   	/* dataset dim sizes */
     hsize_t max_dims[RANK] =
 		{H5S_UNLIMITED, H5S_UNLIMITED};	/* dataset maximum dim sizes */
@@ -1491,7 +1671,7 @@ extend_writeAll(char *filename)
     hsize_t	chunk_dims[RANK];		/* chunk sizes */
     hid_t	dataset_pl;			/* dataset create prop. list */
 
-    hssize_t	start[RANK];			/* for hyperslab setting */
+    hsize_t	start[RANK];			/* for hyperslab setting */
     hsize_t	count[RANK];			/* for hyperslab setting */
     hsize_t	stride[RANK];			/* for hyperslab setting */
     hsize_t 	block[RANK];			/* for hyperslab setting */
@@ -1502,6 +1682,7 @@ extend_writeAll(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Extend independent write test on file %s\n", filename);
 
@@ -1718,7 +1899,7 @@ extend_writeAll(char *filename)
 
 /* Example of using the parallel HDF5 library to read an extendible dataset */
 void
-extend_readAll(char *filename)
+extend_readAll(void)
 {
     hid_t fid;			/* HDF5 file ID */
     hid_t acc_tpl;		/* File access templates */
@@ -1727,12 +1908,13 @@ extend_readAll(char *filename)
     hid_t mem_dataspace;	/* memory dataspace ID */
     hid_t dataset1, dataset2;	/* Dataset ID */
     hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
+    const char *filename;
     hsize_t dims[RANK];   	/* dataset dim sizes */
     DATATYPE *data_array1 = NULL;	/* data buffer */
     DATATYPE *data_array2 = NULL;	/* data buffer */
     DATATYPE *data_origin1 = NULL; 	/* expected data buffer */
 
-    hssize_t   start[RANK];			/* for hyperslab setting */
+    hsize_t start[RANK];			/* for hyperslab setting */
     hsize_t count[RANK], stride[RANK];		/* for hyperslab setting */
     hsize_t block[RANK];			/* for hyperslab setting */
 
@@ -1742,6 +1924,7 @@ extend_readAll(char *filename)
     MPI_Comm comm = MPI_COMM_WORLD;
     MPI_Info info = MPI_INFO_NULL;
 
+    filename = GetTestParameters();
     if (VERBOSE_MED)
 	printf("Extend independent read test on file %s\n", filename);
 
@@ -1896,3 +2079,161 @@ extend_readAll(char *filename)
     if (data_array2) free(data_array2);
     if (data_origin1) free(data_origin1);
 }
+
+/*
+ * Example of using the parallel HDF5 library to read a compressed
+ * dataset in an HDF5 file with collective parallel access support.
+ */
+
+#ifdef H5_HAVE_FILTER_DEFLATE
+void
+compress_readAll(void)
+{
+    hid_t fid;                  /* HDF5 file ID */
+    hid_t acc_tpl;		/* File access templates */
+    hid_t dcpl;                 /* Dataset creation property list */
+    hid_t xfer_plist;		/* Dataset transfer properties list */
+    hid_t dataspace;		/* Dataspace ID */
+    hid_t dataset;		/* Dataset ID */
+    int rank=1;                 /* Dataspace rank */
+    hsize_t dim=dim0;           /* Dataspace dimensions */
+    unsigned u;                 /* Local index variable */
+    hbool_t use_gpfs = FALSE;   /* Use GPFS hints */
+    DATATYPE *data_read = NULL;	/* data buffer */
+    DATATYPE *data_orig = NULL; /* expected data buffer */
+    const char *filename;
+    MPI_Comm comm = MPI_COMM_WORLD;
+    MPI_Info info = MPI_INFO_NULL;
+    int mpi_size, mpi_rank;
+    herr_t ret;         	/* Generic return value */
+
+    filename = GetTestParameters();
+    if (VERBOSE_MED)
+	printf("Collective chunked dataset read test on file %s\n", filename);
+
+    /* Retrieve MPI parameters */
+    MPI_Comm_size(comm,&mpi_size);
+    MPI_Comm_rank(comm,&mpi_rank);
+
+    /* Allocate data buffer */
+    data_orig = (DATATYPE *)HDmalloc((size_t)dim*sizeof(DATATYPE));
+    VRFY((data_orig != NULL), "data_origin1 malloc succeeded");
+    data_read = (DATATYPE *)HDmalloc((size_t)dim*sizeof(DATATYPE));
+    VRFY((data_read != NULL), "data_array1 malloc succeeded");
+
+    /* Initialize data buffers */
+    for(u=0; u<dim;u++)
+        data_orig[u]=u;
+
+    /* Process zero creates the file with a compressed, chunked dataset */
+    if(mpi_rank==0) {
+        hsize_t chunk_dim;           /* Chunk dimensions */
+
+        /* Create the file */
+        fid = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+        VRFY((fid > 0), "H5Fcreate succeeded");
+
+        /* Create property list for chunking and compression */
+        dcpl = H5Pcreate(H5P_DATASET_CREATE);
+        VRFY((dcpl > 0), "H5Pcreate succeeded");
+
+        ret=H5Pset_layout(dcpl, H5D_CHUNKED);
+        VRFY((ret >= 0), "H5Pset_layout succeeded");
+
+        /* Use eight chunks */
+        chunk_dim=dim/8;
+        ret=H5Pset_chunk(dcpl, rank, &chunk_dim);
+        VRFY((ret >= 0), "H5Pset_chunk succeeded");
+
+        ret=H5Pset_deflate(dcpl, 9);
+        VRFY((ret >= 0), "H5Pset_deflate succeeded");
+
+        /* Create dataspace */
+        dataspace = H5Screate_simple(rank, &dim, NULL);
+        VRFY((dataspace > 0), "H5Screate_simple succeeded");
+
+        /* Create dataset */
+        dataset = H5Dcreate(fid, "compressed_data", H5T_NATIVE_INT, dataspace, dcpl); 
+        VRFY((dataset > 0), "H5Screate_simple succeeded");
+
+        /* Write compressed data */
+        ret=H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data_orig);
+        VRFY((ret >= 0), "H5Dwrite succeeded");
+
+        /* Close objects */
+        ret=H5Pclose(dcpl);
+        VRFY((ret >= 0), "H5Pclose succeeded");
+        ret=H5Sclose(dataspace);
+        VRFY((ret >= 0), "H5Sclose succeeded");
+        ret=H5Dclose(dataset);
+        VRFY((ret >= 0), "H5Dclose succeeded");
+        ret=H5Fclose(fid);
+        VRFY((ret >= 0), "H5Fclose succeeded");
+    }
+
+    /* Wait for file to be created */
+    MPI_Barrier(comm);
+
+    /* -------------------
+     * OPEN AN HDF5 FILE
+     * -------------------*/
+
+    /* setup file access template */
+    acc_tpl = create_faccess_plist(comm, info, facc_type, use_gpfs);
+    VRFY((acc_tpl >= 0), "");
+
+    /* open the file collectively */
+    fid=H5Fopen(filename,H5F_ACC_RDWR,acc_tpl);
+    VRFY((fid > 0), "H5Fopen succeeded");
+
+    /* Release file-access template */
+    ret=H5Pclose(acc_tpl);
+    VRFY((ret >= 0), "H5Pclose succeeded");
+
+
+    /* Open dataset with compressed chunks */
+    dataset = H5Dopen(fid, "compressed_data");
+    VRFY((dataset > 0), "H5Dopen succeeded");
+
+    /* Try reading & writing data */
+    if(dataset>0) {
+        /* Create dataset transfer property list */
+        xfer_plist = H5Pcreate(H5P_DATASET_XFER);
+        VRFY((xfer_plist > 0), "H5Pcreate succeeded");
+
+        ret=H5Pset_dxpl_mpio(xfer_plist, H5FD_MPIO_COLLECTIVE);
+        VRFY((ret >= 0), "H5Pset_dxpl_mpio succeeded");
+
+        /* Try reading the data */
+        ret=H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, xfer_plist, data_read);
+        VRFY((ret >= 0), "H5Pset_dxpl_mpio succeeded");
+
+        /* Verify data read */
+        for(u=0; u<dim; u++)
+            if(data_orig[u]!=data_read[u]) {
+                printf("Line #%d: written!=retrieved: data_orig[%u]=%d, data_read[%u]=%d\n",__LINE__,
+                    (unsigned)u,data_orig[u],(unsigned)u,data_read[u]);
+                nerrors++;
+            }
+
+        /* Writing to the compressed, chunked dataset in parallel should fail */
+        H5E_BEGIN_TRY {
+            ret=H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, xfer_plist, data_read);
+        } H5E_END_TRY;
+        VRFY((ret < 0), "H5Dwrite failed");
+
+        ret=H5Pclose(xfer_plist);
+        VRFY((ret >= 0), "H5Pclose succeeded");
+        ret=H5Dclose(dataset);
+        VRFY((ret >= 0), "H5Dclose succeeded");
+    } /* end if */
+
+    ret=H5Fclose(fid);
+    VRFY((ret >= 0), "H5Fclose succeeded");
+
+    /* release data buffers */
+    if (data_read) HDfree(data_read);
+    if (data_orig) HDfree(data_orig);
+}
+#endif /* H5_HAVE_FILTER_DEFLATE */
+
