@@ -12,8 +12,6 @@
  * access to either file, you may request a copy from hdfhelp@ncsa.uiuc.edu. *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-/* $Id: t_mdset.c,v 1.25.2.6 2004/01/23 00:10:36 acheng Exp $ */
-
 #include "testphdf5.h"
 
 #define DIM  2
@@ -22,6 +20,8 @@
 #define GROUP_DEPTH 128
 enum obj_type { is_group, is_dset };
  
+
+int get_size(void);
 void write_dataset(hid_t, hid_t, hid_t);
 int  read_dataset(hid_t, hid_t, hid_t);
 void create_group_recursive(hid_t, hid_t, hid_t, int);
@@ -29,31 +29,86 @@ void recursive_read_group(hid_t, hid_t, hid_t, int);
 void group_dataset_read(hid_t fid, int mpi_rank, int m);
 void write_attribute(hid_t, int, int);
 int  read_attribute(hid_t, int, int);
-int  check_value(DATATYPE *, DATATYPE *);
-void get_slab(hssize_t[], hsize_t[], hsize_t[], hsize_t[]);
+int  check_value(DATATYPE *, DATATYPE *, int);
+void get_slab(hsize_t[], hsize_t[], hsize_t[], hsize_t[], int);
+
+
+/*
+ * The size value computed by this function is used extensively in 
+ * configuring tests for the current number of processes.  
+ *
+ * This function was created as part of an effort to allow the 
+ * test functions in this file to run on an arbitrary number of
+ * processors.
+ *                                       JRM - 8/11/04
+ */
+
+int get_size(void)
+{
+    int mpi_rank;
+    int mpi_size;
+    int size = SIZE;
+  
+    MPI_Comm_rank (MPI_COMM_WORLD, &mpi_rank); /* needed for VRFY */
+    MPI_Comm_size (MPI_COMM_WORLD, &mpi_size);
+
+    if ( mpi_size > size ) {
+
+        if ( (mpi_size % 2) == 0 ) {
+
+            size = mpi_size;
+
+        } else {
+
+            size = mpi_size + 1;
+        }
+    }
+
+    VRFY((mpi_size <= size), "mpi_size <= size");
+    VRFY(((size % 2) == 0), "size isn't even");
+
+    return(size);
+
+} /* get_size() */
 
 /*
  * Example of using PHDF5 to create ndatasets datasets.  Each process write
  * a slab of array to the file.
+ *
+ * Changes:	Updated function to use a dynamically calculated size,
+ *		instead of the old SIZE #define.  This should allow it
+ *		to function with an arbitrary number of processors.
+ *
+ *						JRM - 8/11/04
  */
-void multiple_dset_write(char *filename, int ndatasets)
+void multiple_dset_write(void)
 {
-    int i, j, n, mpi_size, mpi_rank;
+    int i, j, n, mpi_size, mpi_rank, size;
     hid_t iof, plist, dataset, memspace, filespace;
     hid_t dcpl;                         /* Dataset creation property list */
     hbool_t use_gpfs = FALSE;           /* Use GPFS hints */
-    hssize_t chunk_origin [DIM];
+    hsize_t chunk_origin [DIM];
     hsize_t chunk_dims [DIM], file_dims [DIM];
     hsize_t count[DIM]={1,1};
-    double outme [SIZE][SIZE];
+    double * outme = NULL;
     double fill=1.0;                    /* Fill value */
     char dname [100];
     herr_t ret;
+    const H5Ptest_param_t *pt;
+    char	*filename;
+    int		ndatasets;
+
+    pt = GetTestParameters();
+    filename = pt->name;
+    ndatasets = pt->count;
+
+    size = get_size();
 
     MPI_Comm_rank (MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size (MPI_COMM_WORLD, &mpi_size);
 
-    VRFY((mpi_size <= SIZE), "mpi_size <= SIZE");
+    outme = HDmalloc((size_t)(size * size * sizeof(double)));
+    VRFY((outme != NULL), "HDmalloc succeeded for outme");
 
     plist = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     VRFY((plist>=0), "create_faccess_plist succeeded");
@@ -63,7 +118,7 @@ void multiple_dset_write(char *filename, int ndatasets)
     VRFY((ret>=0), "H5Pclose succeeded");
 
     /* decide the hyperslab according to process number. */
-    get_slab(chunk_origin, chunk_dims, count, file_dims);  
+    get_slab(chunk_origin, chunk_dims, count, file_dims, size);  
 
     memspace = H5Screate_simple (DIM, chunk_dims, NULL);
     filespace = H5Screate_simple (DIM, file_dims, NULL);
@@ -83,9 +138,9 @@ void multiple_dset_write(char *filename, int ndatasets)
 	VRFY((dataset > 0), dname); 
 
 	/* calculate data to write */
-	for (i = 0; i < SIZE; i++)
-	    for (j = 0; j < SIZE; j++)
-	        outme [i][j] = n*1000 + mpi_rank;
+	for (i = 0; i < size; i++)
+	    for (j = 0; j < size; j++)
+                outme [(i * size) + j] = n*1000 + mpi_rank;
 
 	H5Dwrite (dataset, H5T_NATIVE_DOUBLE, memspace, filespace, H5P_DEFAULT, outme);
 
@@ -102,24 +157,49 @@ void multiple_dset_write(char *filename, int ndatasets)
     H5Sclose (memspace);
     H5Pclose (dcpl);
     H5Fclose (iof);
+
+    HDfree(outme);
 }
 
+
 /* Example of using PHDF5 to create, write, and read compact dataset.  
+ *
+ * Changes:	Updated function to use a dynamically calculated size,
+ *		instead of the old SIZE #define.  This should allow it
+ *		to function with an arbitrary number of processors.
+ *
+ *						JRM - 8/11/04
  */
-void compact_dataset(char *filename)
+void compact_dataset(void)
 {
-    int i, j, mpi_size, mpi_rank, err_num=0;
+    int i, j, mpi_size, mpi_rank, size, err_num=0;
     hbool_t use_gpfs = FALSE;
     hid_t iof, plist, dcpl, dxpl, dataset, filespace;
-    hsize_t file_dims [DIM]={SIZE,SIZE};
-    double outme [SIZE][SIZE], inme[SIZE][SIZE];
+    hsize_t file_dims [DIM];
+    double * outme;
+    double * inme;
     char dname[]="dataset";
     herr_t ret;
+    const char *filename;
+
+    size = get_size();
+
+    for ( i = 0; i < DIM; i++ )
+    {
+        file_dims[i] = size;
+    }
                                 
     MPI_Comm_rank (MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size (MPI_COMM_WORLD, &mpi_size);
 
-    VRFY((mpi_size <= SIZE), "mpi_size <= SIZE");
+    outme = HDmalloc((size_t)(size * size * sizeof(double)));
+    VRFY((outme != NULL), "HDmalloc succeeded for outme");
+
+    inme = HDmalloc((size_t)(size * size * sizeof(double)));
+    VRFY((outme != NULL), "HDmalloc succeeded for inme");
+
+    filename = GetTestParameters();
+    VRFY((mpi_size <= size), "mpi_size <= size");
 
     plist = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     iof = H5Fcreate (filename, H5F_ACC_TRUNC, H5P_DEFAULT, plist);
@@ -145,9 +225,9 @@ void compact_dataset(char *filename)
     VRFY((ret >= 0), "H5Pcreate xfer succeeded");
 
     /* Recalculate data to write.  Each process writes the same data. */
-    for (i = 0; i < SIZE; i++)
-         for (j = 0; j < SIZE; j++)
-              outme [i][j] = (i+j)*1000;
+    for (i = 0; i < size; i++)
+         for (j = 0; j < size; j++)
+              outme[(i * size) + j] = (i+j)*1000;
 
     ret=H5Dwrite (dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, dxpl, outme);
     VRFY((ret >= 0), "H5Dwrite succeeded");
@@ -176,24 +256,34 @@ void compact_dataset(char *filename)
     VRFY((ret >= 0), "H5Dread succeeded");
 
     /* Verify data value */
-    for (i = 0; i < SIZE; i++)
-        for (j = 0; j < SIZE; j++)
-            if(inme[i][j] != outme[i][j])
+    for (i = 0; i < size; i++)
+        for (j = 0; j < size; j++)
+            if ( inme[(i * size) + j] != outme[(i * size) + j])
                 if(err_num++ < MAX_ERR_REPORT || VERBOSE_MED)
-                    printf("Dataset Verify failed at [%d][%d]: expect %f, got %f\n", i, j, outme[i][j], inme[i][j]); 
+                    printf("Dataset Verify failed at [%d][%d]: expect %f, got %f\n", i, j, outme[(i * size) + j], inme[(i * size) + j]); 
                                                             
     H5Pclose(plist);
     H5Pclose(dxpl);
     H5Dclose(dataset);
     H5Fclose(iof);
+    HDfree(inme);
+    HDfree(outme);
 }
 
 /* Example of using PHDF5 to create "large" datasets.  (>2GB, >4GB, >8GB)
  * Actual data is _not_ written to these datasets.  Dataspaces are exact
  * sizes (2GB, 4GB, etc.), but the metadata for the file pushes the file over
  * the boundary of interest.
+ *
+ * Changes:	Removed the assert that mpi_size <= the SIZE #define.
+ *		As best I can tell, this assert isn't needed here,
+ *		and in any case, the SIZE #define is being removed 
+ *		in an update of the functions in this file to run
+ *		with an arbitrary number of processes.
+ *
+ *                                         JRM - 8/11/04
  */
-void big_dataset(const char *filename)
+void big_dataset(void)
 {
     int mpi_size, mpi_rank;     /* MPI info */
     hbool_t use_gpfs = FALSE;   /* Don't use GPFS stuff for this test */
@@ -205,11 +295,15 @@ void big_dataset(const char *filename)
     char dname[]="dataset";     /* Name of dataset */
     MPI_Offset file_size;       /* Size of file on disk */
     herr_t ret;                 /* Generic return value */
+    const char *filename;
                                 
     MPI_Comm_rank (MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size (MPI_COMM_WORLD, &mpi_size);
 
-    VRFY((mpi_size <= SIZE), "mpi_size <= SIZE");
+    /* Verify MPI_Offset can handle larger than 2GB sizes */
+    VRFY((sizeof(MPI_Offset)>4), "sizeof(MPI_Offset)>4");
+
+    filename = GetTestParameters();
 
     fapl = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     VRFY((fapl >= 0), "create_faccess_plist succeeded");        
@@ -309,8 +403,18 @@ void big_dataset(const char *filename)
 /* Example of using PHDF5 to read a partial written dataset.   The dataset does
  * not have actual data written to the entire raw data area and relies on the
  * default fill value of zeros to work correctly.
+ *
+ * Changes:	Removed the assert that mpi_size <= the SIZE #define.
+ *		As best I can tell, this assert isn't needed here,
+ *		and in any case, the SIZE #define is being removed 
+ *		in an update of the functions in this file to run
+ *		with an arbitrary number of processes.
+ * 
+ *		Also added code to free dynamically allocated buffers.
+ *
+ *                                         JRM - 8/11/04
  */
-void dataset_fillvalue(const char *filename)
+void dataset_fillvalue(void)
 {
     int mpi_size, mpi_rank;     /* MPI info */
     hbool_t use_gpfs = FALSE;   /* Don't use GPFS stuff for this test */
@@ -323,18 +427,19 @@ void dataset_fillvalue(const char *filename)
         filespace;              /* Dataset's dataspace ID */
     char dname[]="dataset";     /* Name of dataset */
     hsize_t     dset_dims[4] = {0, 6, 7, 8};
-    hssize_t    req_start[4] = {0, 0, 0, 0};
+    hsize_t     req_start[4] = {0, 0, 0, 0};
     hsize_t     req_count[4] = {1, 6, 7, 8};
     hsize_t     dset_size;      /* Dataset size */
     int *rdata, *wdata;         /* Buffers for data to read and write */
     int *twdata, *trdata;        /* Temporary pointer into buffer */
     int acc, i, j, k, l;        /* Local index variables */
     herr_t ret;                 /* Generic return value */
+    const char *filename;
                                 
     MPI_Comm_rank (MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size (MPI_COMM_WORLD, &mpi_size);
 
-    VRFY((mpi_size <= SIZE), "mpi_size <= SIZE");
+    filename = GetTestParameters();
 
     /* Set the dataset dimension to be one row more than number of processes */
     /* and calculate the actual dataset size. */
@@ -479,33 +584,58 @@ void dataset_fillvalue(const char *filename)
     /* Close fapl */
     ret=H5Pclose (fapl);
     VRFY((ret >= 0), "H5Pclose succeeded");
+
+    /* free the buffers */
+    HDfree(rdata);
+    HDfree(wdata);
 }
 
 /* Write multiple groups with a chunked dataset in each group collectively. 
  * These groups and datasets are for testing independent read later.
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *                                              JRM - 8/16/04
  */
-void collective_group_write(char *filename, int ngroups)
+void collective_group_write(void)
 {
-    int mpi_rank, mpi_size;
+    int mpi_rank, mpi_size, size;
     int i, j, m;
     hbool_t use_gpfs = FALSE;
     char gname[64], dname[32];
     hid_t fid, gid, did, plist, dcpl, memspace, filespace;
-    DATATYPE outme[SIZE][SIZE];
-    hssize_t chunk_origin[DIM];
+    DATATYPE * outme = NULL;
+    hsize_t chunk_origin[DIM];
     hsize_t chunk_dims[DIM], file_dims[DIM], count[DIM];
-    const hsize_t chunk_size[2] = {SIZE/2, SIZE/2};  /* Chunk dimensions */
+    hsize_t chunk_size[2];  /* Chunk dimensions - computed shortly */
     herr_t ret1, ret2;
+    const H5Ptest_param_t *pt;
+    char	*filename;
+    int		ngroups;
+
+    pt = GetTestParameters();
+    filename = pt->name;
+    ngroups = pt->count;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
+    size = get_size();
+
+    chunk_size[0] = (hsize_t)(size / 2);
+    chunk_size[1] = (hsize_t)(size / 2);
+
+    outme = HDmalloc((size_t)(size * size * sizeof(DATATYPE)));
+    VRFY((outme != NULL), "HDmalloc succeeded for outme");
+
     plist = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     fid = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, plist);
     H5Pclose(plist);
 
     /* decide the hyperslab according to process number. */
-    get_slab(chunk_origin, chunk_dims, count, file_dims);  
+    get_slab(chunk_origin, chunk_dims, count, file_dims, size);  
 
     /* select hyperslab in memory and file spaces.  These two operations are
      * identical since the datasets are the same. */
@@ -536,9 +666,9 @@ void collective_group_write(char *filename, int ngroups)
         did = H5Dcreate(gid, dname, H5T_NATIVE_INT, filespace, dcpl);
         VRFY((did > 0), dname);
 
-        for(i=0; i < SIZE; i++)
-            for(j=0; j < SIZE; j++)
-     	        outme[i][j] = (i+j)*1000 + mpi_rank;
+        for(i=0; i < size; i++)
+            for(j=0; j < size; j++)
+                outme[(i * size) + j] = (i+j)*1000 + mpi_rank;
 
         H5Dwrite(did, H5T_NATIVE_INT, memspace, filespace, H5P_DEFAULT, 
                  outme);
@@ -558,16 +688,25 @@ void collective_group_write(char *filename, int ngroups)
     H5Sclose(filespace);
     H5Sclose(memspace);
     H5Fclose(fid);
+
+    HDfree(outme);
 }
 
 /* Let two sets of processes open and read different groups and chunked 
  * datasets independently. 
  */
-void independent_group_read(char *filename, int ngroups)
+void independent_group_read(void)
 {
     int      mpi_rank, m;
     hid_t    plist, fid;
     hbool_t  use_gpfs = FALSE;
+    const H5Ptest_param_t *pt;
+    char	*filename;
+    int		ngroups;
+
+    pt = GetTestParameters();
+    filename = pt->name;
+    ngroups = pt->count;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     
@@ -589,16 +728,33 @@ void independent_group_read(char *filename, int ngroups)
     H5Fclose(fid);
 }
 
-/* Open and read datasets and compare data */
+/* Open and read datasets and compare data
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *		Also added code to verify the results of dynamic memory
+ *		allocations, and to free dynamically allocated memeory
+ *		when we are done with it.
+ *
+ *                                              JRM - 8/16/04
+ */
 void group_dataset_read(hid_t fid, int mpi_rank, int m)
 {
-    int      ret, i, j;
+    int      ret, i, j, size;
     char     gname[64], dname[32];
     hid_t    gid, did;
-    DATATYPE *outdata, *indata;
+    DATATYPE *outdata = NULL;
+    DATATYPE *indata = NULL;
 
-    indata = (DATATYPE*)malloc(SIZE*SIZE*sizeof(DATATYPE));
-    outdata = (DATATYPE*)malloc(SIZE*SIZE*sizeof(DATATYPE));
+    size = get_size();
+
+    indata = (DATATYPE*)HDmalloc((size_t)(size * size * sizeof(DATATYPE)));
+    VRFY((indata != NULL), "HDmalloc succeeded for indata");
+
+    outdata = (DATATYPE*)HDmalloc((size_t)(size * size * sizeof(DATATYPE)));
+    VRFY((outdata != NULL), "HDmalloc succeeded for outdata");
     
     /* open every group under root group. */
     sprintf(gname, "group%d", m);
@@ -613,19 +769,20 @@ void group_dataset_read(hid_t fid, int mpi_rank, int m)
     H5Dread(did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, indata);
 
     /* this is the original value */
-    for(i=0; i<SIZE; i++)
-       for(j=0; j<SIZE; j++) { 
-           *outdata = (i+j)*1000 + mpi_rank;
-           outdata++;
+    for(i=0; i<size; i++)
+       for(j=0; j<size; j++) { 
+           outdata[(i * size) + j] = (i+j)*1000 + mpi_rank;
        }
-    outdata -= SIZE*SIZE;
 
     /* compare the original value(outdata) to the value in file(indata).*/
-    ret = check_value(indata, outdata);
+    ret = check_value(indata, outdata, size);
     VRFY((ret==0), "check the data");
 
     H5Dclose(did);
     H5Gclose(gid);
+
+    HDfree(indata);
+    HDfree(outdata);
 }
 
 /*
@@ -653,27 +810,42 @@ void group_dataset_read(hid_t fid, int mpi_rank, int m)
  *      * means the group has dataset(s).
  *      + means the group has attribute(s).
  *      ' means the datasets in the groups have attribute(s).
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *                                              JRM - 8/16/04
  */
-void multiple_group_write(char *filename, int ngroups)
+void multiple_group_write(void)
 {
-    int mpi_rank, mpi_size;
+    int mpi_rank, mpi_size, size;
     int m;
     hbool_t use_gpfs = FALSE;
     char gname[64];
     hid_t fid, gid, plist, memspace, filespace;
-    hssize_t chunk_origin[DIM];
+    hsize_t chunk_origin[DIM];
     hsize_t chunk_dims[DIM], file_dims[DIM], count[DIM];
     herr_t ret;
+    const H5Ptest_param_t *pt;
+    char	*filename;
+    int		ngroups;
+
+    pt = GetTestParameters();
+    filename = pt->name;
+    ngroups = pt->count;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
+    size = get_size();
+
     plist = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     fid = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, plist);
     H5Pclose(plist);
 
     /* decide the hyperslab according to process number. */
-    get_slab(chunk_origin, chunk_dims, count, file_dims);  
+    get_slab(chunk_origin, chunk_dims, count, file_dims, size);  
 
     /* select hyperslab in memory and file spaces.  These two operations are
      * identical since the datasets are the same. */
@@ -729,28 +901,38 @@ void multiple_group_write(char *filename, int ngroups)
 /* 
  * In a group, creates NDATASETS datasets.  Each process writes a hyperslab
  * of a data array to the file.
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *                                              JRM - 8/16/04
  */ 
 void write_dataset(hid_t memspace, hid_t filespace, hid_t gid)
 {
-    int i, j, n;
+    int i, j, n, size;
     int mpi_rank, mpi_size;
     char dname[32];
-    DATATYPE outme[SIZE][SIZE];
+    DATATYPE * outme = NULL;
     hid_t did;
-
   
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
+    size = get_size();
+
+    outme = HDmalloc((size_t)(size * size * sizeof(double)));
+    VRFY((outme != NULL), "HDmalloc succeeded for outme");
+
     for(n=0; n < NDATASET; n++) {
          sprintf(dname, "dataset%d", n);
          did = H5Dcreate(gid, dname, H5T_NATIVE_INT, filespace, 
                          H5P_DEFAULT);
          VRFY((did > 0), dname);
 
-         for(i=0; i < SIZE; i++)
-             for(j=0; j < SIZE; j++)
-     	         outme[i][j] = n*1000 + mpi_rank;
+         for(i=0; i < size; i++)
+             for(j=0; j < size; j++)
+     	         outme[(i * size) + j] = n*1000 + mpi_rank;
 
          H5Dwrite(did, H5T_NATIVE_INT, memspace, filespace, H5P_DEFAULT, 
                   outme);
@@ -760,6 +942,7 @@ void write_dataset(hid_t memspace, hid_t filespace, hid_t gid)
 	 
          H5Dclose(did);
     }
+    HDfree(outme);
 }
 
 /* 
@@ -798,26 +981,41 @@ void create_group_recursive(hid_t memspace, hid_t filespace, hid_t gid,
 /* 
  * This function is to verify the data from multiple group testing.  It opens
  * every dataset in every group and check their correctness.  
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *                                              JRM - 8/11/04
  */
-void multiple_group_read(char *filename, int ngroups)
+void multiple_group_read(void)
 {
-    int      mpi_rank, mpi_size, error_num;
+    int      mpi_rank, mpi_size, error_num, size;
     int      m;
     hbool_t  use_gpfs = FALSE;
     char     gname[64];
     hid_t    plist, fid, gid, memspace, filespace;
-    hssize_t chunk_origin[DIM];
+    hsize_t  chunk_origin[DIM];
     hsize_t  chunk_dims[DIM], file_dims[DIM], count[DIM];
+    const H5Ptest_param_t *pt;
+    char	*filename;
+    int		ngroups;
+
+    pt = GetTestParameters();
+    filename = pt->name;
+    ngroups = pt->count;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
+    size = get_size();
+
     plist = create_faccess_plist(MPI_COMM_WORLD, MPI_INFO_NULL, facc_type, use_gpfs);
     fid = H5Fopen(filename, H5F_ACC_RDONLY, plist);
     H5Pclose(plist);
 
     /* decide hyperslab for each process */
-    get_slab(chunk_origin, chunk_dims, count, file_dims);
+    get_slab(chunk_origin, chunk_dims, count, file_dims, size);
 
     /* select hyperslab for memory and file space */
     memspace  = H5Screate_simple(DIM, file_dims, NULL);
@@ -866,19 +1064,30 @@ void multiple_group_read(char *filename, int ngroups)
 /* 
  * This function opens all the datasets in a certain, checks the data using 
  * dataset_vrfy function.
+ *
+ * Changes:     Updated function to use a dynamically calculated size,
+ *              instead of the old SIZE #define.  This should allow it
+ *              to function with an arbitrary number of processors.
+ *
+ *                                              JRM - 8/11/04
  */
 int read_dataset(hid_t memspace, hid_t filespace, hid_t gid)
 {
-    int i, j, n, mpi_rank, mpi_size, attr_errors=0, vrfy_errors=0;
+    int i, j, n, mpi_rank, mpi_size, size, attr_errors=0, vrfy_errors=0;
     char dname[32];
-    DATATYPE *outdata, *indata;
+    DATATYPE *outdata = NULL, *indata = NULL;
     hid_t did;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
-    indata = (DATATYPE*)malloc(SIZE*SIZE*sizeof(DATATYPE));
-    outdata = (DATATYPE*)malloc(SIZE*SIZE*sizeof(DATATYPE));
+    size = get_size();
+
+    indata = (DATATYPE*)HDmalloc((size_t)(size * size * sizeof(DATATYPE)));
+    VRFY((indata != NULL), "HDmalloc succeeded for indata");
+
+    outdata = (DATATYPE*)HDmalloc((size_t)(size * size * sizeof(DATATYPE)));
+    VRFY((outdata != NULL), "HDmalloc succeeded for outdata");
 
     for(n=0; n<NDATASET; n++) {
         sprintf(dname, "dataset%d", n);
@@ -889,15 +1098,15 @@ int read_dataset(hid_t memspace, hid_t filespace, hid_t gid)
                 indata);
 
         /* this is the original value */
-        for(i=0; i<SIZE; i++)
-	    for(j=0; j<SIZE; j++) { 
+        for(i=0; i<size; i++)
+	    for(j=0; j<size; j++) { 
 	         *outdata = n*1000 + mpi_rank;
                  outdata++;
 	    }
-        outdata -= SIZE*SIZE;
+        outdata -= size * size;
 
         /* compare the original value(outdata) to the value in file(indata).*/
-        vrfy_errors = check_value(indata, outdata);
+        vrfy_errors = check_value(indata, outdata, size);
 
         /* check attribute.*/  
         if( (attr_errors = read_attribute(did, is_dset, n))>0 )
@@ -906,8 +1115,8 @@ int read_dataset(hid_t memspace, hid_t filespace, hid_t gid)
         H5Dclose(did);
     }
 
-    free(indata);
-    free(outdata);
+    HDfree(indata);
+    HDfree(outdata);
 
     return vrfy_errors;
 }
@@ -1011,26 +1220,33 @@ int read_attribute(hid_t obj_id, int this_type, int num)
 }
 
 /* This functions compares the original data with the read-in data for its 
- * hyperslab part only by process ID. */
-int check_value(DATATYPE *indata, DATATYPE *outdata) 
+ * hyperslab part only by process ID.
+ *
+ * Changes:	Modified function to use a passed in size parameter
+ *		instead of the old SIZE #define.  This should let us
+ *		run with an arbitrary number of processes.
+ *
+ *					JRM - 8/16/04
+ */
+int check_value(DATATYPE *indata, DATATYPE *outdata, int size) 
 {
     int mpi_rank, mpi_size, err_num=0;
     hsize_t i, j;
-    hssize_t chunk_origin[DIM];
+    hsize_t chunk_origin[DIM];
     hsize_t  chunk_dims[DIM], count[DIM];
 
     MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
-    
-    get_slab(chunk_origin, chunk_dims, count, NULL);
 
-    indata += chunk_origin[0]*SIZE;
-    outdata += chunk_origin[0]*SIZE;
+    get_slab(chunk_origin, chunk_dims, count, NULL, size);
+
+    indata += chunk_origin[0]*size;
+    outdata += chunk_origin[0]*size;
     for(i=chunk_origin[0]; i<(chunk_origin[0]+chunk_dims[0]); i++)
          for(j=chunk_origin[1]; j<(chunk_origin[1]+chunk_dims[1]); j++) {
               if( *indata != *outdata )
 	          if(err_num++ < MAX_ERR_REPORT || VERBOSE_MED)
-		      printf("Dataset Verify failed at [%ld][%ld](row %ld, col%ld): expect %d, got %d\n", (long)i, (long)j, (long)i, (long)j, *outdata, *indata); 
+		      printf("Dataset Verify failed at [%lu][%lu](row %lu, col%lu): expect %d, got %d\n", (unsigned long)i, (unsigned long)j, (unsigned long)i, (unsigned long)j, *outdata, *indata); 
 	 }
     if(err_num > MAX_ERR_REPORT && !VERBOSE_MED)
         printf("[more errors ...]\n");
@@ -1039,9 +1255,20 @@ int check_value(DATATYPE *indata, DATATYPE *outdata)
     return err_num;
 }
 
-/* Decide the portion of data chunk in dataset by process ID. */
-void get_slab(hssize_t chunk_origin[], hsize_t chunk_dims[], hsize_t count[],
-              hsize_t file_dims[])
+/* Decide the portion of data chunk in dataset by process ID.
+ *
+ * Changes:	Modified function to use a passed in size parameter
+ *		instead of the old SIZE #define.  This should let us
+ *		run with an arbitrary number of processes.
+ *
+ *					JRM - 8/11/04
+ */
+
+void get_slab(hsize_t chunk_origin[], 
+              hsize_t chunk_dims[], 
+              hsize_t count[],
+              hsize_t file_dims[],
+              int size)
 {
     int mpi_rank, mpi_size;
     
@@ -1049,19 +1276,307 @@ void get_slab(hssize_t chunk_origin[], hsize_t chunk_dims[], hsize_t count[],
     MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
 
     if(chunk_origin != NULL) {
-        chunk_origin[0] = mpi_rank * (SIZE/mpi_size);
+        chunk_origin[0] = mpi_rank * (size/mpi_size);
         chunk_origin[1] = 0;
     }
     if(chunk_dims != NULL) {
-        chunk_dims[0]   = SIZE/mpi_size;
-        chunk_dims[1]   = SIZE;
+        chunk_dims[0]   = size/mpi_size;
+        chunk_dims[1]   = size;
     }
     if(file_dims != NULL) 
-        file_dims[0] = file_dims[1] = SIZE;
+        file_dims[0] = file_dims[1] = size;
     if(count != NULL) 
         count[0] = count[1] = 1;
 }
 
+/* 
+ * This function is based on bug demonstration code provided by Thomas
+ * Guignon (thomas.guignon@ifp.fr), and is intended to verify the 
+ * correctness of my fix for that bug.
+ *
+ * In essence, the bug appeared when at least one process attempted to 
+ * write a point selection -- for which collective I/O is not supported, 
+ * and at least one other attempted to write some other type of selection 
+ * for which collective I/O is supported.  
+ *
+ * Since the processes did not compare notes before performing the I/O, 
+ * some would attempt collective I/O while others performed independent
+ * I/O.  A hang resulted.
+ *
+ * This function reproduces this situation.  At present the test hangs
+ * on failure.
+ *                                         JRM - 9/13/04
+ *
+ * Changes:	None.
+ */
+
+#define N 4
+
+void io_mode_confusion(void)
+{
+    /*
+     * HDF5 APIs definitions
+     */
+    
+    const int   rank = 1;
+    const char *dataset_name = "IntArray";
+
+    hid_t       file_id, dset_id;         /* file and dataset identifiers */
+    hid_t       filespace, memspace;      /* file and memory dataspace */
+                                          /* identifiers               */
+    hsize_t     dimsf[1];                 /* dataset dimensions */
+    int         data[N] = {1};            /* pointer to data buffer to write */ 
+    hsize_t     coord[N] = {0L,1L,2L,3L};
+    hsize_t     start[1];
+    hsize_t     stride[1];
+    hsize_t     count[1];
+    hsize_t     block[1];
+    hid_t       plist_id;                 /* property list identifier */
+    herr_t      status;
+
+
+    /*
+     * MPI variables
+     */
+
+    int mpi_size, mpi_rank;
+
+
+    /*
+     * test bed related variables
+     */
+
+    const char *	fcn_name = "io_mode_confusion";
+    const hbool_t	verbose = FALSE;
+    const H5Ptest_param_t *	pt;
+    char *		filename;
+
+
+    pt = GetTestParameters();
+    filename = pt->name;
+
+    MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
+    MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
+
+    /*
+     * Set up file access property list with parallel I/O access
+     */
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Setting up property list.\n", 
+                  mpi_rank, fcn_name);
+
+    plist_id = H5Pcreate(H5P_FILE_ACCESS);
+
+    VRFY((plist_id != -1), "H5Pcreate() failed");
+
+    status = H5Pset_fapl_mpio(plist_id, MPI_COMM_WORLD, MPI_INFO_NULL);
+
+    VRFY(( status >= 0 ), "H5Pset_fapl_mpio() failed");
+
+
+    /*
+     * Create a new file collectively and release property list identifier.
+     */
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Creating new file.\n", mpi_rank, fcn_name);
+
+    file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, plist_id);
+
+    VRFY(( file_id >= 0 ), "H5Fcreate() failed");
+
+    status = H5Pclose(plist_id);
+
+    VRFY(( status >= 0 ), "H5Pclose() failed");
+
+
+    /*
+     * Create the dataspace for the dataset.
+     */
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Creating the dataspace for the dataset.\n",
+                  mpi_rank, fcn_name);
+
+    dimsf[0] = N;
+
+    filespace = H5Screate_simple(rank, dimsf, NULL);
+
+    VRFY(( filespace >= 0 ), "H5Screate_simple() failed.");
+
+
+    /*
+     * Create the dataset with default properties and close filespace.
+     */
+
+    if ( verbose )
+        HDfprintf(stdout,
+                  "%0d:%s: Creating the dataset, and closing filespace.\n",
+                  mpi_rank, fcn_name);
+
+    dset_id = H5Dcreate(file_id, dataset_name, H5T_NATIVE_INT, filespace,
+                        H5P_DEFAULT);
+
+    VRFY(( dset_id >= 0 ), "H5Dcreate() failed");
+
+    status = H5Sclose(filespace);
+
+    VRFY(( status >= 0 ), "H5Sclose() failed");
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling H5Screate_simple().\n", 
+                  mpi_rank, fcn_name);
+
+    memspace = H5Screate_simple(rank, dimsf, NULL);
+
+    VRFY(( memspace >= 0 ), "H5Screate_simple() failed.");
+
+
+    if( mpi_rank == 0 ) {
+     
+        if ( verbose )
+            HDfprintf(stdout, "%0d:%s: Calling H5Sselect_all(memspace).\n",
+                      mpi_rank, fcn_name);
+
+        status = H5Sselect_all(memspace);
+
+        VRFY(( status >= 0 ), "H5Sselect_all() failed");
+
+    } else {
+
+        if ( verbose )
+            HDfprintf(stdout, "%0d:%s: Calling H5Sselect_none(memspace).\n",
+                      mpi_rank, fcn_name);
+
+        status = H5Sselect_none(memspace);
+
+        VRFY(( status >= 0 ), "H5Sselect_none() failed");
+     
+    }
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling MPI_Barrier().\n", 
+                  mpi_rank, fcn_name);
+
+    MPI_Barrier(MPI_COMM_WORLD);
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling H5Dget_space().\n", 
+                  mpi_rank, fcn_name);
+
+    filespace = H5Dget_space(dset_id);
+
+    VRFY(( filespace >= 0 ), "H5Dget_space() failed");
+
+
+    start[0] = 0L;
+    stride[0] = 1;
+    count[0] = 1;
+    block[0] = N;
+
+    if ( mpi_rank == 0 ) {
+
+        /* select all */
+
+        if ( verbose )
+             HDfprintf(stdout,
+                       "%0d:%s: Calling H5Sselect_elements() -- set up hang?\n",
+                       mpi_rank, fcn_name);
+
+        status = H5Sselect_elements(filespace, H5S_SELECT_SET, N, 
+                                    (const hsize_t **)&coord);
+
+        VRFY(( status >= 0 ), "H5Sselect_elements() failed");
+
+    } else {
+
+        /* select nothing */
+
+        if ( verbose )
+            HDfprintf(stdout, "%0d:%s: Calling H5Sselect_none().\n",
+                      mpi_rank, fcn_name);
+
+        status = H5Sselect_none(filespace);
+
+        VRFY(( status >= 0 ), "H5Sselect_none() failed");
+
+    }
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling MPI_Barrier().\n", 
+                  mpi_rank, fcn_name);
+
+    MPI_Barrier(MPI_COMM_WORLD);
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling H5Pcreate().\n", mpi_rank, fcn_name);
+
+    plist_id = H5Pcreate(H5P_DATASET_XFER);
+
+    VRFY(( plist_id != -1 ), "H5Pcreate() failed");
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling H5Pset_dxpl_mpio().\n", 
+                  mpi_rank, fcn_name);
+
+    status = H5Pset_dxpl_mpio(plist_id, H5FD_MPIO_COLLECTIVE);
+
+    VRFY(( status >= 0 ), "H5Pset_dxpl_mpio() failed");
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Calling H5Dwrite() -- hang here?.\n",
+                  mpi_rank, fcn_name);
+
+    status = H5Dwrite(dset_id, H5T_NATIVE_INT, memspace, filespace,
+                      plist_id, data);
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Returned from H5Dwrite(), status=%d.\n", 
+                  mpi_rank, fcn_name, status);
+
+    VRFY(( status >= 0 ), "H5Dwrite() failed");
+
+    /*
+     * Close/release resources.
+     */
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Cleaning up from test.\n", 
+                  mpi_rank, fcn_name);
+
+    status = H5Dclose(dset_id);
+    VRFY(( status >= 0 ), "H5Dclose() failed");
+
+    status = H5Sclose(filespace);
+    VRFY(( status >= 0 ), "H5Dclose() failed");
+
+    status = H5Sclose(memspace);
+    VRFY(( status >= 0 ), "H5Sclose() failed");
+
+    status = H5Pclose(plist_id);
+    VRFY(( status >= 0 ), "H5Pclose() failed");
+
+    status = H5Fclose(file_id);
+    VRFY(( status >= 0 ), "H5Fclose() failed");
+
+
+    if ( verbose )
+        HDfprintf(stdout, "%0d:%s: Done.\n", mpi_rank);
+
+    return;
+
+} /* io_mode_confusion() */
+
+#undef N
+
 /*=============================================================================
  *                         End of t_mdset.c
  *===========================================================================*/
