@@ -43,10 +43,6 @@ int main(int argc, char *argv[])
   (void) setvbuf(stderr, (char *) NULL, _IOLBF, 0);
   (void) setvbuf(stdout, (char *) NULL, _IOLBF, 0);
 
-#if defined __MWERKS__
-    argc = ccommand(&argv);
-#endif
-
  /*
   * validate the number of command line arguments
   */
@@ -158,7 +154,7 @@ int main(int argc, char *argv[])
       break;
 
       case 14:    /* read data size */
-        if (getInputSize(in, HDstrtol(argv[i], NULL, BASE_10)) == -1)
+        if (getInputSize(in, (int)HDstrtol(argv[i], NULL, BASE_10)) == -1)
         {
           (void) fprintf(stderr, err8, argv[i]);
 					goto err;
@@ -384,7 +380,6 @@ readIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
       
@@ -418,7 +413,6 @@ readIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
   
@@ -451,7 +445,6 @@ readIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
 
@@ -486,7 +479,6 @@ readIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
   	  break;
 #endif /* ifndef WIN32 */ 
@@ -551,7 +543,6 @@ readUIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
     
@@ -584,7 +575,6 @@ readUIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
   
@@ -617,7 +607,6 @@ readUIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
 
@@ -652,7 +641,6 @@ readUIntegerData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
 #endif /* ifndef WIN32 */
@@ -716,7 +704,6 @@ readFloatData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
 
@@ -754,7 +741,6 @@ readFloatData(FILE **strm, struct Input *in)
         default:
           (void) fprintf(stderr, err2);
           return (-1);
-        break;
       }
     break;
 
@@ -1796,12 +1782,10 @@ getCompressionParameter(struct Input *in, FILE** strm)
       }
       in->compressionParam = ival;
       return (0);
-    break;
 
     default:
       (void) fprintf(stderr, err3);
       return (-1);
-    break;      
   }
 }
 
@@ -1842,25 +1826,25 @@ setDefaultValues(struct Input *in, int count)
   HDstrcpy(in->path.group[0], temp);
 
   in->outputArchitecture = 0; /* NATIVE */
-  in->outputByteOrder = 0;  /* BE    */
+  in->outputByteOrder = -1;  /* use default    */
   in->compressionType = 0;  /* GZIP   */
   for (i=0; i<NUM_KEYS; i++)
     in->configOptionVector[i] = 0;
 }
 
 hid_t 
-createOutputDataType(struct Input in)
+createOutputDataType(struct Input *in)
 {
   hid_t new_type = (-1);
   const char *err1 = "Invalid value for output class.\n";
   
-  switch (in.outputClass)
+  switch (in->outputClass)
   {
     case 0: 
-      switch (in.outputArchitecture)
+      switch (in->outputArchitecture)
       {
         case 0: /* NATIVE */
-          switch(in.outputSize)
+          switch(in->outputSize)
           {
             case 8:
               new_type = H5Tcopy (H5T_NATIVE_CHAR);
@@ -1878,14 +1862,27 @@ createOutputDataType(struct Input in)
               new_type = H5Tcopy (H5T_NATIVE_LLONG);
             break;
           }
+              switch(in->outputByteOrder)
+              {
+                case -1: /* default */
+                break;
+                case 0:
+                  H5Tset_order (new_type,H5T_ORDER_BE);
+                break;
+
+                case 1:
+                  H5Tset_order (new_type,H5T_ORDER_LE);
+                break;
+              }                  
         break;
           
         case 1: /* STD */
-          switch(in.outputSize)
+          switch(in->outputSize)
           {
             case 8:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_STD_I8BE);
                 break;
@@ -1897,8 +1894,9 @@ createOutputDataType(struct Input in)
             break;
 
             case 16:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_STD_I16BE);
                 break;
@@ -1910,8 +1908,9 @@ createOutputDataType(struct Input in)
             break;
 
             case 32:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_STD_I32BE);
                 break;
@@ -1923,8 +1922,9 @@ createOutputDataType(struct Input in)
             break;
 
             case 64:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_STD_I64BE);
                 break;
@@ -1941,10 +1941,10 @@ createOutputDataType(struct Input in)
         break;
 
     case 1:
-      switch (in.outputArchitecture)
+      switch (in->outputArchitecture)
       {
         case 0:
-          switch(in.outputSize)
+          switch(in->outputSize)
           {
             case 32:
               new_type = H5Tcopy (H5T_NATIVE_FLOAT);                                
@@ -1954,6 +1954,18 @@ createOutputDataType(struct Input in)
               new_type = H5Tcopy (H5T_NATIVE_DOUBLE);
             break;
           }
+              switch(in->outputByteOrder)
+              {
+                case -1: /* DEFAULT */
+                break;
+                case 0:
+                  H5Tset_order (new_type,H5T_ORDER_BE);
+                break;
+
+                case 1:
+                  H5Tset_order (new_type,H5T_ORDER_LE);
+                break;
+              }                  
         break;
   
         case 1:
@@ -1961,11 +1973,12 @@ createOutputDataType(struct Input in)
         break;
 
         case 2:
-          switch(in.outputSize)
+          switch(in->outputSize)
           {
             case 32:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_IEEE_F32BE);
                 break;
@@ -1977,8 +1990,9 @@ createOutputDataType(struct Input in)
             break;
 
             case 64:
-              switch(in.outputByteOrder)
+              switch(in->outputByteOrder)
               {
+                case -1:
                 case 0:
                   new_type = H5Tcopy (H5T_IEEE_F64BE);
                 break;
@@ -1995,10 +2009,10 @@ createOutputDataType(struct Input in)
       break;
 
     case 2:
-        switch (in.outputArchitecture)
+        switch (in->outputArchitecture)
         {
           case 0:
-            switch(in.outputSize)
+            switch(in->outputSize)
             {
               case 8:
                 new_type = H5Tcopy (H5T_NATIVE_UCHAR);
@@ -2016,14 +2030,27 @@ createOutputDataType(struct Input in)
                 new_type = H5Tcopy (H5T_NATIVE_ULLONG);
               break;
             }
+              switch(in->outputByteOrder)
+              {
+                case -1: /* Default */
+                break;
+                case 0:
+                  H5Tset_order (new_type,H5T_ORDER_BE);
+                break;
+
+                case 1:
+                  H5Tset_order (new_type,H5T_ORDER_LE);
+                break;
+              }                  
             break;
           
           case 1:
-            switch(in.outputSize)
+            switch(in->outputSize)
             {
               case 8:
-                switch(in.outputByteOrder)
+                switch(in->outputByteOrder)
                 {
+                  case -1:
                   case 0:
                     new_type = H5Tcopy (H5T_STD_U8BE);
                   break;
@@ -2035,8 +2062,9 @@ createOutputDataType(struct Input in)
                 break;
 
               case 16:
-                switch(in.outputByteOrder)
+                switch(in->outputByteOrder)
                 {
+                  case -1:
                   case 0:
                     new_type = H5Tcopy (H5T_STD_U16BE);
                   break;
@@ -2048,8 +2076,9 @@ createOutputDataType(struct Input in)
                 break;
 
               case 32:
-                switch(in.outputByteOrder)
+                switch(in->outputByteOrder)
                 {
+                  case -1:
                   case 0:
                     new_type = H5Tcopy (H5T_STD_U32BE);
                   break;
@@ -2061,8 +2090,9 @@ createOutputDataType(struct Input in)
                 break;
 
               case 64:
-                switch(in.outputByteOrder)
+                switch(in->outputByteOrder)
                 {
+                  case -1:
                   case 0:
                     new_type = H5Tcopy (H5T_STD_U64BE);
                   break;
@@ -2091,16 +2121,16 @@ createOutputDataType(struct Input in)
 }
 
 hid_t 
-createInputDataType(struct Input in)
+createInputDataType(struct Input *in)
 {
   hid_t new_type = (-1);
   const char *err1 = "Invalid value for input class.\n";
 
-  switch (in.inputClass)
+  switch (in->inputClass)
   {
     case 0:
     case 4:
-        switch(in.inputSize)
+        switch(in->inputSize)
         {
           case 8:
               new_type = H5Tcopy (H5T_NATIVE_CHAR);
@@ -2123,7 +2153,7 @@ createInputDataType(struct Input in)
     case 1:
     case 2:
     case 3:
-        switch(in.inputSize)
+        switch(in->inputSize)
         {
           case 32:
               new_type = H5Tcopy (H5T_NATIVE_FLOAT);
@@ -2140,7 +2170,7 @@ createInputDataType(struct Input in)
 
     case 6:
     case 7:
-        switch(in.inputSize)
+        switch(in->inputSize)
         {
           case 8:
               new_type = H5Tcopy (H5T_NATIVE_UCHAR);
@@ -2177,8 +2207,6 @@ process(struct Options *opt)
   hid_t intype, outtype;
   hid_t proplist;  
   hsize_t numOfElements = 1; 
-  H5E_auto_t func;
-  void *client_data;
   int j,k;
 
   const char *err1 = "Error creating HDF output file: %s.\n";
@@ -2187,18 +2215,16 @@ process(struct Options *opt)
   const char *err4 = "Error in creating or opening external file.\n";  
   const char *err5 = "Error in creating the output data set. Dataset with the same name may exist at the specified path\n";
   const char *err6 = "Error in writing the output data set.\n";
-  H5Eget_auto(&func, &client_data);
-    
-    /* disable error reporting */
-    H5Eset_auto(NULL, NULL);
-  if ((file_id = H5Fopen(opt->outfile, H5F_ACC_RDWR, H5P_DEFAULT)) < 0)
-  if ((file_id = H5Fcreate(opt->outfile, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT)) == FAIL) 
-  {
-    (void) fprintf(stderr, err1, opt->outfile);
-    return (-1);
-  }
-    /*enable error reporting */
-    H5Eset_auto(func, client_data);
+
+  H5E_BEGIN_TRY {
+    if ((file_id = H5Fopen(opt->outfile, H5F_ACC_RDWR, H5P_DEFAULT)) < 0) {
+      if ((file_id = H5Fcreate(opt->outfile, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT)) == FAIL) 
+      {
+        (void) fprintf(stderr, err1, opt->outfile);
+        return (-1);
+      }
+    }
+  } H5E_END_TRY;
     
   for (k = 0; k < opt->fcount; k++)
   {
@@ -2221,11 +2247,8 @@ process(struct Options *opt)
     for (j=0; j<in->rank;j++)
       numOfElements *= in->sizeOfDimension[j];
 
-  /* store error reporting parameters */
-    H5Eget_auto(&func, &client_data);
-    
     /* disable error reporting */
-    H5Eset_auto(NULL, NULL);
+    H5E_BEGIN_TRY {
     
     /* create parent groups */
     if (in->path.count > 1)
@@ -2253,11 +2276,11 @@ process(struct Options *opt)
     }
 
     /*enable error reporting */
-    H5Eset_auto(func, client_data);
+    } H5E_END_TRY;
     
     /*create data type */
-    intype = createInputDataType(*in);
-    outtype = createOutputDataType(*in);    
+    intype = createInputDataType(in);
+    outtype = createOutputDataType(in);    
     
     /* create property list */
     proplist = H5Pcreate (H5P_DATASET_CREATE);      
@@ -2297,10 +2320,9 @@ process(struct Options *opt)
     {
       dataspace = H5Screate_simple(in->rank, in->sizeOfDimension, NULL);
     }
-    H5Eget_auto(&func, &client_data);
     
     /* disable error reporting */
-    H5Eset_auto(NULL, NULL); 
+    H5E_BEGIN_TRY {
     /* create data set */
     if ((dataset = H5Dcreate(handle, in->path.group[j], outtype, dataspace, proplist)) < 0) 
     {
@@ -2312,7 +2334,7 @@ process(struct Options *opt)
     }
  
     /*enable error reporting */
-    H5Eset_auto(func, client_data);
+    } H5E_END_TRY;
 
      /* write dataset */
     if (H5Dwrite(dataset, intype, H5S_ALL, H5S_ALL, H5P_DEFAULT, (VOIDP)in->data) < 0) 
