@@ -43,9 +43,6 @@
 
 #define ALIGN(A,Z)		((((A) + (Z) - 1) / (Z)) * (Z))
 
-#define START_OF_DATA		0x0001
-#define END_OF_DATA		0x0002
-
 /* global variables */
 int         indent;
 int         compound_data;
@@ -53,6 +50,41 @@ FILE       *rawdatastream;	/* should initialize to stdout but gcc moans about it
 
 /* module-scoped variables */
 static int  h5tools_init_g;     /* if h5tools lib has been initialized */
+#ifdef H5_HAVE_PARALLEL
+static int  h5tools_mpi_init_g; /* if MPI_Init() has been called */
+#endif /* H5_HAVE_PARALLEL */
+
+/* Names of VFDs */
+static const char *drivernames[]={
+    "sec2",
+    "family",
+    "split",
+    "multi",
+#ifdef H5_HAVE_STREAM
+    "stream",
+#endif	/* H5_HAVE_STREAM */
+#ifdef H5_HAVE_PARALLEL
+    "mpio",
+    "mpiposix"
+#endif /* H5_HAVE_PARALLEL */
+};
+
+/* This enum should match the entries in the above drivers_list since they
+ * are indexes into the drivers_list array. */
+enum {
+    SEC2_IDX = 0
+   ,FAMILY_IDX
+   ,SPLIT_IDX
+   ,MULTI_IDX
+#ifdef H5_HAVE_STREAM
+   ,STREAM_IDX
+#endif	/* H5_HAVE_STREAM */
+#ifdef H5_HAVE_PARALLEL
+   ,MPIO_IDX
+   ,MPIPOSIX_IDX
+#endif /* H5_HAVE_PARALLEL */
+} driver_idx;
+#define NUM_DRIVERS     (sizeof(drivernames) / sizeof(drivernames[0]))
 
 /*-------------------------------------------------------------------------
  * Audience:    Public
@@ -107,10 +139,126 @@ h5tools_close(void)
 		rawdatastream = NULL;
 	}
 
+        /* Shut down the library */
+        H5close();
+
+#ifdef H5_HAVE_PARALLEL
+        /* Check if we need to shut down MPI */
+        if(h5tools_mpi_init_g) {
+            MPI_Finalize();
+            h5tools_mpi_init_g=0;
+        } /* end if */
+#endif
+
 	h5tools_init_g = 0;
     }
 }
 
+/*-------------------------------------------------------------------------
+ * Audience:    Private
+ * Chapter:     H5Tools Library
+ * Purpose:	Get a FAPL for a driver
+ * Description:
+ *      Get a FAPL for a given VFL driver name.
+ * Return:
+ *      None
+ * Programmer:
+ *      Quincey Koziol, 2004-02-04
+ * Modifications:
+ *-------------------------------------------------------------------------
+ */
+static hid_t
+h5tools_get_fapl(const char *driver, unsigned *drivernum,
+#ifdef H5_HAVE_PARALLEL
+int argc, const char *argv[]
+#else /* H5_HAVE_PARALLEL */
+int UNUSED argc, const char UNUSED *argv[]
+#endif /* H5_HAVE_PARALLEL */
+)
+{
+    hid_t               fapl = H5P_DEFAULT;
+
+    /* Determine which driver the user wants to open the file with. Try
+     * that driver. If it can't open it, then fail. */
+    if (!strcmp(driver, drivernames[SEC2_IDX])) {
+        if(drivernum)
+            *drivernum = SEC2_IDX;
+    } else if (!strcmp(driver, drivernames[FAMILY_IDX])) {
+        /* FAMILY Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            /* Set member size to be 0 to indicate the current first member size 
+             * is the member size. 
+             */ 
+            H5Pset_fapl_family(fapl, (hsize_t)0, H5P_DEFAULT);
+
+            if(drivernum)
+                *drivernum = FAMILY_IDX;
+        } /* end if */
+    } else if (!strcmp(driver, drivernames[SPLIT_IDX])) {
+        /* SPLIT Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            H5Pset_fapl_split(fapl, "-m.h5", H5P_DEFAULT, "-r.h5", H5P_DEFAULT);
+
+            if(drivernum)
+                *drivernum = SPLIT_IDX;
+        } /* end if */
+    } else if (!strcmp(driver, drivernames[MULTI_IDX])) {
+        /* MULTI Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            H5Pset_fapl_multi(fapl, NULL, NULL, NULL, NULL, TRUE);
+
+            if(drivernum)
+                *drivernum = MULTI_IDX;
+        } /* end if */
+#ifdef H5_HAVE_STREAM
+    } else if (!strcmp(driver, drivernames[STREAM_IDX])) {
+        /* STREAM Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            H5Pset_fapl_stream(fapl, NULL);
+
+            if(drivernum)
+                *drivernum = STREAM_IDX;
+        } /* end if */
+#endif	/* H5_HAVE_STREAM */
+#ifdef H5_HAVE_PARALLEL
+    } else if (!strcmp(driver, drivernames[MPIO_IDX])) {
+        /* MPI-I/O Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            H5Pset_fapl_mpio(fapl, MPI_COMM_WORLD, MPI_INFO_NULL);
+
+            /* Initialize the MPI library, if it wasn't already */
+            if(!h5tools_mpi_init_g) {
+                MPI_Init(&argc, (char ***)&argv);
+
+                h5tools_mpi_init_g=1;
+            } /* end if */
+
+            if(drivernum)
+                *drivernum = MPIO_IDX;
+        } /* end if */
+    } else if (!strcmp(driver, drivernames[MPIPOSIX_IDX])) {
+        /* MPI-I/O Driver */
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS))>=0) {
+            H5Pset_fapl_mpiposix(fapl, MPI_COMM_WORLD, TRUE);
+
+            /* Initialize the MPI library, if it wasn't already */
+            if(!h5tools_mpi_init_g) {
+                MPI_Init(&argc, (char ***)&argv);
+
+                h5tools_mpi_init_g=1;
+            } /* end if */
+
+            if(drivernum)
+                *drivernum = MPIPOSIX_IDX;
+        } /* end if */
+#endif /* H5_HAVE_PARALLEL */
+    } else {
+        fapl=(-1);
+    }
+
+    return(fapl);
+}
+
 /*-------------------------------------------------------------------------
  * Audience:    Public
  * Chapter:     H5Tools Library
@@ -168,145 +316,49 @@ h5tools_close(void)
  */
 hid_t
 h5tools_fopen(const char *fname, const char *driver, char *drivername,
-              size_t drivername_size)
+              size_t drivername_size, int argc, const char *argv[])
 {
-    static struct d_list {
-        const char     *name;
-        hid_t		fapl;
-    } drivers_list[] = {
-        { "sec2", FAIL }
-        ,{ "family", FAIL }
-        ,{ "split", FAIL }
-        ,{ "multi", FAIL }
-#ifdef H5_HAVE_STREAM
-        ,{ "stream", FAIL }
-#endif	/* H5_HAVE_STREAM */
-    };
-    /* This enum should match the entries in the above drivers_list since they
-     * are indexes into the drivers_list array. */
-    enum {
-        SEC2_IDX = 0
-       ,FAMILY_IDX
-       ,SPLIT_IDX
-       ,MULTI_IDX
-#ifdef H5_HAVE_STREAM
-       ,STREAM_IDX
-#endif	/* H5_HAVE_STREAM */
-    };
-#define NUM_DRIVERS     (sizeof(drivers_list) / sizeof(struct d_list))
-
-    static int          initialized = 0;
-    register unsigned   drivernum;
-    hid_t               fid = FAIL;
-#ifndef VERSION12
-    hid_t               fapl = H5P_DEFAULT;
-#endif  /* !VERSION12 */
-
-    if (!initialized) {
-        /* Build a list of file access property lists which we should try
-         * when opening the file.  Eventually we'd like some way for the
-         * user to augment/replace this list interactively. */
-        ++initialized;
-
-        /* SEC2 Driver */
-        drivers_list[SEC2_IDX].fapl = H5P_DEFAULT;
-        
-#ifndef VERSION12
-        /* FAMILY Driver */
-        drivers_list[FAMILY_IDX].fapl = fapl = H5Pcreate(H5P_FILE_ACCESS);
-        H5Pset_fapl_family(fapl, (hsize_t)0, H5P_DEFAULT);
-
-        /* SPLIT Driver */
-        drivers_list[SPLIT_IDX].fapl = fapl = H5Pcreate(H5P_FILE_ACCESS);
-        H5Pset_fapl_split(fapl, "-m.h5", H5P_DEFAULT, "-r.h5", H5P_DEFAULT);
-
-        /* MULTI Driver */
-        drivers_list[MULTI_IDX].fapl = fapl = H5Pcreate(H5P_FILE_ACCESS);
-        H5Pset_fapl_multi(fapl, NULL, NULL, NULL, NULL, TRUE);
-
-#ifdef H5_HAVE_STREAM
-        /* STREAM Driver */
-        drivers_list[STREAM_IDX].fapl = fapl = H5Pcreate(H5P_FILE_ACCESS);
-        H5Pset_fapl_stream(fapl, NULL);
-#endif	/* H5_HAVE_STREAM */
-#endif	/* !VERSION12 */
-    }
+    unsigned    drivernum;
+    hid_t       fid = FAIL;
+    hid_t       fapl = H5P_DEFAULT;
 
     if (driver && *driver) {
-        /* Determine which driver the user wants to open the file with. Try
-         * that driver. If it can't open it, then fail. */
-        if (!strcmp(driver, drivers_list[SEC2_IDX].name)) {
-            H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[SEC2_IDX].fapl);
-            } H5E_END_TRY;
-
-            if (fid == FAIL)
-                goto done;
-
-            drivernum = SEC2_IDX;
-        } else if (!strcmp(driver, drivers_list[FAMILY_IDX].name)) {
-            H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[FAMILY_IDX].fapl);
-            } H5E_END_TRY;
-
-            if (fid == FAIL)
-                goto done;
-
-            drivernum = FAMILY_IDX;
-        } else if (!strcmp(driver, drivers_list[SPLIT_IDX].name)) {
-            H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[SPLIT_IDX].fapl);
-            } H5E_END_TRY;
-
-            if (fid == FAIL)
-                goto done;
-
-            drivernum = SPLIT_IDX;
-        } else if (!strcmp(driver, drivers_list[MULTI_IDX].name)) {
-            H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[MULTI_IDX].fapl);
-            } H5E_END_TRY;
-
-            if (fid == FAIL)
-                goto done;
-
-            drivernum = MULTI_IDX;
-#ifdef H5_HAVE_STREAM
-        } else if (!strcmp(driver, drivers_list[STREAM_IDX].name)) {
-            H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[STREAM_IDX].fapl);
-            } H5E_END_TRY;
-
-            if (fid == FAIL)
-                goto done;
-
-            drivernum = STREAM_IDX;
-#endif	/* H5_HAVE_STREAM */
-        } else {
+        /* Get the correct FAPL for the given driver */
+        if((fapl=h5tools_get_fapl(driver,&drivernum,argc,argv))<0)
             goto done;
-        }
+
+        H5E_BEGIN_TRY {
+            fid = H5Fopen(fname, H5F_ACC_RDONLY, fapl);
+        } H5E_END_TRY;
+
+        if (fid == FAIL)
+            goto done;
+
     } else {
         /* Try to open the file using each of the drivers */
         for (drivernum = 0; drivernum < NUM_DRIVERS; drivernum++) {
+            /* Get the correct FAPL for the given driver */
+            if((fapl=h5tools_get_fapl(drivernames[drivernum],NULL,argc,argv))<0)
+                goto done;
+
             H5E_BEGIN_TRY {
-                fid = H5Fopen(fname, H5F_ACC_RDONLY,
-                              drivers_list[drivernum].fapl);
+                fid = H5Fopen(fname, H5F_ACC_RDONLY, fapl);
             } H5E_END_TRY;
 
             if (fid != FAIL)
                 break;
+            else {
+                /* Close the FAPL */
+                H5Pclose(fapl);
+                fapl=H5P_DEFAULT;
+            } /* end else */
         }
     }
 
     /* Save the driver name */
     if (drivername && drivername_size) {
         if (fid != FAIL) {
-            strncpy(drivername, drivers_list[drivernum].name, drivername_size);
+            strncpy(drivername, drivernames[drivernum], drivername_size);
             drivername[drivername_size - 1] = '\0';
         } else {
             /*no file opened*/
@@ -315,6 +367,8 @@ h5tools_fopen(const char *fname, const char *driver, char *drivername,
     }
 
 done:
+    if(fapl!=H5P_DEFAULT)
+        H5Pclose(fapl);
     return fid;
 }
 
@@ -361,6 +415,10 @@ h5tools_ncols(const char *s)
  *      Robb Matzke, 1999-09-29
  *	If a new prefix is printed then the current element number is set back
  *	to zero.
+ *      pvn, 2004-07-08
+ * Added support for printing array indices: 
+ *  the indentation is printed before the prefix (printed one indentation
+ *  level before) 
  *-------------------------------------------------------------------------
  */
 static void
@@ -368,6 +426,7 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
                       h5tools_context_t *ctx, hsize_t elmtno, int secnum)
 {
     h5tools_str_t prefix;
+    h5tools_str_t str; /*temporary for indentation */
     size_t templength = 0;
     int i, indentlevel = 0;
 	
@@ -375,17 +434,18 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
 	return;
  
     memset(&prefix, 0, sizeof(h5tools_str_t));
+    memset(&str, 0, sizeof(h5tools_str_t));
 
     /* Terminate previous line, if any */
     if (ctx->cur_column) {
-	fputs(OPT(info->line_suf, ""), stream);
-        putc('\n', stream);
-	fputs(OPT(info->line_sep, ""), stream);
+	    fputs(OPT(info->line_suf, ""), stream);
+     putc('\n', stream);
+	    fputs(OPT(info->line_sep, ""), stream);
     }
 
     /* Calculate new prefix */
     h5tools_str_prefix(&prefix, info, elmtno, ctx->ndims, ctx->p_min_idx,
-                       ctx->p_max_idx);
+                       ctx->p_max_idx,ctx);
 
     /* Write new prefix to output */
     if (ctx->indent_level >= 0) {
@@ -400,6 +460,14 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
 	indentlevel = ctx->default_indent_level;
     }
 
+    /* when printing array indices, print the indentation before the prefix
+       the prefix is printed one indentation level before */
+    if (info->pindex) {
+     for (i = 0; i < indentlevel-1; i++){
+      fputs(h5tools_str_fmt(&str, 0, info->line_indent), stream);
+     }
+    }
+
     if (elmtno == 0 && secnum == 0 && info->line_1st)
         fputs(h5tools_str_fmt(&prefix, 0, info->line_1st), stream);
     else if (secnum && info->line_cont)
@@ -410,8 +478,15 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
     templength = h5tools_str_len(&prefix);
 
     for (i = 0; i < indentlevel; i++){
+        /*we already made the indent for the array indices case */
+     if (!info->pindex) {
         fputs(h5tools_str_fmt(&prefix, 0, info->line_indent), stream);
         templength += h5tools_str_len(&prefix);
+     }
+     else {
+      /*we cannot count the prefix for the array indices case */
+      templength += h5tools_str_len(&str);
+     }
     }
 
     ctx->cur_column = ctx->prev_prefix_len = templength;
@@ -420,6 +495,7 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
 
     /* Free string */
     h5tools_str_close(&prefix);
+    h5tools_str_close(&str);
 }
 
 /*-------------------------------------------------------------------------
@@ -452,7 +528,7 @@ h5tools_simple_prefix(FILE *stream, const h5dump_t *info,
  *      handled correctly.
  *-------------------------------------------------------------------------
  */
-static void
+void
 h5tools_dump_simple_data(FILE *stream, const h5dump_t *info, hid_t container,
                          h5tools_context_t *ctx/*in,out*/, unsigned flags,
                          hsize_t nelmts, hid_t type, void *_mem)
@@ -485,7 +561,7 @@ h5tools_dump_simple_data(FILE *stream, const h5dump_t *info, hid_t container,
     for (i = 0; i < nelmts; i++, ctx->cur_elmt++, elmt_counter++) {
         /* Render the element */
         h5tools_str_reset(&buffer);
-            h5tools_str_sprint(&buffer, info, container, type, mem + i * size, ctx);
+        h5tools_str_sprint(&buffer, info, container, type, mem + i * size, ctx);
 
         if (i + 1 < nelmts || (flags & END_OF_DATA) == 0)
             h5tools_str_append(&buffer, "%s", OPT(info->elmt_suf1, ","));
@@ -621,8 +697,8 @@ h5tools_dump_simple_subset(FILE *stream, const h5dump_t *info, hid_t dset,
 {
     herr_t              ret;                    /*the value to return   */
     hid_t		f_space;		/*file data space	*/
-    hsize_t		i;                      /*counters		*/
-    hssize_t		zero = 0;               /*vector of zeros	*/
+    int			i;                      /*counters		*/
+    hsize_t		zero = 0;               /*vector of zeros	*/
     unsigned int	flags;			/*buffer extent flags	*/
     hsize_t		total_size[H5S_MAX_RANK];/*total size of dataset*/
 
@@ -660,7 +736,7 @@ h5tools_dump_simple_subset(FILE *stream, const h5dump_t *info, hid_t dset,
 
     /* assume entire data space to be printed */
     if (ctx.ndims > 0)
-        for (i = 0; i < (hsize_t)ctx.ndims; i++)
+        for (i = 0; i < ctx.ndims; i++)
             ctx.p_min_idx[i] = 0;
 
     H5Sget_simple_extent_dims(f_space, total_size, NULL);
@@ -670,14 +746,17 @@ h5tools_dump_simple_subset(FILE *stream, const h5dump_t *info, hid_t dset,
     count = sset->count[ctx.ndims - 1];
     sset->count[ctx.ndims - 1] = 1;
 
+    if(ctx.ndims>0)
+     init_acc_pos(&ctx,total_size);
+
     for (; count > 0; sset->start[ctx.ndims - 1] += sset->stride[ctx.ndims - 1],
                       count--) {
         /* calculate the potential number of elements we're going to print */
         H5Sselect_hyperslab(f_space, H5S_SELECT_SET, 
-        (hssize_t*)sset->start, 
-        (hsize_t*)sset->stride,
-        (hsize_t*)sset->count, 
-        (hsize_t*)sset->block);
+            sset->start, 
+            sset->stride,
+            sset->count, 
+            sset->block);
         sm_nelmts = H5Sget_select_npoints(f_space);
 
         /*
@@ -735,7 +814,7 @@ h5tools_dump_simple_subset(FILE *stream, const h5dump_t *info, hid_t dset,
         if (count == 1)
             flags |= END_OF_DATA;
 
-        for (i = 0; i < (hsize_t)ctx.ndims; i++) {
+        for (i = 0; i < ctx.ndims; i++) {
             ctx.p_max_idx[i] = ctx.p_min_idx[i] + MIN(total_size[i], sm_size[i]);
         }
 
@@ -783,9 +862,10 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
                          hid_t p_type, int indentlevel)
 {
     hid_t		f_space;		/*file data space	*/
-    hsize_t		elmtno, i;		/*counters		*/
+    hsize_t		elmtno;			/*counter		*/
+    int			i;			/*counter		*/
     int			carry;			/*counter carry value	*/
-    hssize_t		zero[8];		/*vector of zeros	*/
+    hsize_t		zero[8];		/*vector of zeros	*/
     unsigned int	flags;			/*buffer extent flags	*/
     hsize_t		total_size[H5S_MAX_RANK];/*total size of dataset*/
 
@@ -802,7 +882,7 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
     hid_t		sm_space;		/*stripmine data space	*/
     
     /* Hyperslab info */
-    hssize_t		hs_offset[H5S_MAX_RANK];/*starting offset	*/
+    hsize_t		hs_offset[H5S_MAX_RANK];/*starting offset	*/
     hsize_t		hs_size[H5S_MAX_RANK];	/*size this pass	*/
     hsize_t		hs_nelmts;		/*elements in request	*/
 
@@ -831,18 +911,21 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
 
     /* Assume entire data space to be printed */
     if (ctx.ndims > 0)
-        for (i = 0; i < (hsize_t)ctx.ndims; i++)
+        for (i = 0; i < ctx.ndims; i++)
             ctx.p_min_idx[i] = 0;
 
     H5Sget_simple_extent_dims(f_space, total_size, NULL);
-    ctx.size_last_dim = (total_size[ctx.ndims - 1]);
 
     /* calculate the number of elements we're going to print */
     p_nelmts = 1;
 
-    if (ctx.ndims > 0)
-        for (i = 0, p_nelmts = 1; i < (hsize_t)ctx.ndims; i++)
+    if (ctx.ndims > 0) {
+        for (i = 0; i < ctx.ndims; i++)
             p_nelmts *= total_size[i];
+        ctx.size_last_dim = (total_size[ctx.ndims - 1]);
+    } /* end if */
+    else
+        ctx.size_last_dim = 0;
  
     if (p_nelmts == 0) {
         /* nothing to print */
@@ -874,6 +957,9 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
     sm_nelmts = sm_nbytes / p_type_nbytes;
     sm_space = H5Screate_simple(1, &sm_nelmts, NULL);
 
+    if(ctx.ndims>0)
+     init_acc_pos(&ctx,total_size);
+
     /* The stripmine loop */
     memset(hs_offset, 0, sizeof hs_offset);
     memset(zero, 0, sizeof zero);
@@ -881,7 +967,7 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
     for (elmtno = 0; elmtno < p_nelmts; elmtno += hs_nelmts) {
         /* Calculate the hyperslab size */
         if (ctx.ndims > 0) {
-            for (i = 0, hs_nelmts = 1; i < (hsize_t)ctx.ndims; i++) {
+            for (i = 0, hs_nelmts = 1; i < ctx.ndims; i++) {
                 hs_size[i] = MIN(total_size[i] - hs_offset[i], sm_size[i]);
                 ctx.p_max_idx[i] = ctx.p_min_idx[i] + hs_size[i];
                 hs_nelmts *= hs_size[i];
@@ -920,7 +1006,7 @@ h5tools_dump_simple_dset(FILE *stream, const h5dump_t *info, hid_t dset,
             ctx.p_min_idx[i - 1] = ctx.p_max_idx[i - 1];
             hs_offset[i - 1] += hs_size[i - 1];
 
-            if (hs_offset[i - 1] == (hssize_t)total_size[i - 1])
+            if (hs_offset[i - 1] == total_size[i - 1])
                 hs_offset[i - 1] = 0;
             else
                 carry = 0;
@@ -965,7 +1051,7 @@ static int
 h5tools_dump_simple_mem(FILE *stream, const h5dump_t *info, hid_t obj_id,
                         hid_t type, hid_t space, void *mem, int indentlevel)
 {
-    hsize_t		i;			/*counters		*/
+    int			i;			/*counters		*/
     hsize_t		nelmts;			/*total selected elmts	*/
     h5tools_context_t	ctx;			/*printing context	*/
 
@@ -984,18 +1070,25 @@ h5tools_dump_simple_mem(FILE *stream, const h5dump_t *info, hid_t obj_id,
     ctx.need_prefix = 1;
     
     /* Assume entire data space to be printed */
-    for (i = 0; i < (hsize_t)ctx.ndims; i++)
+    for (i = 0; i < ctx.ndims; i++)
         ctx.p_min_idx[i] = 0;
 
     H5Sget_simple_extent_dims(space, ctx.p_max_idx, NULL);
 
-    for (i = 0, nelmts = 1; ctx.ndims != 0 && i < (hsize_t)ctx.ndims; i++)
+    for (i = 0, nelmts = 1; ctx.ndims != 0 && i < ctx.ndims; i++)
         nelmts *= ctx.p_max_idx[i] - ctx.p_min_idx[i];
     
     if (nelmts == 0)
         return SUCCEED; /*nothing to print*/
-    assert(ctx.p_max_idx[ctx.ndims - 1]==(hsize_t)((int)ctx.p_max_idx[ctx.ndims - 1]));
-    ctx.size_last_dim = (int)(ctx.p_max_idx[ctx.ndims - 1]);
+    if(ctx.ndims>0) {
+        assert(ctx.p_max_idx[ctx.ndims - 1]==(hsize_t)((int)ctx.p_max_idx[ctx.ndims - 1]));
+        ctx.size_last_dim = (int)(ctx.p_max_idx[ctx.ndims - 1]);
+    } /* end if */
+    else
+        ctx.size_last_dim = 0;
+
+    if(ctx.ndims>0)
+     init_acc_pos(&ctx,ctx.p_max_idx);
 
     /* Print it */
     h5tools_dump_simple_data(stream, info, obj_id, &ctx,
@@ -1050,6 +1143,7 @@ h5tools_dump_dset(FILE *stream, const h5dump_t *info, hid_t dset, hid_t _p_type,
     hid_t     f_space;
     hid_t     p_type = _p_type;
     hid_t     f_type;
+    H5S_class_t space_type;
     int       status = FAIL;
     h5dump_t  info_dflt;
 
@@ -1068,7 +1162,7 @@ h5tools_dump_dset(FILE *stream, const h5dump_t *info, hid_t dset, hid_t _p_type,
         if (info->raw)
             p_type = H5Tcopy(f_type);
         else
-            p_type = H5Tget_native_type(f_type,H5T_DIR_DEFAULT);
+            p_type = h5tools_get_native_type(f_type);
 
         H5Tclose(f_type);
 
@@ -1079,15 +1173,18 @@ h5tools_dump_dset(FILE *stream, const h5dump_t *info, hid_t dset, hid_t _p_type,
     /* Check the data space */
     f_space = H5Dget_space(dset);
 
+    space_type = H5Sget_simple_extent_type(f_space);
+
     /* Print the data */
-    if (H5Sis_simple(f_space) > 0) {
+    if (space_type == H5S_SIMPLE || space_type == H5S_SCALAR) {
         if (!sset)
             status = h5tools_dump_simple_dset(rawdatastream, info, dset, p_type,
                                               indentlevel);
         else
             status = h5tools_dump_simple_subset(rawdatastream, info, dset, p_type,
                                                 sset, indentlevel);
-    }
+    } else /* space is H5S_NULL */
+        status = SUCCEED;
 
     /* Close the dataspace */
     H5Sclose(f_space);
@@ -1139,3 +1236,32 @@ h5tools_dump_mem(FILE *stream, const h5dump_t *info, hid_t obj_id, hid_t type,
     return h5tools_dump_simple_mem(stream, info, obj_id, type, space, mem,
                                    indentlevel);
 }
+
+/*-------------------------------------------------------------------------
+ * Function:    init_acc_pos
+ *
+ * Purpose:     initialize accumulator and matrix position
+ *
+ * Return:      void
+ *
+ * Programmer:  pvn
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+void init_acc_pos(h5tools_context_t	*ctx, hsize_t *dims)
+{
+ int i;
+
+ assert(ctx->ndims);
+ 
+ ctx->acc[ctx->ndims-1]=1;
+ for(i=(ctx->ndims-2); i>=0; i--)
+ {
+  ctx->acc[i]=ctx->acc[i+1] * dims[i+1];
+ }
+ for ( i = 0; i < ctx->ndims; i++)
+  ctx->pos[i]=0;
+}
+
