@@ -64,13 +64,8 @@ error_msg(const char *progname, const char *fmt, ...)
 
     va_start(ap, fmt);
     HDfflush(stdout);
-#ifdef WIN32
-    HDfprintf(stdout, "%s error: ", progname);
-    HDvfprintf(stdout, fmt, ap);
-#else
     HDfprintf(stderr, "%s error: ", progname);
     HDvfprintf(stderr, fmt, ap);
-#endif
     
     va_end(ap);
 }
